/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#if !NO_ENCODER
using System;

namespace Iced.Intel.EncoderInternal {
	static class OpCodeHandlers {
		public static readonly OpCodeHandler[] Handlers;

		public static uint[] GetData() =>
			new uint[DecoderConstants.NumberOfCodeValues * 3] {
				(uint)EncFlags1.Legacy | (uint)Code.INVALID, 0, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm8_r8 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm16_r16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm32_r32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm64_r64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_r8_rm8 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_r16_rm16 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_r32_rm32 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_r64_rm64 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_AL_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_AX_imm16 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_EAX_imm32 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_RAX_imm32 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_ES | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.ES,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_ES | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.ES,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_ES | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.ES,
				(uint)EncFlags1.Legacy | (uint)Code.Popd_ES | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.ES,
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm8_r8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm16_r16 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm32_r32 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm64_r64 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_r8_rm8 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_r16_rm16 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_r32_rm32 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_r64_rm64 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_AL_imm8 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_AX_imm16 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_EAX_imm32 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_RAX_imm32 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_CS | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.CS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_CS | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.CS,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_CS | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.CS,
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm8_r8 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm16_r16 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm32_r32 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm64_r64 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_r8_rm8 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_r16_rm16 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_r32_rm32 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_r64_rm64 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_AL_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_AX_imm16 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_EAX_imm32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_RAX_imm32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_SS | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.SS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_SS | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.SS,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_SS | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.SS,
				(uint)EncFlags1.Legacy | (uint)Code.Popd_SS | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.SS,
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm8_r8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm16_r16 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm32_r32 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm64_r64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_r8_rm8 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_r16_rm16 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_r32_rm32 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_r64_rm64 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_AL_imm8 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_AX_imm16 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_EAX_imm32 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_RAX_imm32 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_DS | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.DS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_DS | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.DS,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_DS | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.DS,
				(uint)EncFlags1.Legacy | (uint)Code.Popd_DS | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.DS,
				(uint)EncFlags1.Legacy | (uint)Code.And_rm8_r8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm16_r16 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm32_r32 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm64_r64 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_r8_rm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_r16_rm16 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_r32_rm32 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_r64_rm64 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_AL_imm8 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_AX_imm16 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_EAX_imm32 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_RAX_imm32 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Daa | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm8_r8 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm16_r16 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm32_r32 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm64_r64 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_r8_rm8 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_r16_rm16 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_r32_rm32 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_r64_rm64 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_AL_imm8 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_AX_imm16 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_EAX_imm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_RAX_imm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Das | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm8_r8 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm16_r16 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm32_r32 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm64_r64 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_r8_rm8 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_r16_rm16 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_r32_rm32 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_r64_rm64 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_AL_imm8 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_AX_imm16 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_EAX_imm32 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_RAX_imm32 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aaa | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm8_r8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm16_r16 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm32_r32 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm64_r64 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_r8_rm8 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_r16_rm16 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_r32_rm32 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_r64_rm64 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_AL_imm8 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_AX_imm16 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_EAX_imm32 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_RAX_imm32 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aas | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_r16 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_r32 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_r16 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_r32 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd,
				(uint)EncFlags1.Legacy | (uint)Code.Push_r16 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw,
				(uint)EncFlags1.Legacy | (uint)Code.Push_r32 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd,
				(uint)EncFlags1.Legacy | (uint)Code.Push_r64 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.r64_ro,
				(uint)EncFlags1.Legacy | (uint)Code.Pop_r16 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw,
				(uint)EncFlags1.Legacy | (uint)Code.Pop_r32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd,
				(uint)EncFlags1.Legacy | (uint)Code.Pop_r64 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.r64_ro,
				(uint)EncFlags1.Legacy | (uint)Code.Pushaw | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pushad | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Popaw | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Popad | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Bound_r16_m1616 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mw2 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bound_r32_m3232 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Md2 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Arpl_rm16_r16 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Arpl_r32m16_r32 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.RdMw | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsxd_r16_rm16 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsxd_r32_rm32 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsxd_r64_rm32 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Push_imm16 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_imm32 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Id,
				(uint)EncFlags1.Legacy | (uint)Code.Pushq_imm32 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.Id64,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r16_rm16_imm16 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r32_rm32_imm32 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r64_rm64_imm32 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_imm8 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ib16,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_imm8 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ib32,
				(uint)EncFlags1.Legacy | (uint)Code.Pushq_imm8 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.Ib64,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r16_rm16_imm8 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r32_rm32_imm8 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r64_rm64_imm8 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insb_m8_DX | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yb | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insw_m16_DX | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yw | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insd_m32_DX | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yd | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Outsb_DX_m8 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.Xb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Outsw_DX_m16 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.Xw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Outsd_DX_m32 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.Xd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel8_16 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel8_32 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel8_64 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel8_16 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel8_32 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel8_64 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel8_16 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel8_32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel8_64 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel8_16 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel8_32 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel8_64 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel8_16 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel8_32 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel8_64 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel8_16 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel8_32 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel8_64 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel8_16 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel8_32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel8_64 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel8_16 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel8_32 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel8_64 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel8_16 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel8_32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel8_64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel8_16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel8_32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel8_64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel8_16 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel8_32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel8_64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel8_16 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel8_32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel8_64 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel8_16 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel8_32 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel8_64 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel8_16 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel8_32 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel8_64 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel8_16 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel8_32 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel8_64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel8_16 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel8_32 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel8_64 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm8_imm8 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm16_imm16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm32_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm64_imm32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm8_imm8_82 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only1632, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Add_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Or_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adc_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sbb_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.And_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sub_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xor_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm16_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib16 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm32_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib32 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmp_rm64_imm8 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm8_r8 | ((uint)0x84 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm16_r16 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm32_r32 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm64_r64 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_rm8_r8 | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_rm16_r16 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_rm32_r32 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_rm64_r64 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm8_r8 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm16_r16 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm32_r32 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm64_r64 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r8_rm8 | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r16_rm16 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32_rm32 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r64_rm64 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm16_Sreg | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Sw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32m16_Sreg | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Sw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r64m16_Sreg | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Sw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lea_r16_m | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lea_r32_m | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lea_r64_m | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_Sreg_rm16 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Sw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_Sreg_r32m16 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Sw | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_Sreg_r64m16 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Sw | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pop_rm16 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Pop_rm32 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Pop_rm64 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Nopw | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Nopd | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Nopq | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_r16_AX | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw | ((uint)LegacyOpKind.AX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_r32_EAX | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd | ((uint)LegacyOpKind.EAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xchg_r64_RAX | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.r64_ro | ((uint)LegacyOpKind.RAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pause | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cbw | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cwde | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cdqe | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cwd | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cdq | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cqo | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Call_ptr1616 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Aww,
				(uint)EncFlags1.Legacy | (uint)Code.Call_ptr1632 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Adw,
				(uint)EncFlags1.Legacy | (uint)Code.Wait | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pushfw | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pushfd | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pushfq | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Popfw | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Popfd | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Popfq | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sahf | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lahf | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mov_AL_moffs8 | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ob << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_AX_moffs16 | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Ow << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_EAX_moffs32 | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Od << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_RAX_moffs64 | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Oq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_moffs8_AL | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Ob | ((uint)LegacyOpKind.AL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_moffs16_AX | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ow | ((uint)LegacyOpKind.AX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_moffs32_EAX | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Od | ((uint)LegacyOpKind.EAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_moffs64_RAX | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Oq | ((uint)LegacyOpKind.RAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsb_m8_m8 | ((uint)0xA4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yb | ((uint)LegacyOpKind.Xb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsw_m16_m16 | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yw | ((uint)LegacyOpKind.Xw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsd_m32_m32 | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yd | ((uint)LegacyOpKind.Xd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsq_m64_m64 | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yq | ((uint)LegacyOpKind.Xq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpsb_m8_m8 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.Xb | ((uint)LegacyOpKind.Yb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpsw_m16_m16 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.Xw | ((uint)LegacyOpKind.Yw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpsd_m32_m32 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.Xd | ((uint)LegacyOpKind.Yd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpsq_m64_m64 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.Xq | ((uint)LegacyOpKind.Yq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_AL_imm8 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_AX_imm16 | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_EAX_imm32 | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_RAX_imm32 | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Stosb_m8_AL | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yb | ((uint)LegacyOpKind.AL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Stosw_m16_AX | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yw | ((uint)LegacyOpKind.AX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Stosd_m32_EAX | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yd | ((uint)LegacyOpKind.EAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Stosq_m64_RAX | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.Yq | ((uint)LegacyOpKind.RAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lodsb_AL_m8 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Xb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lodsw_AX_m16 | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Xw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lodsd_EAX_m32 | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Xd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lodsq_RAX_m64 | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Rep, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Xq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Scasb_AL_m8 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Yb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Scasw_AX_m16 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Yw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Scasd_EAX_m32 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Yd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Scasq_RAX_m64 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.RepeRepne, (uint)LegacyOpKind.RAX | ((uint)LegacyOpKind.Yq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r8_imm8 | ((uint)0xB0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.r8_rb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r16_imm16 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32_imm32 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r64_imm64 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.r64_ro | ((uint)LegacyOpKind.Iq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm8_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm16_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm32_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm64_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Retnw_imm16 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retnd_imm16 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retnq_imm16 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retnw | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Bnd, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Retnd | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Bnd, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Retnq | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Bnd, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Les_r16_m1616 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Les_r32_m1632 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lds_r16_m1616 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lds_r32_m1632 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm8_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xabort_imm8 | ((uint)0xC6F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Ib,
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm16_imm16 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm32_imm32 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_rm64_imm32 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Xrelease, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xbegin_rel16 | ((uint)0xC7F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Jxw,
				(uint)EncFlags1.Legacy | (uint)Code.Xbegin_rel32 | ((uint)0xC7F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Jxd,
				(uint)EncFlags1.Legacy | (uint)Code.Enterw_imm16_imm8 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Iw | ((uint)LegacyOpKind.Ib21 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enterd_imm16_imm8 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Iw | ((uint)LegacyOpKind.Ib21 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enterq_imm16_imm8 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.Iw | ((uint)LegacyOpKind.Ib21 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Leavew | ((uint)0xC9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Leaved | ((uint)0xC9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Leaveq | ((uint)0xC9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Retfw_imm16 | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retfd_imm16 | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retfq_imm16 | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Iw,
				(uint)EncFlags1.Legacy | (uint)Code.Retfw | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Retfd | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Retfq | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Int3 | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Int_imm8 | ((uint)0xCD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Ib,
				(uint)EncFlags1.Legacy | (uint)Code.Into | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Iretw | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Iretd | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Iretq | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm8_1 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm16_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm32_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm64_1 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Imm1 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm8_CL | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rol_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ror_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcl_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcr_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shl_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shr_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sal_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm16_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm32_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sar_rm64_CL | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aam_imm8 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, (uint)LegacyOpKind.Ib,
				(uint)EncFlags1.Legacy | (uint)Code.Aad_imm8 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, (uint)LegacyOpKind.Ib,
				(uint)EncFlags1.Legacy | (uint)Code.Salc | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xlat_m8 | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.MRBX,
				(uint)EncFlags1.Legacy | (uint)Code.Fadd_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fmul_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fcom_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fcomp_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fsub_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fsubr_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fdiv_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fdivr_m32fp | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fadd_st0_sti | ((uint)0xD8C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fmul_st0_sti | ((uint)0xD8C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcom_st0_sti | ((uint)0xD8D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcomp_st0_sti | ((uint)0xD8D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fsub_st0_sti | ((uint)0xD8E0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fsubr_st0_sti | ((uint)0xD8E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdiv_st0_sti | ((uint)0xD8F0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdivr_st0_sti | ((uint)0xD8F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fld_m32fp | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fst_m32fp | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_m32fp | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf32,
				(uint)EncFlags1.Legacy | (uint)Code.Fldenv_m14byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.M14,
				(uint)EncFlags1.Legacy | (uint)Code.Fldenv_m28byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.M28,
				(uint)EncFlags1.Legacy | (uint)Code.Fldcw_m2byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mw,
				(uint)EncFlags1.Legacy | (uint)Code.Fnstenv_m14byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.M14,
				(uint)EncFlags1.Legacy | (uint)Code.Fstenv_m14byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.M14,
				(uint)EncFlags1.Legacy | (uint)Code.Fnstenv_m28byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.M28,
				(uint)EncFlags1.Legacy | (uint)Code.Fstenv_m28byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.M28,
				(uint)EncFlags1.Legacy | (uint)Code.Fnstcw_m2byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mw,
				(uint)EncFlags1.Legacy | (uint)Code.Fstcw_m2byte | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.Mw,
				(uint)EncFlags1.Legacy | (uint)Code.Fld_st0_sti | ((uint)0xD9C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fxch_st0_sti | ((uint)0xD9C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fnop | ((uint)0xD9D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fstpnce_sti | ((uint)0xD9D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fchs | ((uint)0xD9E0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fabs | ((uint)0xD9E1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Ftst | ((uint)0xD9E4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fxam | ((uint)0xD9E5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fld1 | ((uint)0xD9E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldl2t | ((uint)0xD9E9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldl2e | ((uint)0xD9EA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldpi | ((uint)0xD9EB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldlg2 | ((uint)0xD9EC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldln2 | ((uint)0xD9ED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fldz | ((uint)0xD9EE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.F2xm1 | ((uint)0xD9F0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fyl2x | ((uint)0xD9F1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fptan | ((uint)0xD9F2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fpatan | ((uint)0xD9F3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fxtract | ((uint)0xD9F4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fprem1 | ((uint)0xD9F5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fdecstp | ((uint)0xD9F6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fincstp | ((uint)0xD9F7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fprem | ((uint)0xD9F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fyl2xp1 | ((uint)0xD9F9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fsqrt | ((uint)0xD9FA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fsincos | ((uint)0xD9FB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Frndint | ((uint)0xD9FC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fscale | ((uint)0xD9FD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fsin | ((uint)0xD9FE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fcos | ((uint)0xD9FF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fiadd_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fimul_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Ficom_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Ficomp_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fisub_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fisubr_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fidiv_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fidivr_m32int | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovb_st0_sti | ((uint)0xDAC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmove_st0_sti | ((uint)0xDAC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovbe_st0_sti | ((uint)0xDAD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovu_st0_sti | ((uint)0xDAD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fucompp | ((uint)0xDAE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fild_m32int | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fisttp_m32int | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fist_m32int | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fistp_m32int | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi32,
				(uint)EncFlags1.Legacy | (uint)Code.Fld_m80fp | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf80,
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_m80fp | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf80,
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovnb_st0_sti | ((uint)0xDBC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovne_st0_sti | ((uint)0xDBC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovnbe_st0_sti | ((uint)0xDBD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcmovnu_st0_sti | ((uint)0xDBD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fneni | ((uint)0xDBE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Feni | ((uint)0xDBE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fndisi | ((uint)0xDBE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fdisi | ((uint)0xDBE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fnclex | ((uint)0xDBE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fclex | ((uint)0xDBE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fninit | ((uint)0xDBE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Finit | ((uint)0xDBE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fnsetpm | ((uint)0xDBE4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fsetpm | ((uint)0xDBE4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Frstpm | ((uint)0xDBE5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fucomi_st0_sti | ((uint)0xDBE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcomi_st0_sti | ((uint)0xDBF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fadd_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fmul_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fcom_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fcomp_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fsub_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fsubr_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fdiv_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fdivr_m64fp | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fadd_sti_st0 | ((uint)0xDCC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fmul_sti_st0 | ((uint)0xDCC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcom_st0_sti_DCD0 | ((uint)0xDCD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcomp_st0_sti_DCD8 | ((uint)0xDCD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fsubr_sti_st0 | ((uint)0xDCE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fsub_sti_st0 | ((uint)0xDCE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdivr_sti_st0 | ((uint)0xDCF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdiv_sti_st0 | ((uint)0xDCF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fld_m64fp | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fisttp_m64int | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fst_m64fp | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_m64fp | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mf64,
				(uint)EncFlags1.Legacy | (uint)Code.Frstor_m94byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.M98,
				(uint)EncFlags1.Legacy | (uint)Code.Frstor_m108byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.M108,
				(uint)EncFlags1.Legacy | (uint)Code.Fnsave_m94byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.M98,
				(uint)EncFlags1.Legacy | (uint)Code.Fsave_m94byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.M98,
				(uint)EncFlags1.Legacy | (uint)Code.Fnsave_m108byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.M108,
				(uint)EncFlags1.Legacy | (uint)Code.Fsave_m108byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.M108,
				(uint)EncFlags1.Legacy | (uint)Code.Fnstsw_m2byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mw,
				(uint)EncFlags1.Legacy | (uint)Code.Fstsw_m2byte | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.Mw,
				(uint)EncFlags1.Legacy | (uint)Code.Ffree_sti | ((uint)0xDDC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fxch_st0_sti_DDC8 | ((uint)0xDDC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fst_sti | ((uint)0xDDD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_sti | ((uint)0xDDD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fucom_st0_sti | ((uint)0xDDE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fucomp_st0_sti | ((uint)0xDDE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fiadd_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fimul_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Ficom_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Ficomp_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fisub_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fisubr_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fidiv_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fidivr_m16int | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Faddp_sti_st0 | ((uint)0xDEC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fmulp_sti_st0 | ((uint)0xDEC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcomp_st0_sti_DED0 | ((uint)0xDED0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcompp | ((uint)0xDED9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Fsubrp_sti_st0 | ((uint)0xDEE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fsubp_sti_st0 | ((uint)0xDEE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdivrp_sti_st0 | ((uint)0xDEF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fdivp_sti_st0 | ((uint)0xDEF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi | ((uint)LegacyOpKind.ST << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fild_m16int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fisttp_m16int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fist_m16int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fistp_m16int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi16,
				(uint)EncFlags1.Legacy | (uint)Code.Fbld_m80bcd | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfbcd,
				(uint)EncFlags1.Legacy | (uint)Code.Fild_m64int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi64,
				(uint)EncFlags1.Legacy | (uint)Code.Fbstp_m80bcd | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfbcd,
				(uint)EncFlags1.Legacy | (uint)Code.Fistp_m64int | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Mfi64,
				(uint)EncFlags1.Legacy | (uint)Code.Ffreep_sti | ((uint)0xDFC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fxch_st0_sti_DFC8 | ((uint)0xDFC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_sti_DFD0 | ((uint)0xDFD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fstp_sti_DFD8 | ((uint)0xDFD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.STi,
				(uint)EncFlags1.Legacy | (uint)Code.Fnstsw_AX | ((uint)0xDFE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AX,
				(uint)EncFlags1.Legacy | (uint)Code.Fstsw_AX | ((uint)0xDFE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Fwait, (uint)LegacyOpKind.AX,
				(uint)EncFlags1.Legacy | (uint)Code.Fstdw_AX | ((uint)0xDFE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, (uint)LegacyOpKind.AX,
				(uint)EncFlags1.Legacy | (uint)Code.Fstsg_AX | ((uint)0xDFE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632, (uint)LegacyOpKind.AX,
				(uint)EncFlags1.Legacy | (uint)Code.Fucomip_st0_sti | ((uint)0xDFE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fcomip_st0_sti | ((uint)0xDFF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.ST | ((uint)LegacyOpKind.STi << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_16_CX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_32_CX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_16_ECX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_32_ECX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a32_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_64_ECX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_16_RCX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loopne_rel8_64_RCX | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_16_CX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_32_CX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_16_ECX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_32_ECX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a32_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_64_ECX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_16_RCX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loope_rel8_64_RCX | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_16_CX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_32_CX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_16_ECX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_32_ECX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a32_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_64_ECX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_16_RCX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Loop_rel8_64_RCX | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jcxz_rel8_16 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jcxz_rel8_32 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jecxz_rel8_16 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32_o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jecxz_rel8_32 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a32_o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jecxz_rel8_64 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jrcxz_rel8_16 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jrcxz_rel8_64 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.In_AL_imm8 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.In_AX_imm8 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.In_EAX_imm8 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_imm8_AL | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Ib | ((uint)LegacyOpKind.AL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_imm8_AX | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ib | ((uint)LegacyOpKind.AX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_imm8_EAX | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ib | ((uint)LegacyOpKind.EAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Call_rel16 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Call_rel32_32 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Call_rel32_64 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel16 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel32_32 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel32_64 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Bnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_ptr1616 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Aww,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_ptr1632 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Adw,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel8_16 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.wJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel8_32 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.dJb,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rel8_64 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.qJb,
				(uint)EncFlags1.Legacy | (uint)Code.In_AL_DX | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.AL | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.In_AX_DX | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.AX | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.In_EAX_DX | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.EAX | ((uint)LegacyOpKind.DX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_DX_AL | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.AL << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_DX_AX | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.AX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Out_DX_EAX | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.DX | ((uint)LegacyOpKind.EAX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Int1 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Hlt | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cmc | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm8_imm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm8_imm8_F6r1 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Not_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Neg_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Mul_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Div_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Idiv_rm8 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm16_imm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm32_imm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm64_imm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm16_imm16_F7r1 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Iw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm32_imm32_F7r1 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Id << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Test_rm64_imm32_F7r1 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Id64 << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Not_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Not_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Not_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Neg_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Neg_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Neg_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Mul_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Mul_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Mul_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Div_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Div_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Div_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Idiv_rm16 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Idiv_rm32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Idiv_rm64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Clc | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Stc | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cli | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sti | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cld | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Std | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_rm8 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_rm8 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Inc_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Dec_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Call_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Call_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Call_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Call_m1616 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Eww,
				(uint)EncFlags1.Legacy | (uint)Code.Call_m1632 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Edw,
				(uint)EncFlags1.Legacy | (uint)Code.Call_m1664 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eqw,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.BndNotrack, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_m1616 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Eww,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_m1632 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Edw,
				(uint)EncFlags1.Legacy | (uint)Code.Jmp_m1664 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eqw,
				(uint)EncFlags1.Legacy | (uint)Code.Push_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Push_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Push_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Sldt_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Sldt_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Sldt_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Str_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Str_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Str_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Lldt_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Lldt_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Lldt_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Ltr_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Ltr_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Ltr_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Verr_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Verr_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Verr_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Verw_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Verw_r32m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Verw_r64m16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Jmpe_rm16 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Jmpe_rm32 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Sgdt_m1632_16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Sgdt_m1632 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Sgdt_m1664 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Sidt_m1632_16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Sidt_m1632 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Sidt_m1664 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lgdt_m1632_16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lgdt_m1632 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lgdt_m1664 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lidt_m1632_16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lidt_m1632 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Lidt_m1664 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Ms,
				(uint)EncFlags1.Legacy | (uint)Code.Smsw_rm16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Smsw_r32m16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Smsw_r64m16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Rstorssp_m64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Mq,
				(uint)EncFlags1.Legacy | (uint)Code.Lmsw_rm16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Lmsw_r32m16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ew_d,
				(uint)EncFlags1.Legacy | (uint)Code.Lmsw_r64m16 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Ew_q,
				(uint)EncFlags1.Legacy | (uint)Code.Invlpg_m | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Enclv | ((uint)0x01C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmcall | ((uint)0x01C1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmlaunch | ((uint)0x01C2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmresume | ((uint)0x01C3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmxoff | ((uint)0x01C4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pconfig | ((uint)0x01C5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitorw | ((uint)0x01C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitord | ((uint)0x01C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitorq | ((uint)0x01C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mwait | ((uint)0x01C9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clac | ((uint)0x01CA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Stac | ((uint)0x01CB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Encls | ((uint)0x01CF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xgetbv | ((uint)0x01D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsetbv | ((uint)0x01D1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmfunc | ((uint)0x01D4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xend | ((uint)0x01D5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xtest | ((uint)0x01D6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Enclu | ((uint)0x01D7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmrunw | ((uint)0x01D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmrund | ((uint)0x01D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmrunq | ((uint)0x01D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmmcall | ((uint)0x01D9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmloadw | ((uint)0x01DA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmloadd | ((uint)0x01DA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmloadq | ((uint)0x01DA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmsavew | ((uint)0x01DB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmsaved | ((uint)0x01DB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmsaveq | ((uint)0x01DB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Stgi | ((uint)0x01DC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clgi | ((uint)0x01DD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Skinit | ((uint)0x01DE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Invlpgaw | ((uint)0x01DF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Invlpgad | ((uint)0x01DF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Invlpgaq | ((uint)0x01DF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Setssbsy | ((uint)0x01E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Saveprevssp | ((uint)0x01EA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdpkru | ((uint)0x01EE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Wrpkru | ((uint)0x01EF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Swapgs | ((uint)0x01F8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdtscp | ((uint)0x01F9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitorxw | ((uint)0x01FA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP | (uint)LegacyFlags.a16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitorxd | ((uint)0x01FA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP | (uint)LegacyFlags.a32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Monitorxq | ((uint)0x01FA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mcommit | ((uint)0x01FA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mwaitx | ((uint)0x01FB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clzerow | ((uint)0x01FC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clzerod | ((uint)0x01FC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clzeroq | ((uint)0x01FC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdpru | ((uint)0x01FD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lar_r16_rm16 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lar_r32_r32m16 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lar_r64_r64m16 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lsl_r16_rm16 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lsl_r32_r32m16 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lsl_r64_r64m16 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Loadallreset286 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Loadall286 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Syscall | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Clts | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Loadall386 | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sysretd | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sysretq | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Invd | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Wbinvd | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Wbnoinvd | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cl1invmb | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Ud2 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F0D | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F0D | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F0D | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Prefetch_m8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Prefetchw_m8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Prefetchwt1_m8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Femms | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Umov_rm8_r8 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Umov_rm16_r16 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Umov_rm32_r32 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Umov_r8_rm8 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Gb | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Umov_r16_rm16 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Umov_r32_rm32 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movups_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovups_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovups_ymm_ymmm256 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_xmm_k1z_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_ymm_k1z_ymmm256 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_zmm_k1z_zmmm512 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movupd_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovupd_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovupd_ymm_ymmm256 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_xmm_k1z_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_ymm_k1z_ymmm256 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_zmm_k1z_zmmm512 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movss_xmm_xmmm32 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovss_xmm_xmm_xmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovss_xmm_m32 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovss_xmm_k1z_xmm_xmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovss_xmm_k1z_m32 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsd_xmm_xmmm64 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsd_xmm_xmm_xmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsd_xmm_m64 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsd_xmm_k1z_m64 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movups_xmmm128_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovups_xmmm128_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovups_ymmm256_ymm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_xmmm128_k1z_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_ymmm256_k1z_ymm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovups_zmmm512_k1z_zmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movupd_xmmm128_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovupd_xmmm128_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovupd_ymmm256_ymm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_xmmm128_k1z_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_ymmm256_k1z_ymm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovupd_zmmm512_k1z_zmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movss_xmmm32_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovss_xmm_xmm_xmm_0F11 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.RX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovss_m32_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovss_m32_k1_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsd_xmmm64_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsd_xmm_xmm_xmm_0F11 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.RX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsd_m64_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsd_m64_k1_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movhlps_xmm_xmm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movlps_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovhlps_xmm_xmm_xmm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovlps_xmm_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovhlps_xmm_xmm_xmm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovlps_xmm_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movlpd_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovlpd_xmm_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovlpd_xmm_xmm_m64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsldup_xmm_xmmm128 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsldup_xmm_xmmm128 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovsldup_ymm_ymmm256 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsldup_xmm_k1z_xmmm128 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsldup_ymm_k1z_ymmm256 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovsldup_zmm_k1z_zmmm512 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movddup_xmm_xmmm64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovddup_xmm_xmmm64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovddup_ymm_ymmm256 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovddup_xmm_k1z_xmmm64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.MOVDDUP_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovddup_ymm_k1z_ymmm256 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.MOVDDUP_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovddup_zmm_k1z_zmmm512 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.MOVDDUP_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movlps_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovlps_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovlps_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movlpd_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovlpd_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovlpd_m64_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Unpcklps_xmm_xmmm128 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpcklps_xmm_xmm_xmmm128 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpcklps_ymm_ymm_ymmm256 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Unpcklpd_xmm_xmmm128 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpcklpd_xmm_xmm_xmmm128 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpcklpd_ymm_ymm_ymmm256 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Unpckhps_xmm_xmmm128 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpckhps_xmm_xmm_xmmm128 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpckhps_ymm_ymm_ymmm256 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Unpckhpd_xmm_xmmm128 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpckhpd_xmm_xmm_xmmm128 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vunpckhpd_ymm_ymm_ymmm256 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movlhps_xmm_xmm | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovlhps_xmm_xmm_xmm | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovlhps_xmm_xmm_xmm | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movhps_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovhps_xmm_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovhps_xmm_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movhpd_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovhpd_xmm_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovhpd_xmm_xmm_m64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movshdup_xmm_xmmm128 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovshdup_xmm_xmmm128 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovshdup_ymm_ymmm256 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovshdup_xmm_k1z_xmmm128 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovshdup_ymm_k1z_ymmm256 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovshdup_zmm_k1z_zmmm512 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movhps_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovhps_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovhps_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movhpd_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovhpd_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovhpd_m64_xmm | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F18 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F18 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F18 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F19 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F19 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F19 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1A | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1A | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1A | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1B | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1B | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1B | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1C | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1C | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1C | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1D | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1D | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1D | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1E | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1E | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1E | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm16_r16_0F1F | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm32_r32_0F1F | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.ReservedNop_rm64_r64_0F1F | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Prefetchnta_m8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Prefetcht0_m8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Prefetcht1_m8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Prefetcht2_m8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Bndldx_bnd_mib | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.MIB << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmov_bnd_bndm64 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.BMq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmov_bnd_bndm128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.BMo << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcl_bnd_rm32 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Ed_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcl_bnd_rm64 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Eq_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcu_bnd_rm32 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Ed_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcu_bnd_rm64 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Eq_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndstx_mib_bnd | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.MIB | ((uint)LegacyOpKind.B << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmov_bndm64_bnd | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.BMq | ((uint)LegacyOpKind.B << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmov_bndm128_bnd | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.BMo | ((uint)LegacyOpKind.B << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmk_bnd_m32 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Md_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndmk_bnd_m64 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Mq_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcn_bnd_rm32 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Ed_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bndcn_bnd_rm64 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.B | ((uint)LegacyOpKind.Eq_MPX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cldemote_m8 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Rdsspd_r32 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdsspq_r64 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Endbr64 | ((uint)0x1EFA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Endbr32 | ((uint)0x1EFB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Nop_rm16 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew,
				(uint)EncFlags1.Legacy | (uint)Code.Nop_rm32 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Nop_rm64 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32_cr | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Rd | ((uint)LegacyOpKind.Cd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r64_cr | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Rq | ((uint)LegacyOpKind.Cq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32_dr | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Rd | ((uint)LegacyOpKind.Dd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r64_dr | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Rq | ((uint)LegacyOpKind.Dq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_cr_r32 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Cd | ((uint)LegacyOpKind.Rd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_cr_r64 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Cq | ((uint)LegacyOpKind.Rq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_dr_r32 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Dd | ((uint)LegacyOpKind.Rd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_dr_r64 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Dq | ((uint)LegacyOpKind.Rq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_r32_tr | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Rd | ((uint)LegacyOpKind.Td << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mov_tr_r32 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Td | ((uint)LegacyOpKind.Rd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movaps_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovaps_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovaps_ymm_ymmm256 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_xmm_k1z_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_ymm_k1z_ymmm256 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_zmm_k1z_zmmm512 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movapd_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovapd_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovapd_ymm_ymmm256 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_xmm_k1z_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_ymm_k1z_ymmm256 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_zmm_k1z_zmmm512 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movaps_xmmm128_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovaps_xmmm128_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovaps_ymmm256_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_xmmm128_k1z_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_ymmm256_k1z_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovaps_zmmm512_k1z_zmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movapd_xmmm128_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovapd_xmmm128_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovapd_ymmm256_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_xmmm128_k1z_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_ymmm256_k1z_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovapd_zmmm512_k1z_zmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtpi2ps_xmm_mmm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtpi2pd_xmm_mmm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsi2ss_xmm_rm32 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsi2ss_xmm_rm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsi2ss_xmm_xmm_rm32 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsi2ss_xmm_xmm_rm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsi2sd_xmm_rm32 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsi2sd_xmm_rm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsi2sd_xmm_xmm_rm32 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsi2sd_xmm_xmm_rm64 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntps_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntps_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntps_m256_ymm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntps_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntps_m256_ymm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntps_m512_zmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntpd_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntpd_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntpd_m256_ymm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntpd_m128_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntpd_m256_ymm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntpd_m512_zmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntss_m32_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntsd_m64_xmm | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttps2pi_mm_xmmm64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttpd2pi_mm_xmmm128 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttss2si_r32_xmmm32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttss2si_r64_xmmm32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttss2si_r32_xmmm32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttss2si_r64_xmmm32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttss2si_r32_xmmm32_sae | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttss2si_r64_xmmm32_sae | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttsd2si_r32_xmmm64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttsd2si_r64_xmmm64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttsd2si_r32_xmmm64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttsd2si_r64_xmmm64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttsd2si_r32_xmmm64_sae | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttsd2si_r64_xmmm64_sae | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtps2pi_mm_xmmm64 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtpd2pi_mm_xmmm128 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtss2si_r32_xmmm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtss2si_r64_xmmm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtss2si_r32_xmmm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtss2si_r64_xmmm32 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtss2si_r32_xmmm32_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtss2si_r64_xmmm32_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsd2si_r32_xmmm64 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsd2si_r64_xmmm64 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsd2si_r32_xmmm64 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsd2si_r64_xmmm64 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsd2si_r32_xmmm64_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsd2si_r64_xmmm64_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ucomiss_xmm_xmmm32 | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vucomiss_xmm_xmmm32 | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vucomiss_xmm_xmmm32_sae | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ucomisd_xmm_xmmm64 | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vucomisd_xmm_xmmm64 | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vucomisd_xmm_xmmm64_sae | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Comiss_xmm_xmmm32 | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Comisd_xmm_xmmm64 | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcomiss_xmm_xmmm32 | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcomisd_xmm_xmmm64 | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcomiss_xmm_xmmm32_sae | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcomisd_xmm_xmmm64_sae | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Wrmsr | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdtsc | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdmsr | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Rdpmc | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sysenter | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sysexitd | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sysexitq | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Getsec | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Cmovo_r16_rm16 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovo_r32_rm32 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovo_r64_rm64 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovno_r16_rm16 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovno_r32_rm32 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovno_r64_rm64 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovb_r16_rm16 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovb_r32_rm32 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovb_r64_rm64 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovae_r16_rm16 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovae_r32_rm32 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovae_r64_rm64 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmove_r16_rm16 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmove_r32_rm32 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmove_r64_rm64 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovne_r16_rm16 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovne_r32_rm32 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovne_r64_rm64 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovbe_r16_rm16 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovbe_r32_rm32 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovbe_r64_rm64 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmova_r16_rm16 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmova_r32_rm32 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmova_r64_rm64 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovs_r16_rm16 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovs_r32_rm32 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovs_r64_rm64 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovns_r16_rm16 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovns_r32_rm32 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovns_r64_rm64 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovp_r16_rm16 | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovp_r32_rm32 | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovp_r64_rm64 | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovnp_r16_rm16 | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovnp_r32_rm32 | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovnp_r64_rm64 | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovl_r16_rm16 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovl_r32_rm32 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovl_r64_rm64 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovge_r16_rm16 | ((uint)0x4D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovge_r32_rm32 | ((uint)0x4D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovge_r64_rm64 | ((uint)0x4D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovle_r16_rm16 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovle_r32_rm32 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovle_r64_rm64 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovg_r16_rm16 | ((uint)0x4F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovg_r32_rm32 | ((uint)0x4F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmovg_r64_rm64 | ((uint)0x4F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandw_k_k_k | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandq_k_k_k | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandb_k_k_k | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandd_k_k_k | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandnw_k_k_k | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandnq_k_k_k | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandnb_k_k_k | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kandnd_k_k_k | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Knotw_k_k | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Knotq_k_k | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Knotb_k_k | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Knotd_k_k | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Korw_k_k_k | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Korq_k_k_k | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Korb_k_k_k | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kord_k_k_k | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxnorw_k_k_k | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxnorq_k_k_k | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxnorb_k_k_k | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxnord_k_k_k | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxorw_k_k_k | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxorq_k_k_k | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxorb_k_k_k | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kxord_k_k_k | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kaddw_k_k_k | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kaddq_k_k_k | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kaddb_k_k_k | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kaddd_k_k_k | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kunpckwd_k_k_k | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kunpckdq_k_k_k | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L1_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kunpckbw_k_k_k | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L1_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.HK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RK << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movmskps_r32_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movmskps_r64_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskps_r32_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskps_r64_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskps_r32_ymm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskps_r64_ymm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movmskpd_r32_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movmskpd_r64_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskpd_r32_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskpd_r64_xmm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskpd_r32_ymm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovmskpd_r64_ymm | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sqrtps_xmm_xmmm128 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtps_xmm_xmmm128 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtps_ymm_ymmm256 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sqrtpd_xmm_xmmm128 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtpd_xmm_xmmm128 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtpd_ymm_ymmm256 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sqrtss_xmm_xmmm32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtss_xmm_xmm_xmmm32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sqrtsd_xmm_xmmm64 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsqrtsd_xmm_xmm_xmmm64 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rsqrtps_xmm_xmmm128 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrsqrtps_xmm_xmmm128 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrsqrtps_ymm_ymmm256 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rsqrtss_xmm_xmmm32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrsqrtss_xmm_xmm_xmmm32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcpps_xmm_xmmm128 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrcpps_xmm_xmmm128 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrcpps_ymm_ymmm256 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Rcpss_xmm_xmmm32 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vrcpss_xmm_xmm_xmmm32 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Andps_xmm_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandps_xmm_xmm_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandps_ymm_ymm_ymmm256 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Andpd_xmm_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandpd_xmm_xmm_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandpd_ymm_ymm_ymmm256 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Andnps_xmm_xmmm128 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandnps_xmm_xmm_xmmm128 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandnps_ymm_ymm_ymmm256 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Andnpd_xmm_xmmm128 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandnpd_xmm_xmm_xmmm128 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vandnpd_ymm_ymm_ymmm256 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Orps_xmm_xmmm128 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vorps_xmm_xmm_xmmm128 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vorps_ymm_ymm_ymmm256 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Orpd_xmm_xmmm128 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vorpd_xmm_xmm_xmmm128 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vorpd_ymm_ymm_ymmm256 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xorps_xmm_xmmm128 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vxorps_xmm_xmm_xmmm128 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vxorps_ymm_ymm_ymmm256 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xorpd_xmm_xmmm128 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vxorpd_xmm_xmm_xmmm128 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vxorpd_ymm_ymm_ymmm256 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Addps_xmm_xmmm128 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddps_xmm_xmm_xmmm128 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddps_ymm_ymm_ymmm256 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Addpd_xmm_xmmm128 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddpd_xmm_xmm_xmmm128 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddpd_ymm_ymm_ymmm256 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Addss_xmm_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddss_xmm_xmm_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Addsd_xmm_xmmm64 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddsd_xmm_xmm_xmmm64 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mulps_xmm_xmmm128 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulps_xmm_xmm_xmmm128 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulps_ymm_ymm_ymmm256 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mulpd_xmm_xmmm128 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulpd_xmm_xmm_xmmm128 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulpd_ymm_ymm_ymmm256 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mulss_xmm_xmmm32 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulss_xmm_xmm_xmmm32 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mulsd_xmm_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmulsd_xmm_xmm_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtps2pd_xmm_xmmm64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2pd_xmm_xmmm64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2pd_ymm_xmmm128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtpd2ps_xmm_xmmm128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtpd2ps_xmm_xmmm128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtpd2ps_xmm_ymmm256 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtss2sd_xmm_xmmm32 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtsd2ss_xmm_xmmm64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtdq2ps_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtdq2ps_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtdq2ps_ymm_ymmm256 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtps2dq_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2dq_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2dq_ymm_ymmm256 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttps2dq_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttps2dq_xmm_xmmm128 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttps2dq_ymm_ymmm256 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Subps_xmm_xmmm128 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubps_xmm_xmm_xmmm128 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubps_ymm_ymm_ymmm256 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Subpd_xmm_xmmm128 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubpd_xmm_xmm_xmmm128 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubpd_ymm_ymm_ymmm256 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Subss_xmm_xmmm32 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubss_xmm_xmm_xmmm32 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Subsd_xmm_xmmm64 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vsubsd_xmm_xmm_xmmm64 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Minps_xmm_xmmm128 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminps_xmm_xmm_xmmm128 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminps_ymm_ymm_ymmm256 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Minpd_xmm_xmmm128 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminpd_xmm_xmm_xmmm128 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminpd_ymm_ymm_ymmm256 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Minss_xmm_xmmm32 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminss_xmm_xmm_xmmm32 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Minsd_xmm_xmmm64 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vminsd_xmm_xmm_xmmm64 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Divps_xmm_xmmm128 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivps_xmm_xmm_xmmm128 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivps_ymm_ymm_ymmm256 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Divpd_xmm_xmmm128 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivpd_xmm_xmm_xmmm128 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivpd_ymm_ymm_ymmm256 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Divss_xmm_xmmm32 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivss_xmm_xmm_xmmm32 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Divsd_xmm_xmmm64 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdivsd_xmm_xmm_xmmm64 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maxps_xmm_xmmm128 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxps_xmm_xmm_xmmm128 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxps_ymm_ymm_ymmm256 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maxpd_xmm_xmmm128 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxpd_xmm_xmm_xmmm128 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxpd_ymm_ymm_ymmm256 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maxss_xmm_xmmm32 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxss_xmm_xmm_xmmm32 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maxsd_xmm_xmmm64 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaxsd_xmm_xmm_xmmm64 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpcklbw_mm_mmm32 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpcklbw_xmm_xmmm128 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpcklwd_mm_mmm32 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpcklwd_xmm_xmmm128 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckldq_mm_mmm32 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckldq_xmm_xmmm128 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckldq_xmm_xmm_xmmm128 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckldq_ymm_ymm_ymmm256 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packsswb_mm_mmm64 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packsswb_xmm_xmmm128 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpacksswb_xmm_xmm_xmmm128 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpacksswb_ymm_ymm_ymmm256 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtb_mm_mmm64 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtb_xmm_xmmm128 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtb_k_k1_xmm_xmmm128 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtb_k_k1_ymm_ymmm256 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtb_k_k1_zmm_zmmm512 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtw_mm_mmm64 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtw_xmm_xmmm128 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtw_k_k1_xmm_xmmm128 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtw_k_k1_ymm_ymmm256 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtw_k_k1_zmm_zmmm512 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtd_mm_mmm64 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtd_xmm_xmmm128 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtd_k_k1_xmm_xmmm128b32 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtd_k_k1_ymm_ymmm256b32 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtd_k_k1_zmm_zmmm512b32 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packuswb_mm_mmm64 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packuswb_xmm_xmmm128 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackuswb_xmm_xmm_xmmm128 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackuswb_ymm_ymm_ymmm256 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhbw_mm_mmm64 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhbw_xmm_xmmm128 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhwd_mm_mmm64 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhwd_xmm_xmmm128 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhdq_mm_mmm64 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhdq_xmm_xmmm128 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packssdw_mm_mmm64 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packssdw_xmm_xmmm128 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackssdw_xmm_xmm_xmmm128 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackssdw_ymm_ymm_ymmm256 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpcklqdq_xmm_xmmm128 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Punpckhqdq_xmm_xmmm128 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movd_mm_rm32 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_mm_rm64 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movd_xmm_rm32 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_xmm_rm64 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovd_xmm_rm32 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovq_xmm_rm64 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovd_xmm_rm32 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovq_xmm_rm64 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_mm_mmm64 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdqa_xmm_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqa_xmm_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqa_ymm_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdqu_xmm_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqu_xmm_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqu_ymm_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshufw_mm_mmm64_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshufd_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufd_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufd_ymm_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshufhw_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufhw_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufhw_ymm_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshuflw_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshuflw_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshuflw_ymm_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlw_mm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlw_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlw_xmm_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlw_ymm_ymm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psraw_mm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psraw_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsraw_xmm_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group4 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsraw_ymm_ymm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group4 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllw_mm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllw_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllw_xmm_xmm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllw_ymm_ymm_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group0 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrld_mm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrld_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrld_xmm_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrld_ymm_ymm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrad_mm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrad_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrad_xmm_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group4 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrad_ymm_ymm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group4 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group4 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pslld_mm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pslld_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslld_xmm_xmm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslld_ymm_ymm_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlq_mm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlq_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlq_xmm_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlq_ymm_ymm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrldq_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrldq_xmm_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group3 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrldq_ymm_ymm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group3 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrldq_xmm_xmmm128_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group3 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrldq_ymm_ymmm256_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group3 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrldq_zmm_zmmm512_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group3 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllq_mm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.N | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllq_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllq_xmm_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllq_ymm_ymm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group6 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pslldq_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslldq_xmm_xmm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group7 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.HX | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslldq_ymm_ymm_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group7 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.HY | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslldq_xmm_xmmm128_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group7 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslldq_ymm_ymmm256_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group7 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslldq_zmm_zmmm512_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group7 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.HZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqb_mm_mmm64 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqb_xmm_xmmm128 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqb_k_k1_xmm_xmmm128 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqb_k_k1_ymm_ymmm256 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqb_k_k1_zmm_zmmm512 | ((uint)0x74 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqw_mm_mmm64 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqw_xmm_xmmm128 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqw_k_k1_xmm_xmmm128 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqw_k_k1_ymm_ymmm256 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqw_k_k1_zmm_zmmm512 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqd_mm_mmm64 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqd_xmm_xmmm128 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqd_k_k1_xmm_xmmm128b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqd_k_k1_ymm_ymmm256b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqd_k_k1_zmm_zmmm512b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Emms | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vzeroupper | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, 0,
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vzeroall | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Vmread_rm32_r32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Vmread_rm64_r64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Extrq_xmm_imm8_imm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66, (uint)LegacyOpKind.RX | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib11 << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttss2usi_r32_xmmm32_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttss2usi_r64_xmmm32_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insertq_xmm_xmm_imm8_imm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift) | ((uint)LegacyOpKind.Ib11 << (int)LegacyFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Vmwrite_r32_rm32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Vmwrite_r64_rm64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Extrq_xmm_xmm | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtss2usi_r32_xmmm32_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtss2usi_r64_xmmm32_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insertq_xmm_xmm | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsd2usi_r32_xmmm64_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtsd2usi_r64_xmmm64_er | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Fixed_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Fixed_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Haddpd_xmm_xmmm128 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhaddpd_xmm_xmm_xmmm128 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhaddpd_ymm_ymm_ymmm256 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Haddps_xmm_xmmm128 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhaddps_xmm_xmm_xmmm128 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhaddps_ymm_ymm_ymmm256 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Hsubpd_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhsubpd_xmm_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhsubpd_ymm_ymm_ymmm256 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Hsubps_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhsubps_xmm_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vhsubps_ymm_ymm_ymmm256 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movd_rm32_mm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.P << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_rm64_mm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.P << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movd_rm32_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_rm64_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovd_rm32_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Ed | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovq_rm64_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Eq | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovd_rm32_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Ed | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovq_rm64_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Eq | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_xmm_xmmm64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovq_xmm_xmmm64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovq_xmm_xmmm64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_mmm64_mm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Q | ((uint)LegacyOpKind.P << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdqa_xmmm128_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqa_xmmm128_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqa_ymmm256_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdqu_xmmm128_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqu_xmmm128_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovdqu_ymmm256_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.WY | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel16 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel32_32 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jo_rel32_64 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel16 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel32_32 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jno_rel32_64 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel16 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel32_32 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jb_rel32_64 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel16 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel32_32 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jae_rel32_64 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel16 | ((uint)0x84 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel32_32 | ((uint)0x84 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Je_rel32_64 | ((uint)0x84 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel16 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel32_32 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jne_rel32_64 | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel16 | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel32_32 | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jbe_rel32_64 | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel16 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel32_32 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Ja_rel32_64 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel16 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel32_32 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Js_rel32_64 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel16 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel32_32 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jns_rel32_64 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel16 | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel32_32 | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jp_rel32_64 | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel16 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel32_32 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jnp_rel32_64 | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel16 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel32_32 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jl_rel32_64 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel16 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel32_32 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jge_rel32_64 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel16 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel32_32 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jle_rel32_64 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel16 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.Jw,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel32_32 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.dJd,
				(uint)EncFlags1.Legacy | (uint)Code.Jg_rel32_64 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.HintTakenBnd, (uint)LegacyOpKind.qJd,
				(uint)EncFlags1.Legacy | (uint)Code.Seto_rm8 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setno_rm8 | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setb_rm8 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setae_rm8 | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Sete_rm8 | ((uint)0x94 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setne_rm8 | ((uint)0x95 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setbe_rm8 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Seta_rm8 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Sets_rm8 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setns_rm8 | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setp_rm8 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setnp_rm8 | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setl_rm8 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setge_rm8 | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setle_rm8 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.Legacy | (uint)Code.Setg_rm8 | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb,
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovw_k_km16 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.WK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovq_k_km64 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.WK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovb_k_km8 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.WK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovd_k_km32 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.WK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovw_m16_k | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.MK | ((uint)VexOpKind.VK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovq_m64_k | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.MK | ((uint)VexOpKind.VK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovb_m8_k | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.MK | ((uint)VexOpKind.VK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovd_m32_k | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.MK | ((uint)VexOpKind.VK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovw_k_r32 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.Rd << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovb_k_r32 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.Rd << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovd_k_r32 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.Rd << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovq_k_r64 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.Rq << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovw_r32_k | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.Gd | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovb_r32_k | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.Gd | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovd_r32_k | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.Gd | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kmovq_r64_k | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kortestw_k_k | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kortestq_k_k | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kortestb_k_k | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kortestd_k_k | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Ktestw_k_k | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Ktestq_k_k | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Ktestb_k_k | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Ktestd_k_k | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_FS | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_FS | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushq_FS | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_FS | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Popd_FS | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Popq_FS | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.FS,
				(uint)EncFlags1.Legacy | (uint)Code.Cpuid | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm16_r16 | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm32_r32 | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm64_r64 | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm16_r16_imm8 | ((uint)0xA4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm32_r32_imm8 | ((uint)0xA4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm64_r64_imm8 | ((uint)0xA4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm16_r16_CL | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm32_r32_CL | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shld_rm64_r64_CL | ((uint)0xA5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Montmul_16 | ((uint)0xA6C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Montmul_32 | ((uint)0xA6C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Montmul_64 | ((uint)0xA6C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha1_16 | ((uint)0xA6C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha1_32 | ((uint)0xA6C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha1_64 | ((uint)0xA6C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha256_16 | ((uint)0xA6D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha256_32 | ((uint)0xA6D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xsha256_64 | ((uint)0xA6D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xbts_r16_rm16 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xbts_r32_rm32 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xstore_16 | ((uint)0xA7C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xstore_32 | ((uint)0xA7C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Xstore_64 | ((uint)0xA7C0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptEcb_16 | ((uint)0xA7C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptEcb_32 | ((uint)0xA7C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptEcb_64 | ((uint)0xA7C8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCbc_16 | ((uint)0xA7D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCbc_32 | ((uint)0xA7D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCbc_64 | ((uint)0xA7D0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCtr_16 | ((uint)0xA7D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCtr_32 | ((uint)0xA7D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCtr_64 | ((uint)0xA7D8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCfb_16 | ((uint)0xA7E0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCfb_32 | ((uint)0xA7E0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptCfb_64 | ((uint)0xA7E0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptOfb_16 | ((uint)0xA7E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a16 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptOfb_32 | ((uint)0xA7E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.a32 | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.XcryptOfb_64 | ((uint)0xA7E8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.Rep, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Ibts_rm16_r16 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ibts_rm32_r32 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg486_rm8_r8 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg486_rm16_r16 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg486_rm32_r32 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pushw_GS | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushd_GS | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Pushq_GS | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Popw_GS | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Popd_GS | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Popq_GS | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F, (uint)LegacyOpKind.GS,
				(uint)EncFlags1.Legacy | (uint)Code.Rsm | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm16_r16 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm32_r32 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm64_r64 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm16_r16_imm8 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm32_r32_imm8 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm64_r64_imm8 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm16_r16_CL | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm32_r32_CL | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shrd_rm64_r64_CL | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.CL << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Fxsave_m512byte | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Fxsave64_m512byte | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Rdfsbase_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdfsbase_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group0 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Fxrstor_m512byte | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Fxrstor64_m512byte | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Rdgsbase_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdgsbase_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Ldmxcsr_m32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Md,
				(uint)EncFlags1.Legacy | (uint)Code.Wrfsbase_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Wrfsbase_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group2 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vldmxcsr_m32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_LZ_WIG, (uint)VexOpKind.Md,
				(uint)EncFlags1.Legacy | (uint)Code.Stmxcsr_m32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Md,
				(uint)EncFlags1.Legacy | (uint)Code.Wrgsbase_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Wrgsbase_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vstmxcsr_m32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group3 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_LZ_WIG, (uint)VexOpKind.Md,
				(uint)EncFlags1.Legacy | (uint)Code.Xsave_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsave64_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Ptwrite_rm32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Ed,
				(uint)EncFlags1.Legacy | (uint)Code.Ptwrite_rm64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq,
				(uint)EncFlags1.Legacy | (uint)Code.Xrstor_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xrstor64_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Incsspd_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Incsspq_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Xsaveopt_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsaveopt64_mem | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Clwb_m8 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Tpause_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Tpause_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Clrssbsy_m64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Mq,
				(uint)EncFlags1.Legacy | (uint)Code.Umonitor_r16 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.a16, (uint)LegacyOpKind.Rw,
				(uint)EncFlags1.Legacy | (uint)Code.Umonitor_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Umonitor_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Umwait_r32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Umwait_r64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Clflush_m8 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Clflushopt_m8 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Mb,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence | ((uint)0xAEE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_E9 | ((uint)0xAEE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_EA | ((uint)0xAEEA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_EB | ((uint)0xAEEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_EC | ((uint)0xAEEC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_ED | ((uint)0xAEED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_EE | ((uint)0xAEEE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Lfence_EF | ((uint)0xAEEF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence | ((uint)0xAEF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F1 | ((uint)0xAEF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F2 | ((uint)0xAEF2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F3 | ((uint)0xAEF3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F4 | ((uint)0xAEF4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F5 | ((uint)0xAEF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F6 | ((uint)0xAEF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Mfence_F7 | ((uint)0xAEF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence | ((uint)0xAEF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_F9 | ((uint)0xAEF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FA | ((uint)0xAEFA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FB | ((uint)0xAEFB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FC | ((uint)0xAEFC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FD | ((uint)0xAEFD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FE | ((uint)0xAEFE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Sfence_FF | ((uint)0xAEFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Pcommit | ((uint)0xAEF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, 0,
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r16_rm16 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r32_rm32 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Imul_r64_rm64 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg_rm8_r8 | ((uint)0xB0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg_rm16_r16 | ((uint)0xB1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg_rm32_r32 | ((uint)0xB1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg_rm64_r64 | ((uint)0xB1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lss_r16_m1616 | ((uint)0xB2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lss_r32_m1632 | ((uint)0xB2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lss_r64_m1664 | ((uint)0xB2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm16_r16 | ((uint)0xB3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm32_r32 | ((uint)0xB3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm64_r64 | ((uint)0xB3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lfs_r16_m1616 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lfs_r32_m1632 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lfs_r64_m1664 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lgs_r16_m1616 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lgs_r32_m1632 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lgs_r64_m1664 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Mp << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r16_rm8 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r32_rm8 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r64_rm8 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r16_rm16 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r32_rm16 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movzx_r64_rm16 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Jmpe_disp16 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Jdisp16,
				(uint)EncFlags1.Legacy | (uint)Code.Jmpe_disp32 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Jdisp32,
				(uint)EncFlags1.Legacy | (uint)Code.Popcnt_r16_rm16 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Popcnt_r32_rm32 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Popcnt_r64_rm64 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud1_r16_rm16 | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud1_r32_rm32 | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud1_r64_rm64 | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm16_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm32_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bt_rm64_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm16_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm32_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bts_rm64_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm16_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm32_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btr_rm64_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm16_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm32_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm64_imm8 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm16_r16 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm32_r32 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Btc_rm64_r64 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsf_r16_rm16 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsf_r32_rm32 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsf_r64_rm64 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Tzcnt_r16_rm16 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Tzcnt_r32_rm32 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Tzcnt_r64_rm64 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsr_r16_rm16 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsr_r32_rm32 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Bsr_r64_rm64 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lzcnt_r16_rm16 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lzcnt_r32_rm32 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lzcnt_r64_rm64 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r16_rm8 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r32_rm8 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r64_rm8 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r16_rm16 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r32_rm16 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movsx_r64_rm16 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xadd_rm8_r8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eb | ((uint)LegacyOpKind.Gb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xadd_rm16_r16 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ew | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xadd_rm32_r32 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32 | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Xadd_rm64_r64 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpps_xmm_xmmm128_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmppd_xmm_xmmm128_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpss_xmm_xmmm32_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpsd_xmm_xmmm64_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movnti_m32_r32 | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Md | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movnti_m64_r64 | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Mq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrw_mm_r32m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.RdMw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrw_mm_r64m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.RqMw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrw_xmm_r32m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RdMw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrw_xmm_r64m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RqMw << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RdMw << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RqMw << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RdMw << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RqMw << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r32_mm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r64_mm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r32_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r64_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrw_r32_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrw_r64_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrw_r32_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gd | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrw_r64_xmm_imm8 | ((uint)0xC5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Gq | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shufps_xmm_xmmm128_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Shufpd_xmm_xmmm128_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg8b_m64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.XacquireXreleaseLock, (uint)LegacyOpKind.Mq,
				(uint)EncFlags1.Legacy | (uint)Code.Cmpxchg16b_m128 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group1 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.Lock, (uint)LegacyOpKind.Mo,
				(uint)EncFlags1.Legacy | (uint)Code.Xrstors_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xrstors64_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group3 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsavec_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsavec64_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group4 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsaves_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Xsaves64_mem | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group5 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Vmptrld_m64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Vmclear_m64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Vmxon_m64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Rdrand_r16 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Rw,
				(uint)EncFlags1.Legacy | (uint)Code.Rdrand_r32 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdrand_r64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group6 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Vmptrst_m64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M,
				(uint)EncFlags1.Legacy | (uint)Code.Rdseed_r16 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Rw,
				(uint)EncFlags1.Legacy | (uint)Code.Rdseed_r32 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdseed_r64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Rdpid_r32 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rd,
				(uint)EncFlags1.Legacy | (uint)Code.Rdpid_r64 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Group7 | (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Rq,
				(uint)EncFlags1.Legacy | (uint)Code.Bswap_r16 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.r16_rw,
				(uint)EncFlags1.Legacy | (uint)Code.Bswap_r32 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.r32_rd,
				(uint)EncFlags1.Legacy | (uint)Code.Bswap_r64 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.r64_ro,
				(uint)EncFlags1.Legacy | (uint)Code.Addsubpd_xmm_xmmm128 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddsubpd_xmm_xmm_xmmm128 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddsubpd_ymm_ymm_ymmm256 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Addsubps_xmm_xmmm128 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddsubps_xmm_xmm_xmmm128 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaddsubps_ymm_ymm_ymmm256 | ((uint)0xD0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlw_mm_mmm64 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlw_xmm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlw_xmm_xmm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlw_ymm_ymm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrld_mm_mmm64 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrld_xmm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrld_xmm_xmm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrld_ymm_ymm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlq_mm_mmm64 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrlq_xmm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlq_xmm_xmm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlq_ymm_ymm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddq_mm_mmm64 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddq_xmm_xmmm128 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddq_xmm_xmm_xmmm128 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddq_ymm_ymm_ymmm256 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xD4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmullw_mm_mmm64 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmullw_xmm_xmmm128 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmullw_xmm_xmm_xmmm128 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmullw_ymm_ymm_ymmm256 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512 | ((uint)0xD5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq_xmmm64_xmm | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.WX | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovq_xmmm64_xmm | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovq_xmmm64_xmm | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movq2dq_xmm_mm | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdq2q_mm_xmm | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovmskb_r32_mm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovmskb_r64_mm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovmskb_r32_xmm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovmskb_r64_xmm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovmskb_r32_xmm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovmskb_r64_xmm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovmskb_r32_ymm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovmskb_r64_ymm | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.RY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubusb_mm_mmm64 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubusb_xmm_xmmm128 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubusb_xmm_xmm_xmmm128 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubusb_ymm_ymm_ymmm256 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512 | ((uint)0xD8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubusw_mm_mmm64 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubusw_xmm_xmmm128 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubusw_xmm_xmm_xmmm128 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubusw_ymm_ymm_ymmm256 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512 | ((uint)0xD9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminub_mm_mmm64 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminub_xmm_xmmm128 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminub_xmm_xmm_xmmm128 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminub_ymm_ymm_ymmm256 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512 | ((uint)0xDA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pand_mm_mmm64 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pand_xmm_xmmm128 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpand_xmm_xmm_xmmm128 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpand_ymm_ymm_ymmm256 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddusb_mm_mmm64 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddusb_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddusb_xmm_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddusb_ymm_ymm_ymmm256 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddusw_mm_mmm64 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddusw_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddusw_xmm_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddusw_ymm_ymm_ymmm256 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxub_mm_mmm64 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxub_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxub_xmm_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxub_ymm_ymm_ymmm256 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pandn_mm_mmm64 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pandn_xmm_xmmm128 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpandn_xmm_xmm_xmmm128 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpandn_ymm_ymm_ymmm256 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pavgb_mm_mmm64 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pavgb_xmm_xmmm128 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpavgb_xmm_xmm_xmmm128 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpavgb_ymm_ymm_ymmm256 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512 | ((uint)0xE0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psraw_mm_mmm64 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psraw_xmm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsraw_xmm_xmm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsraw_ymm_ymm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrad_mm_mmm64 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psrad_xmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrad_xmm_xmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrad_ymm_ymm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pavgw_mm_mmm64 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pavgw_xmm_xmmm128 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpavgw_xmm_xmm_xmmm128 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpavgw_ymm_ymm_ymmm256 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhuw_mm_mmm64 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhuw_xmm_xmmm128 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhuw_xmm_xmm_xmmm128 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhuw_ymm_ymm_ymmm256 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512 | ((uint)0xE4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhw_mm_mmm64 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhw_xmm_xmmm128 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhw_xmm_xmm_xmmm128 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhw_ymm_ymm_ymmm256 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512 | ((uint)0xE5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvttpd2dq_xmm_xmmm128 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttpd2dq_xmm_xmmm128 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvttpd2dq_xmm_ymmm256 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtdq2pd_xmm_xmmm64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtdq2pd_xmm_xmmm64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtdq2pd_ymm_xmmm128 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Cvtpd2dq_xmm_xmmm128 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtpd2dq_xmm_xmmm128 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtpd2dq_xmm_ymmm256 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64 | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er | ((uint)0xE6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntq_m64_mm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.P << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntdq_m128_xmm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.M | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntdq_m128_xmm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntdq_m256_ymm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.M | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdq_m128_xmm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdq_m256_ymm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdq_m512_zmm | ((uint)0xE7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.M | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubsb_mm_mmm64 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubsb_xmm_xmmm128 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubsb_xmm_xmm_xmmm128 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubsb_ymm_ymm_ymmm256 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512 | ((uint)0xE8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubsw_mm_mmm64 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubsw_xmm_xmmm128 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubsw_xmm_xmm_xmmm128 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubsw_ymm_ymm_ymmm256 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512 | ((uint)0xE9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminsw_mm_mmm64 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminsw_xmm_xmmm128 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsw_xmm_xmm_xmmm128 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsw_ymm_ymm_ymmm256 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512 | ((uint)0xEA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Por_mm_mmm64 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Por_xmm_xmmm128 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpor_xmm_xmm_xmmm128 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpor_ymm_ymm_ymmm256 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xEB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddsb_mm_mmm64 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddsb_xmm_xmmm128 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddsb_xmm_xmm_xmmm128 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddsb_ymm_ymm_ymmm256 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddsw_mm_mmm64 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddsw_xmm_xmmm128 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddsw_xmm_xmm_xmmm128 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddsw_ymm_ymm_ymmm256 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxsw_mm_mmm64 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxsw_xmm_xmmm128 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsw_xmm_xmm_xmmm128 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsw_ymm_ymm_ymmm256 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pxor_mm_mmm64 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pxor_xmm_xmmm128 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpxor_xmm_xmm_xmmm128 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpxor_ymm_ymm_ymmm256 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Lddqu_xmm_m128 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vlddqu_xmm_m128 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vlddqu_ymm_m256 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllw_mm_mmm64 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllw_xmm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllw_xmm_xmm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllw_ymm_ymm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pslld_mm_mmm64 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pslld_xmm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslld_xmm_xmm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpslld_ymm_ymm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllq_mm_mmm64 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psllq_xmm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllq_xmm_xmm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllq_ymm_ymm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Mem128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmuludq_mm_mmm64 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmuludq_xmm_xmmm128 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmuludq_xmm_xmm_xmmm128 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmuludq_ymm_ymm_ymmm256 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xF4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaddwd_mm_mmm64 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaddwd_xmm_xmmm128 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaddwd_xmm_xmm_xmmm128 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaddwd_ymm_ymm_ymmm256 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psadbw_mm_mmm64 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psadbw_xmm_xmmm128 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsadbw_xmm_xmm_xmmm128 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsadbw_ymm_ymm_ymmm256 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsadbw_xmm_xmm_xmmm128 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsadbw_ymm_ymm_ymmm256 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsadbw_zmm_zmm_zmmm512 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maskmovq_rDI_mm_mm | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.rDI | ((uint)LegacyOpKind.P << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.N << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Maskmovdqu_rDI_xmm_xmm | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.rDI | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.RX << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovdqu_rDI_xmm_xmm | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.rDI | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubb_mm_mmm64 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubb_xmm_xmmm128 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubb_xmm_xmm_xmmm128 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubb_ymm_ymm_ymmm256 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubw_mm_mmm64 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubw_xmm_xmmm128 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubw_xmm_xmm_xmmm128 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubw_ymm_ymm_ymmm256 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubd_mm_mmm64 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubd_xmm_xmmm128 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubd_xmm_xmm_xmmm128 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubd_ymm_ymm_ymmm256 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32 | ((uint)0xFA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubq_mm_mmm64 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psubq_xmm_xmmm128 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubq_xmm_xmm_xmmm128 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsubq_ymm_ymm_ymmm256 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xFB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddb_mm_mmm64 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddb_xmm_xmmm128 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddb_xmm_xmm_xmmm128 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddb_ymm_ymm_ymmm256 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512 | ((uint)0xFC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddw_mm_mmm64 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddw_xmm_xmmm128 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddw_xmm_xmm_xmmm128 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddw_ymm_ymm_ymmm256 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512 | ((uint)0xFD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddd_mm_mmm64 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Paddd_xmm_xmmm128 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddd_xmm_xmm_xmmm128 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpaddd_ymm_ymm_ymmm256 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32 | ((uint)0xFE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud0_r16_rm16 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud0_r32_rm32 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ud0_r64_rm64 | ((uint)0xFF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshufb_mm_mmm64 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pshufb_xmm_xmmm128 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufb_xmm_xmm_xmmm128 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpshufb_ymm_ymm_ymmm256 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddw_mm_mmm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddw_xmm_xmmm128 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddw_xmm_xmm_xmmm128 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddw_ymm_ymm_ymmm256 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddd_mm_mmm64 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddd_xmm_xmmm128 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddd_xmm_xmm_xmmm128 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddd_ymm_ymm_ymmm256 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddsw_mm_mmm64 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phaddsw_xmm_xmmm128 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddsw_xmm_xmm_xmmm128 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphaddsw_ymm_ymm_ymmm256 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaddubsw_mm_mmm64 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaddubsw_xmm_xmmm128 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubw_mm_mmm64 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubw_xmm_xmmm128 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubw_xmm_xmm_xmmm128 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubw_ymm_ymm_ymmm256 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubd_mm_mmm64 | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubd_xmm_xmmm128 | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubd_xmm_xmm_xmmm128 | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubd_ymm_ymm_ymmm256 | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubsw_mm_mmm64 | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phsubsw_xmm_xmmm128 | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubsw_xmm_xmm_xmmm128 | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphsubsw_ymm_ymm_ymmm256 | ((uint)0x07 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignb_mm_mmm64 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignb_xmm_xmmm128 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignb_xmm_xmm_xmmm128 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignb_ymm_ymm_ymmm256 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignw_mm_mmm64 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignw_xmm_xmmm128 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignw_xmm_xmm_xmmm128 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignw_ymm_ymm_ymmm256 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignd_mm_mmm64 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Psignd_xmm_xmmm128 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignd_xmm_xmm_xmmm128 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsignd_ymm_ymm_ymmm256 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhrsw_mm_mmm64 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulhrsw_xmm_xmmm128 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilps_xmm_xmm_xmmm128 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilps_ymm_ymm_ymmm256 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilpd_xmm_xmm_xmmm128 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilpd_ymm_ymm_ymmm256 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vtestps_xmm_xmmm128 | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vtestps_ymm_ymmm256 | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vtestpd_xmm_xmmm128 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vtestpd_ymm_ymmm256 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pblendvb_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512 | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm | ((uint)0x11 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Eighth_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Eighth_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Eighth_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtph2ps_xmm_xmmm64 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtph2ps_ymm_xmmm128 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128 | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm | ((uint)0x13 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Blendvps_xmm_xmmm128 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Blendvpd_xmm_xmmm128 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermps_ymm_ymm_ymmm256 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Ptest_xmm_xmmm128 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vptest_xmm_xmmm128 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vptest_ymm_ymmm256 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vbroadcastss_xmm_xmmm32 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vbroadcastss_ymm_xmmm32 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vbroadcastsd_ymm_xmmm64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vbroadcastf128_ymm_m128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsb_mm_mmm64 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsb_xmm_xmmm128 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsb_xmm_xmmm128 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsb_ymm_ymmm256 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsb_xmm_k1z_xmmm128 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsb_ymm_k1z_ymmm256 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsb_zmm_k1z_zmmm512 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsw_mm_mmm64 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsw_xmm_xmmm128 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsw_xmm_xmmm128 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsw_ymm_ymmm256 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsw_xmm_k1z_xmmm128 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsw_ymm_k1z_ymmm256 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsw_zmm_k1z_zmmm512 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsd_mm_mmm64 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pabsd_xmm_xmmm128 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsd_xmm_xmmm128 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpabsd_ymm_ymmm256 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxbw_xmm_xmmm64 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbw_xmm_xmmm64 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbw_ymm_xmmm128 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovswb_xmmm64_k1z_xmm | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovswb_xmmm128_k1z_ymm | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovswb_ymmm256_k1z_zmm | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxbd_xmm_xmmm32 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbd_xmm_xmmm32 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbd_ymm_xmmm64 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxbq_xmm_xmmm16 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbq_xmm_xmmm16 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxbq_ymm_xmmm32 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Eighth_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Eighth_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Eighth_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Eighth_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Eighth_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Eighth_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxwd_xmm_xmmm64 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxwd_xmm_xmmm64 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxwd_ymm_xmmm128 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxwq_xmm_xmmm32 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxwq_xmm_xmmm32 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxwq_ymm_xmmm64 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128 | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm | ((uint)0x24 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovsxdq_xmm_xmmm64 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxdq_xmm_xmmm64 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovsxdq_ymm_xmmm128 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmb_k_k1_xmm_xmmm128 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmb_k_k1_ymm_ymmm256 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmb_k_k1_zmm_zmmm512 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmw_k_k1_xmm_xmmm128 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmw_k_k1_ymm_ymmm256 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmw_k_k1_zmm_zmmm512 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmb_k_k1_xmm_xmmm128 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmb_k_k1_ymm_ymmm256 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmb_k_k1_zmm_zmmm512 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmw_k_k1_xmm_xmmm128 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmw_k_k1_ymm_ymmm256 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmw_k_k1_zmm_zmmm512 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmd_k_k1_xmm_xmmm128b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmd_k_k1_ymm_ymmm256b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmd_k_k1_zmm_zmmm512b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmq_k_k1_xmm_xmmm128b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmq_k_k1_ymm_ymmm256b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestmq_k_k1_zmm_zmmm512b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmd_k_k1_xmm_xmmm128b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmd_k_k1_ymm_ymmm256b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmd_k_k1_zmm_zmmm512b32 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmq_k_k1_xmm_xmmm128b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmq_k_k1_ymm_ymmm256b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vptestnmq_k_k1_zmm_zmmm512b64 | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmuldq_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmuldq_xmm_xmm_xmmm128 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmuldq_ymm_ymm_ymmm256 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2b_xmm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2b_ymm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2b_zmm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2w_xmm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2w_ymm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2w_zmm_k | ((uint)0x28 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpeqq_xmm_xmmm128 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqq_k_k1_xmm_xmmm128b64 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqq_k_k1_ymm_ymmm256b64 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpeqq_k_k1_zmm_zmmm512b64 | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovb2m_k_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovb2m_k_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovb2m_k_zmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovw2m_k_xmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovw2m_k_ymm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovw2m_k_zmm | ((uint)0x29 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movntdqa_xmm_m128 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntdqa_xmm_m128 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmovntdqa_ymm_m256 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdqa_xmm_m128 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdqa_ymm_m256 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vmovntdqa_zmm_m512 | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmb2q_xmm_k | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmb2q_ymm_k | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmb2q_zmm_k | ((uint)0x2A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Packusdw_xmm_xmmm128 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackusdw_xmm_xmm_xmmm128 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpackusdw_ymm_ymm_ymmm256 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32 | ((uint)0x2B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovps_xmm_xmm_m128 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovps_ymm_ymm_m256 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x2C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovpd_xmm_xmm_m128 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovpd_ymm_ymm_m256 | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er | ((uint)0x2D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovps_m128_xmm_xmm | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovps_m256_ymm_ymm | ((uint)0x2E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovpd_m128_xmm_xmm | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmaskmovpd_m256_ymm_ymm | ((uint)0x2F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxbw_xmm_xmmm64 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbw_xmm_xmmm64 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbw_ymm_xmmm128 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovwb_xmmm64_k1z_xmm | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovwb_xmmm128_k1z_ymm | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovwb_ymmm256_k1z_zmm | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxbd_xmm_xmmm32 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbd_xmm_xmmm32 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbd_ymm_xmmm64 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdb_xmmm32_k1z_xmm | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdb_xmmm64_k1z_ymm | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdb_xmmm128_k1z_zmm | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxbq_xmm_xmmm16 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbq_xmm_xmmm16 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxbq_ymm_xmmm32 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Eighth_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Eighth_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Eighth_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqb_xmmm16_k1z_xmm | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Eighth_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqb_xmmm32_k1z_ymm | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Eighth_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqb_xmmm64_k1z_zmm | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Eighth_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxwd_xmm_xmmm64 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxwd_xmm_xmmm64 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxwd_ymm_xmmm128 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdw_xmmm64_k1z_xmm | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdw_xmmm128_k1z_ymm | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovdw_ymmm256_k1z_zmm | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxwq_xmm_xmmm32 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxwq_xmm_xmmm32 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxwq_ymm_xmmm64 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128 | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqw_xmmm32_k1z_xmm | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Quarter_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqw_xmmm64_k1z_ymm | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Quarter_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqw_xmmm128_k1z_zmm | ((uint)0x34 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Quarter_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmovzxdq_xmm_xmmm64 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxdq_xmm_xmmm64 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmovzxdq_ymm_xmmm128 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256 | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqd_xmmm64_k1z_xmm | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqd_xmmm128_k1z_ymm | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovqd_ymmm256_k1z_zmm | ((uint)0x35 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermd_ymm_ymm_ymmm256 | ((uint)0x36 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x36 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x36 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x36 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x36 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpgtq_xmm_xmmm128 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtq_k_k1_xmm_xmmm128b64 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtq_k_k1_ymm_ymmm256b64 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpgtq_k_k1_zmm_zmmm512b64 | ((uint)0x37 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminsb_xmm_xmmm128 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsb_xmm_xmm_xmmm128 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsb_ymm_ymm_ymmm256 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2d_xmm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2d_ymm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2d_zmm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2q_xmm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2q_ymm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovm2q_zmm_k | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminsd_xmm_xmmm128 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsd_xmm_xmm_xmmm128 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminsd_ymm_ymm_ymmm256 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovd2m_k_xmm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovd2m_k_ymm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovd2m_k_zmm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovq2m_k_xmm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovq2m_k_ymm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmovq2m_k_zmm | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.RZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminuw_xmm_xmmm128 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminuw_xmm_xmm_xmmm128 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminuw_ymm_ymm_ymmm256 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmw2d_xmm_k | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmw2d_ymm_k | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastmw2d_zmm_k | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.RK << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pminud_xmm_xmmm128 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminud_xmm_xmm_xmmm128 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpminud_ymm_ymm_ymmm256 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxsb_xmm_xmmm128 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsb_xmm_xmm_xmmm128 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsb_ymm_ymm_ymmm256 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512 | ((uint)0x3C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxsd_xmm_xmmm128 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsd_xmm_xmm_xmmm128 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxsd_ymm_ymm_ymmm256 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x3D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxuw_xmm_xmmm128 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxuw_xmm_xmm_xmmm128 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxuw_ymm_ymm_ymmm256 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmaxud_xmm_xmmm128 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxud_xmm_xmm_xmmm128 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaxud_ymm_ymm_ymmm256 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pmulld_xmm_xmmm128 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulld_xmm_xmm_xmmm128 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmulld_ymm_ymm_ymmm256 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Phminposuw_xmm_xmmm128 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vphminposuw_xmm_xmmm128 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlvd_xmm_xmm_xmmm128 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlvd_ymm_ymm_ymmm256 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlvq_xmm_xmm_xmmm128 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsrlvq_ymm_ymm_ymmm256 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x45 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsravd_xmm_xmm_xmmm128 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsravd_ymm_ymm_ymmm256 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllvd_xmm_xmm_xmmm128 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllvd_ymm_ymm_ymmm256 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllvq_xmm_xmm_xmmm128 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpsllvq_ymm_ymm_ymmm256 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x47 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64 | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32 | ((uint)0x4D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64 | ((uint)0x4D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64 | ((uint)0x4E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32 | ((uint)0x4F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64 | ((uint)0x4F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32 | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128 | ((uint)0x52 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128 | ((uint)0x53 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntb_xmm_k1z_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntb_ymm_k1z_ymmm256 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntb_zmm_k1z_zmmm512 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntw_xmm_k1z_xmmm128 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntw_ymm_k1z_ymmm256 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntw_zmm_k1z_zmmm512 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64 | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastd_xmm_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastd_ymm_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32 | ((uint)0x58 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastq_xmm_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastq_ymm_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64 | ((uint)0x59 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vbroadcasti128_ymm_m128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.M << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128 | ((uint)0x5A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256 | ((uint)0x5B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.M << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandb_xmm_k1z_xmmm128 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandb_ymm_k1z_ymmm256 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandb_zmm_k1z_zmmm512 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandw_xmm_k1z_xmmm128 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandw_ymm_k1z_ymmm256 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandw_zmm_k1z_zmmm512 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressb_xmmm128_k1z_xmm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressb_ymmm256_k1z_ymm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressb_zmmm512_k1z_zmm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressw_xmmm128_k1z_xmm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressw_ymmm256_k1z_ymm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressw_zmmm512_k1z_zmm | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x64 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x65 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.EVEX_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VKP1 | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF3 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512 | ((uint)0x75 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64 | ((uint)0x76 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x77 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastb_xmm_xmmm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastb_ymm_xmmm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastw_xmm_xmmm16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpbroadcastw_ymm_xmmm16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_xmm_k1z_r32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_ymm_k1z_r32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastb_zmm_k1z_r32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_xmm_k1z_r32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_ymm_k1z_r32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastw_zmm_k1z_r32 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_xmm_k1z_r32 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_ymm_k1z_r32 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastd_zmm_k1z_r32 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.Rd << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_xmm_k1z_r64 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.Rq << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_ymm_k1z_r64 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.Rq << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpbroadcastq_zmm_k1z_r64 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.None << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.Rq << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invept_r32_m128 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invept_r64_m128 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invvpid_r32_m128 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invvpid_r64_m128 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invpcid_r32_m128 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Invpcid_r64_m128 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandps_xmm_k1z_xmmm128 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandps_ymm_k1z_ymmm256 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandps_zmm_k1z_zmmm512 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandpd_xmm_k1z_xmmm128 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandpd_ymm_k1z_ymmm256 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexpandpd_zmm_k1z_zmmm512 | ((uint)0x88 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandd_xmm_k1z_xmmm128 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandd_ymm_k1z_ymmm256 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandd_zmm_k1z_zmmm512 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandq_xmm_k1z_xmmm128 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandq_ymm_k1z_ymmm256 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpexpandq_zmm_k1z_zmmm512 | ((uint)0x89 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompressps_xmmm128_k1z_xmm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompressps_ymmm256_k1z_ymm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompressps_zmmm512_k1z_zmm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompresspd_xmmm128_k1z_xmm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompresspd_ymmm256_k1z_ymm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcompresspd_zmmm512_k1z_zmm | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressd_xmmm128_k1z_xmm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressd_ymmm256_k1z_ymm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressd_zmmm512_k1z_zmm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressq_xmmm128_k1z_xmm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressq_ymmm256_k1z_ymm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcompressq_zmmm512_k1z_zmm | ((uint)0x8B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WZ | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovd_xmm_xmm_m128 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovd_ymm_ymm_m256 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovq_xmm_xmm_m128 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovq_ymm_ymm_m256 | ((uint)0x8C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.M << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512 | ((uint)0x8D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovd_m128_xmm_xmm | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovd_m256_ymm_ymm | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.M | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovq_m128_xmm_xmm | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.M | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpmaskmovq_m256_ymm_ymm | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.M | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.VY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufbitqmb_k_k1_xmm_xmmm128 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufbitqmb_k_k1_ymm_ymmm256 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshufbitqmb_k_k1_zmm_zmmm512 | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherdd_xmm_vm32x_xmm | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherdd_ymm_vm32y_ymm | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.VM32Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherdq_xmm_vm32x_xmm | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherdq_ymm_vm32x_ymm | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdd_xmm_k1_vm32x | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdd_ymm_k1_vm32y | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM32Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdd_zmm_k1_vm32z | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM32Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdq_xmm_k1_vm32x | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdq_ymm_k1_vm32x | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherdq_zmm_k1_vm32y | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM32Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherqd_xmm_vm64x_xmm | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherqd_xmm_vm64y_xmm | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherqq_xmm_vm64x_xmm | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpgatherqq_ymm_vm64y_ymm | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.VM64Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqd_xmm_k1_vm64x | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqd_xmm_k1_vm64y | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqd_ymm_k1_vm64z | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM64Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqq_xmm_k1_vm64x | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqq_ymm_k1_vm64y | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM64Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpgatherqq_zmm_k1_vm64z | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM64Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherdps_xmm_vm32x_xmm | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherdps_ymm_vm32y_ymm | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.VM32Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherdpd_xmm_vm32x_xmm | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherdpd_ymm_vm32x_ymm | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.VM32X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdps_xmm_k1_vm32x | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdps_ymm_k1_vm32y | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM32Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdps_zmm_k1_vm32z | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM32Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdpd_xmm_k1_vm32x | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdpd_ymm_k1_vm32x | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM32X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherdpd_zmm_k1_vm32y | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM32Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherqps_xmm_vm64x_xmm | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherqps_xmm_vm64y_xmm | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherqpd_xmm_vm64x_xmm | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.VM64X << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgatherqpd_ymm_vm64y_ymm | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.VM64Y << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.HY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqps_xmm_k1_vm64x | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqps_xmm_k1_vm64y | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqps_ymm_k1_vm64z | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM64Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqpd_xmm_k1_vm64x | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.VM64X << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqpd_ymm_k1_vm64y | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.VM64Y << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherqpd_zmm_k1_vm64z | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.VM64Z << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32 | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64 | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32 | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64 | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128 | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HXP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x9C << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32 | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64 | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er | ((uint)0x9D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32 | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64 | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdd_vm32x_k1_xmm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdd_vm32y_k1_ymm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdd_vm32z_k1_zmm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdq_vm32x_k1_xmm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdq_vm32x_k1_ymm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterdq_vm32y_k1_zmm | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqd_vm64x_k1_xmm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqd_vm64y_k1_xmm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Y | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqd_vm64z_k1_ymm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqq_vm64x_k1_xmm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqq_vm64y_k1_ymm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Y | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpscatterqq_vm64z_k1_zmm | ((uint)0xA1 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdps_vm32x_k1_xmm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdps_vm32y_k1_ymm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdps_vm32z_k1_zmm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdpd_vm32x_k1_xmm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdpd_vm32x_k1_ymm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32X | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterdpd_vm32y_k1_zmm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqps_vm64x_k1_xmm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqps_vm64y_k1_xmm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Y | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqps_vm64z_k1_ymm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqpd_vm64x_k1_xmm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64X | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqpd_vm64y_k1_ymm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Y | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterqpd_vm64z_k1_zmm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xA8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32 | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64 | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xA9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128 | ((uint)0xAB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.PF2 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_4X << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HXP3 << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.M << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xAC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32 | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64 | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xAD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64 | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xAF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64 | ((uint)0xB5 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xB8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32 | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64 | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xB9 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xBA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xBC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64 | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xBD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64 | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er | ((uint)0xBE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_er | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64 | ((uint)0xC4 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf0dps_vm32z_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf0dpd_vm32y_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf1dps_vm32z_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf1dpd_vm32y_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf0dps_vm32z_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group5 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf0dpd_vm32y_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group5 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf1dps_vm32z_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf1dpd_vm32y_k1 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM32Y,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf0qps_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf0qpd_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group1 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf1qps_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgatherpf1qpd_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group2 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf0qps_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group5 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf0qpd_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group5 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf1qps_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vscatterpf1qpd_vm64z_k1 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Group6 | (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VM64Z,
				(uint)EncFlags1.Legacy | (uint)Code.Sha1nexte_xmm_xmmm128 | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae | ((uint)0xC8 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha1msg1_xmm_xmmm128 | ((uint)0xC9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha1msg2_xmm_xmmm128 | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae | ((uint)0xCA << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha256rnds2_xmm_xmmm128 | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha256msg1_xmm_xmmm128 | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha256msg2_xmm_xmmm128 | ((uint)0xCD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae | ((uint)0xCD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae | ((uint)0xCD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Gf2p8mulb_xmm_xmmm128 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aesimc_xmm_xmmm128 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesimc_xmm_xmmm128 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aesenc_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesenc_xmm_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesenc_ymm_ymm_ymmm256 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenc_xmm_xmm_xmmm128 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenc_ymm_ymm_ymmm256 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenc_zmm_zmm_zmmm512 | ((uint)0xDC << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aesenclast_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesenclast_xmm_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesenclast_ymm_ymm_ymmm256 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenclast_xmm_xmm_xmmm128 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenclast_ymm_ymm_ymmm256 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesenclast_zmm_zmm_zmmm512 | ((uint)0xDD << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aesdec_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesdec_xmm_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesdec_ymm_ymm_ymmm256 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdec_xmm_xmm_xmmm128 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdec_ymm_ymm_ymmm256 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdec_zmm_zmm_zmmm512 | ((uint)0xDE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aesdeclast_xmm_xmmm128 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesdeclast_xmm_xmm_xmmm128 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaesdeclast_ymm_ymm_ymmm256 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F38 | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_r16_m16 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.Mw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_r32_m32 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Md << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_r64_m64 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Mq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Crc32_r32_rm8 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Crc32_r64_rm8 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eb << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_m16_r16 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o16, (uint)LegacyOpKind.Mw | ((uint)LegacyOpKind.Gw << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_m32_r32 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o32, (uint)LegacyOpKind.Md | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movbe_m64_r64 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Mq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Crc32_r32_rm16 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o16 | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ew << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Crc32_r32_rm32 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.o32 | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Crc32_r64_rm64 | ((uint)0xF1 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Andn_r32_r32_rm32 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Hd << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Andn_r64_r64_rm64 | ((uint)0xF2 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Hq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsr_r32_rm32 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group1 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Hd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsr_r64_rm64 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group1 | (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Hq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsmsk_r32_rm32 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Hd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsmsk_r64_rm64 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group2 | (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Hq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsi_r32_rm32 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group3 | (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Hd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Blsi_r64_rm64 | ((uint)0xF3 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Group3 | (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Hq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Bzhi_r32_rm32_r32 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hd << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Bzhi_r64_rm64_r64 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Wrussd_m32_r32 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Md | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Wrussq_m64_r64 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Mq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Pext_r32_r32_rm32 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Hd << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Pext_r64_r64_rm64 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Hq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Pdep_r32_r32_rm32 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Hd << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Pdep_r64_r64_rm64 | ((uint)0xF5 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Hq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Wrssd_m32_r32 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Md | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Wrssq_m64_r64 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Mq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adcx_r32_rm32 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adcx_r64_rm64 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adox_r32_rm32 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Adox_r64_rm64 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Mulx_r32_r32_rm32 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Hd << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Mulx_r64_r64_rm64 | ((uint)0xF6 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Hq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Bextr_r32_rm32_r32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hd << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Bextr_r64_rm64_r64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Shlx_r32_rm32_r32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hd << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Shlx_r64_rm64_r64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Sarx_r32_rm32_r32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hd << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Sarx_r64_rm64_r64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF3 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Shrx_r32_rm32_r32 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hd << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Shrx_r64_rm64_r64 | ((uint)0xF7 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F38 | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Hq << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdir64b_r16_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66 | (uint)LegacyFlags.a16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdir64b_r32_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdir64b_r64_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmds_r16_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.a16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmds_r32_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF3 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmds_r64_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF3, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmd_r16_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only1632 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.a16, (uint)LegacyOpKind.Gw | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmd_r32_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2 | (uint)LegacyFlags.a32, (uint)LegacyOpKind.Gd | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Enqcmd_r64_m512 | ((uint)0xF8 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PF2, (uint)LegacyOpKind.Gq | ((uint)LegacyOpKind.M << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdiri_m32_r32 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Md | ((uint)LegacyOpKind.Gd << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Movdiri_m64_r64 | ((uint)0xF9 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F38 | (uint)LegacyFlags.rexw | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.Mq | ((uint)LegacyOpKind.Gq << (int)LegacyFlags3.Op1Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermq_ymm_ymmm256_imm8 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8 | ((uint)0x00 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermpd_ymm_ymmm256_imm8 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x03 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilps_xmm_xmmm128_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilps_ymm_ymmm256_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8 | ((uint)0x04 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilpd_xmm_xmmm128_imm8 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermilpd_ymm_ymmm256_imm8 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8 | ((uint)0x05 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8 | ((uint)0x06 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Roundps_xmm_xmmm128_imm8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundps_xmm_xmmm128_imm8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundps_ymm_ymmm256_imm8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8 | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae | ((uint)0x08 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Roundpd_xmm_xmmm128_imm8 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundpd_xmm_xmmm128_imm8 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundpd_ymm_ymmm256_imm8 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.WY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8 | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae | ((uint)0x09 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Roundss_xmm_xmmm32_imm8 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8 | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae | ((uint)0x0A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Roundsd_xmm_xmmm64_imm8 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8 | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae | ((uint)0x0B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Blendps_xmm_xmmm128_imm8 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Blendpd_xmm_xmmm128_imm8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pblendw_xmm_xmmm128_imm8 | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8 | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8 | ((uint)0x0E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Palignr_mm_mmm64_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.P | ((uint)LegacyOpKind.Q << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Palignr_xmm_xmmm128_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8 | ((uint)0x0F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrb_r32m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RdMb | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrb_r64m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RqMb | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrb_r32m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.RdMb | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrb_r64m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.RqMb | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrb_r32m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RdMb | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrb_r64m8_xmm_imm8 | ((uint)0x14 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RqMb | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r32m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.RdMw | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrw_r64m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.RqMw | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrw_r32m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.RdMw | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrw_r64m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.RqMw | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrw_r32m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RdMw | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrw_r64m16_xmm_imm8 | ((uint)0x15 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.RqMw | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrd_rm32_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pextrq_rm64_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrd_rm32_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Ed | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpextrq_rm64_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Eq | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrd_rm32_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Ed | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpextrq_rm64_xmm_imm8 | ((uint)0x16 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Eq | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Extractps_rm32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.Ed | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Extractps_r64m32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.Eq | ((uint)LegacyOpKind.VX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vextractps_rm32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Ed | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vextractps_r64m32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.Eq | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractps_rm32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Ed | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractps_r64m32_xmm_imm8 | ((uint)0x17 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.Eq | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8 | ((uint)0x18 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vextractf128_xmmm128_ymm_imm8 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.WX | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8 | ((uint)0x19 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8 | ((uint)0x1A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8 | ((uint)0x1B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.WX | ((uint)VexOpKind.VX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.WX | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Half_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Half_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Half_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpud_k_k1_xmm_xmmm128b32_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpud_k_k1_ymm_ymmm256b32_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpud_k_k1_zmm_zmmm512b32_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuq_k_k1_xmm_xmmm128b64_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuq_k_k1_ymm_ymmm256b64_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuq_k_k1_zmm_zmmm512b64_imm8 | ((uint)0x1E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpd_k_k1_xmm_xmmm128b32_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpd_k_k1_ymm_ymmm256b32_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpd_k_k1_zmm_zmmm512b32_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpq_k_k1_xmm_xmmm128b64_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpq_k_k1_ymm_ymmm256b64_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpq_k_k1_zmm_zmmm512b64_imm8 | ((uint)0x1F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrb_xmm_r32m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RdMb << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrb_xmm_r64m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.RqMb << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RdMb << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.RqMb << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RdMb << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8 | ((uint)0x20 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_1 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.RqMb << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Insertps_xmm_xmmm32_imm8 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8 | ((uint)0x21 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrd_xmm_rm32_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Ed << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pinsrq_xmm_rm64_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.Eq << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ed << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Eq << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W0 | (uint)EvexFlags.EVEX_WIG32 | ((uint)TupleType.Tuple1_Scalar_4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ed << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8 | ((uint)0x22 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Only64 | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Tuple1_Scalar_8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Eq << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x23 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x25 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8 | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae | ((uint)0x26 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae | ((uint)0x27 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftrb_k_k_imm8 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftrw_k_k_imm8 | ((uint)0x30 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftrd_k_k_imm8 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftrq_k_k_imm8 | ((uint)0x31 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftlb_k_k_imm8 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftlw_k_k_imm8 | ((uint)0x32 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftld_k_k_imm8 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W0, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Kshiftlq_k_k_imm8 | ((uint)0x33 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_L0_W1, (uint)VexOpKind.VK | ((uint)VexOpKind.RK << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8 | ((uint)0x38 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vextracti128_xmmm128_ymm_imm8 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.WX | ((uint)VexOpKind.VY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8 | ((uint)0x39 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple2 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WX | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8 | ((uint)0x3A << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Tuple8 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8 | ((uint)0x3B << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Tuple4 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.WY | ((uint)EvexOpKind.VZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpub_k_k1_xmm_xmmm128_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpub_k_k1_ymm_ymmm256_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpub_k_k1_zmm_zmmm512_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuw_k_k1_xmm_xmmm128_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuw_k_k1_ymm_ymmm256_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpuw_k_k1_zmm_zmmm512_imm8 | ((uint)0x3E << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpb_k_k1_xmm_xmmm128_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpb_k_k1_ymm_ymmm256_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpb_k_k1_zmm_zmmm512_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpw_k_k1_xmm_xmmm128_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpw_k_k1_ymm_ymmm256_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpcmpw_k_k1_zmm_zmmm512_imm8 | ((uint)0x3F << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Dpps_xmm_xmmm128_imm8 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8 | ((uint)0x40 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Dppd_xmm_xmmm128_imm8 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8 | ((uint)0x41 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Mpsadbw_xmm_xmmm128_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8 | ((uint)0x42 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x43 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pclmulqdq_xmm_xmmm128_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_WIG, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_128_WIG | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_256_WIG | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8 | ((uint)0x44 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.EVEX_512_WIG | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8 | ((uint)0x46 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is5X << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is5Y << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is5X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2 | ((uint)0x48 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is5Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is5X << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is5Y << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is5X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2 | ((uint)0x49 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is5Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift) | ((uint)VexOpKind.I2 << (int)VexFlags3.Op4Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm | ((uint)0x4A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm | ((uint)0x4B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm | ((uint)0x4C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae | ((uint)0x50 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae | ((uint)0x51 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae | ((uint)0x54 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae | ((uint)0x55 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8 | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae | ((uint)0x56 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae | ((uint)0x57 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_sae | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256 | ((uint)0x5C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256 | ((uint)0x5D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256 | ((uint)0x5E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256 | ((uint)0x5F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpestrm_xmm_xmmm128_imm8 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpestrm64_xmm_xmmm128_imm8 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8 | ((uint)0x60 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpestri_xmm_xmmm128_imm8 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpestri64_xmm_xmmm128_imm8 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Only64 | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66 | (uint)LegacyFlags.rexw, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpestri_xmm_xmmm128_imm8 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8 | ((uint)0x61 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpistrm_xmm_xmmm128_imm8 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8 | ((uint)0x62 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Pcmpistri_xmm_xmmm128_imm8 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vpcmpistri_xmm_xmmm128_imm8 | ((uint)0x63 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclassps_k_k1_xmmm128b32_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclassps_k_k1_ymmm256b32_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclassps_k_k1_zmmm512b32_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclasspd_k_k1_xmmm128b64_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclasspd_k_k1_ymmm256b64_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclasspd_k_k1_zmmm512b64_imm8 | ((uint)0x66 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclassss_k_k1_xmmm32_imm8 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_LIG_W0 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vfpclasssd_k_k1_xmmm64_imm8 | ((uint)0x67 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1 | (uint)EvexFlags.EVEX_LIG_W1 | ((uint)TupleType.Tuple1_Scalar << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VK | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256 | ((uint)0x68 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256 | ((uint)0x69 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32 | ((uint)0x6A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64 | ((uint)0x6B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256 | ((uint)0x6C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256 | ((uint)0x6D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32 | ((uint)0x6E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64 | ((uint)0x6F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8 | ((uint)0x70 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x71 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_Mem_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_Mem_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8 | ((uint)0x72 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_Mem_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W0 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W0 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W0 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0x73 << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256 | ((uint)0x78 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256 | ((uint)0x79 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32 | ((uint)0x7A << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64 | ((uint)0x7B << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256 | ((uint)0x7C << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W0, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256 | ((uint)0x7D << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4Y << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32 | ((uint)0x7E << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W0, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64 | ((uint)0x7F << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_LIG_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Is4X << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Sha1rnds4_xmm_xmmm128_imm8 | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.PNP, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Gf2p8affineqb_xmm_xmmm128_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Gf2p8affineinvqb_xmm_xmmm128_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_W1, (uint)VexOpKind.VX | ((uint)VexOpKind.HX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WX << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_256_W1, (uint)VexOpKind.VY | ((uint)VexOpKind.HY << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.WY << (int)VexFlags3.Op2Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_128_W1 | ((uint)TupleType.Full_128 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VX | ((uint)EvexOpKind.HX << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WX << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_256_W1 | ((uint)TupleType.Full_256 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VY | ((uint)EvexOpKind.HY << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WY << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.EVEX | (uint)Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)EvexFlags.Encodable_Any | (uint)EvexFlags.Table0F3A | (uint)EvexFlags.P66 | (uint)EvexFlags.k1z_b | (uint)EvexFlags.EVEX_512_W1 | ((uint)TupleType.Full_512 << (int)EvexFlags.TupleTypeShift), (uint)EvexOpKind.VZ | ((uint)EvexOpKind.HZ << (int)EvexFlags3.Op1Shift) | ((uint)EvexOpKind.WZ << (int)EvexFlags3.Op2Shift) | ((uint)EvexOpKind.Ib << (int)EvexFlags3.Op3Shift),
				(uint)EncFlags1.Legacy | (uint)Code.Aeskeygenassist_xmm_xmmm128_imm8 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)LegacyFlags.Encodable_Any | (uint)LegacyFlags.Table0F3A | (uint)LegacyFlags.P66, (uint)LegacyOpKind.VX | ((uint)LegacyOpKind.WX << (int)LegacyFlags3.Op1Shift) | ((uint)LegacyOpKind.Ib << (int)LegacyFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8 | ((uint)0xDF << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.P66 | (uint)VexFlags.VEX_128_WIG, (uint)VexOpKind.VX | ((uint)VexOpKind.WX << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Rorx_r32_rm32_imm8 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Any | (uint)VexFlags.Table0F3A | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W0 | (uint)VexFlags.VEX_WIG32, (uint)VexOpKind.Gd | ((uint)VexOpKind.Ed << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.VEX | (uint)Code.VEX_Rorx_r64_rm64_imm8 | ((uint)0xF0 << (int)EncFlags1.OpCodeShift), (uint)VexFlags.Encodable_Only64 | (uint)VexFlags.Table0F3A | (uint)VexFlags.PF2 | (uint)VexFlags.VEX_LZ_W1, (uint)VexOpKind.Gq | ((uint)VexOpKind.Eq << (int)VexFlags3.Op1Shift) | ((uint)VexOpKind.Ib << (int)VexFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm | ((uint)0x85 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm | ((uint)0x8F << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm | ((uint)0x95 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm | ((uint)0x9F << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_256_W0, (uint)XopOpKind.VY | ((uint)XopOpKind.HY << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WY << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4Y << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128 | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256 | ((uint)0xA2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_256_W1, (uint)XopOpKind.VY | ((uint)XopOpKind.HY << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Is4Y << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.WY << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128 | ((uint)0xA3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Is4X << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotb_xmm_xmmm128_imm8 | ((uint)0xC0 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotw_xmm_xmmm128_imm8 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotd_xmm_xmmm128_imm8 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotq_xmm_xmmm128_imm8 | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8 | ((uint)0xCC << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8 | ((uint)0xCD << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8 | ((uint)0xCE << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8 | ((uint)0xCF << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8 | ((uint)0xEC << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8 | ((uint)0xED << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8 | ((uint)0xEE << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8 | ((uint)0xEF << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP8 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift) | ((uint)XopOpKind.Ib << (int)XopFlags3.Op3Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcfill_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcfill_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blsfill_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group2 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blsfill_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group2 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcs_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group3 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcs_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group3 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Tzmsk_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group4 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Tzmsk_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group4 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcic_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group5 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcic_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group5 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blsic_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group6 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blsic_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group6 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_T1mskc_r32_rm32 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group7 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_T1mskc_r64_rm64 | ((uint)0x01 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group7 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcmsk_r32_rm32 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blcmsk_r64_rm64 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blci_r32_rm32 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group6 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Blci_r64_rm64 | ((uint)0x02 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group6 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Llwpcb_r32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group0 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Rd,
				(uint)EncFlags1.XOP | (uint)Code.XOP_Llwpcb_r64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group0 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Rq,
				(uint)EncFlags1.XOP | (uint)Code.XOP_Slwpcb_r32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Rd,
				(uint)EncFlags1.XOP | (uint)Code.XOP_Slwpcb_r64 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Rq,
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczps_xmm_xmmm128 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczps_ymm_ymmm256 | ((uint)0x80 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_256_W0, (uint)XopOpKind.VY | ((uint)XopOpKind.WY << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczpd_xmm_xmmm128 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczpd_ymm_ymmm256 | ((uint)0x81 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_256_W0, (uint)XopOpKind.VY | ((uint)XopOpKind.WY << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczss_xmm_xmmm32 | ((uint)0x82 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vfrczsd_xmm_xmmm64 | ((uint)0x83 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotb_xmm_xmmm128_xmm | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotb_xmm_xmm_xmmm128 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotw_xmm_xmmm128_xmm | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotw_xmm_xmm_xmmm128 | ((uint)0x91 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotd_xmm_xmmm128_xmm | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotd_xmm_xmm_xmmm128 | ((uint)0x92 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotq_xmm_xmmm128_xmm | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vprotq_xmm_xmm_xmmm128 | ((uint)0x93 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlb_xmm_xmmm128_xmm | ((uint)0x94 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlb_xmm_xmm_xmmm128 | ((uint)0x94 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlw_xmm_xmmm128_xmm | ((uint)0x95 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlw_xmm_xmm_xmmm128 | ((uint)0x95 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshld_xmm_xmmm128_xmm | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshld_xmm_xmm_xmmm128 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlq_xmm_xmmm128_xmm | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshlq_xmm_xmm_xmmm128 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshab_xmm_xmmm128_xmm | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshab_xmm_xmm_xmmm128 | ((uint)0x98 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshaw_xmm_xmmm128_xmm | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshaw_xmm_xmm_xmmm128 | ((uint)0x99 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshad_xmm_xmmm128_xmm | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshad_xmm_xmm_xmmm128 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshaq_xmm_xmmm128_xmm | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.HX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vpshaq_xmm_xmm_xmmm128 | ((uint)0x9B << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W1, (uint)XopOpKind.VX | ((uint)XopOpKind.HX << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.WX << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddbw_xmm_xmmm128 | ((uint)0xC1 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddbd_xmm_xmmm128 | ((uint)0xC2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddbq_xmm_xmmm128 | ((uint)0xC3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddwd_xmm_xmmm128 | ((uint)0xC6 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddwq_xmm_xmmm128 | ((uint)0xC7 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphadddq_xmm_xmmm128 | ((uint)0xCB << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddubw_xmm_xmmm128 | ((uint)0xD1 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddubd_xmm_xmmm128 | ((uint)0xD2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddubq_xmm_xmmm128 | ((uint)0xD3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphadduwd_xmm_xmmm128 | ((uint)0xD6 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphadduwq_xmm_xmmm128 | ((uint)0xD7 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphaddudq_xmm_xmmm128 | ((uint)0xDB << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphsubbw_xmm_xmmm128 | ((uint)0xE1 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphsubwd_xmm_xmmm128 | ((uint)0xE2 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Vphsubdq_xmm_xmmm128 | ((uint)0xE3 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOP9 | (uint)XopFlags.XOP_128_W0, (uint)XopOpKind.VX | ((uint)XopOpKind.WX << (int)XopFlags3.Op1Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Bextr_r32_rm32_imm32 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Gd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Bextr_r64_rm64_imm32 | ((uint)0x10 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Gq | ((uint)XopOpKind.Eq << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Lwpins_r32_rm32_imm32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group0 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Lwpins_r64_rm32_imm32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group0 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Lwpval_r32_rm32_imm32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Any | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W0 | (uint)XopFlags.XOP_WIG32, (uint)XopOpKind.Hd | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.XOP | (uint)Code.XOP_Lwpval_r64_rm32_imm32 | ((uint)0x12 << (int)EncFlags1.OpCodeShift), (uint)XopFlags.Group1 | (uint)XopFlags.Encodable_Only64 | (uint)XopFlags.TableXOPA | (uint)XopFlags.XOP_L0_W1, (uint)XopOpKind.Hq | ((uint)XopOpKind.Ed << (int)XopFlags3.Op1Shift) | ((uint)XopOpKind.Id << (int)XopFlags3.Op2Shift),
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pi2fw_mm_mmm64 | ((uint)0x0C << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pi2fd_mm_mmm64 | ((uint)0x0D << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pf2iw_mm_mmm64 | ((uint)0x1C << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pf2id_mm_mmm64 | ((uint)0x1D << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrcpv_mm_mmm64 | ((uint)0x86 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrsqrtv_mm_mmm64 | ((uint)0x87 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfnacc_mm_mmm64 | ((uint)0x8A << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfpnacc_mm_mmm64 | ((uint)0x8E << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfcmpge_mm_mmm64 | ((uint)0x90 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfmin_mm_mmm64 | ((uint)0x94 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrcp_mm_mmm64 | ((uint)0x96 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrsqrt_mm_mmm64 | ((uint)0x97 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfsub_mm_mmm64 | ((uint)0x9A << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfadd_mm_mmm64 | ((uint)0x9E << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfcmpgt_mm_mmm64 | ((uint)0xA0 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfmax_mm_mmm64 | ((uint)0xA4 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrcpit1_mm_mmm64 | ((uint)0xA6 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrsqit1_mm_mmm64 | ((uint)0xA7 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfsubr_mm_mmm64 | ((uint)0xAA << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfacc_mm_mmm64 | ((uint)0xAE << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfcmpeq_mm_mmm64 | ((uint)0xB0 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfmul_mm_mmm64 | ((uint)0xB4 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pfrcpit2_mm_mmm64 | ((uint)0xB6 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pmulhrw_mm_mmm64 | ((uint)0xB7 << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pswapd_mm_mmm64 | ((uint)0xBB << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.D3NOW | (uint)Code.D3NOW_Pavgusb_mm_mmm64 | ((uint)0xBF << (int)EncFlags1.OpCodeShift), (uint)D3nowFlags.Encodable_Any, 0,
				(uint)EncFlags1.Legacy | (uint)Code.DeclareByte, 0, 0,
				(uint)EncFlags1.Legacy | (uint)Code.DeclareWord, 0, 0,
				(uint)EncFlags1.Legacy | (uint)Code.DeclareDword, 0, 0,
				(uint)EncFlags1.Legacy | (uint)Code.DeclareQword, 0, 0,
			};

		static OpCodeHandlers() {
			var info = GetData();
			var handlers = new OpCodeHandler[DecoderConstants.NumberOfCodeValues];
			int j = 0;
			for (int i = 0; i < info.Length; i += 3, j++) {
				uint dword1 = info[i];
				OpCodeHandler handler;
				switch ((EncodingKind)((dword1 >> (int)EncFlags1.EncodingShift) & (uint)EncFlags1.EncodingMask)) {
				case EncodingKind.Legacy:
					var code = (Code)(dword1 & (uint)EncFlags1.CodeMask);
					if (code == Code.INVALID)
						handler = new InvalidHandler(Code.INVALID);
					else if (code >= Code.DeclareByte)
						handler = new DeclareDataHandler(code);
					else
						handler = new LegacyHandler(dword1, info[i + 1], info[i + 2]);
					break;

				case EncodingKind.VEX:
					handler = new VexHandler(dword1, info[i + 1], info[i + 2]);
					break;

				case EncodingKind.EVEX:
					handler = new EvexHandler(dword1, info[i + 1], info[i + 2]);
					break;

				case EncodingKind.XOP:
					handler = new XopHandler(dword1, info[i + 1], info[i + 2]);
					break;

				case EncodingKind.D3NOW:
					handler = new D3nowHandler(dword1, info[i + 1], info[i + 2]);
					break;

				default:
					throw new InvalidOperationException();
				}
				handlers[j] = handler;
			}
			if (j != handlers.Length)
				throw new InvalidOperationException();
			Handlers = handlers;
		}
	}
}
#endif
