/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#if !NO_INSTR_INFO
namespace Iced.Intel.InstructionInfoInternal {
	static class InfoHandlers {
		internal static readonly OpAccess[] Op1Accesses = new OpAccess[(int)OpInfo1.Last] {
			OpAccess.None,
			OpAccess.Read,
			OpAccess.Read,
			OpAccess.Write,
			OpAccess.CondRead,
			OpAccess.ReadWrite,
			OpAccess.NoMemAccess,
		};

		internal static readonly OpAccess[] Op2Accesses = new OpAccess[(int)OpInfo2.Last] {
			OpAccess.None,
			OpAccess.Read,
			OpAccess.ReadWrite,
		};

		const uint Encoding_Legacy = (uint)EncodingKind.Legacy << (int)InfoFlags2.EncodingShift;
		const uint Encoding_VEX = (uint)EncodingKind.VEX << (int)InfoFlags2.EncodingShift;
		const uint Encoding_EVEX = (uint)EncodingKind.EVEX << (int)InfoFlags2.EncodingShift;
		const uint Encoding_XOP = (uint)EncodingKind.XOP << (int)InfoFlags2.EncodingShift;
		const uint Encoding_D3NOW = (uint)EncodingKind.D3NOW << (int)InfoFlags2.EncodingShift;

		const uint RflagsInfo_C_AC = (uint)RflagsInfo.C_AC << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_C_c = (uint)RflagsInfo.C_c << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_C_d = (uint)RflagsInfo.C_d << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_C_i = (uint)RflagsInfo.C_i << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_a_W_ac_U_opsz = (uint)RflagsInfo.R_a_W_ac_U_opsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_ac_W_acpsz_U_o = (uint)RflagsInfo.R_ac_W_acpsz_U_o << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_acopszid = (uint)RflagsInfo.R_acopszid << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_acopszidAC = (uint)RflagsInfo.R_acopszidAC << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_acpsz = (uint)RflagsInfo.R_acpsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_c = (uint)RflagsInfo.R_c << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_c_W_acopsz = (uint)RflagsInfo.R_c_W_acopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_c_W_c = (uint)RflagsInfo.R_c_W_c << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_c_W_co = (uint)RflagsInfo.R_c_W_co << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_cz = (uint)RflagsInfo.R_cz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_d = (uint)RflagsInfo.R_d << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_d_W_acopsz = (uint)RflagsInfo.R_d_W_acopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_o = (uint)RflagsInfo.R_o << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_o_W_o = (uint)RflagsInfo.R_o_W_o << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_os = (uint)RflagsInfo.R_os << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_osz = (uint)RflagsInfo.R_osz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_p = (uint)RflagsInfo.R_p << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_s = (uint)RflagsInfo.R_s << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_R_z = (uint)RflagsInfo.R_z << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_S_AC = (uint)RflagsInfo.S_AC << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_S_c = (uint)RflagsInfo.S_c << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_S_d = (uint)RflagsInfo.S_d << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_S_i = (uint)RflagsInfo.S_i << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_U_acopsz = (uint)RflagsInfo.U_acopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_acopsz = (uint)RflagsInfo.W_acopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_acopszid = (uint)RflagsInfo.W_acopszid << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_acopszidAC = (uint)RflagsInfo.W_acopszidAC << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_acpsz = (uint)RflagsInfo.W_acpsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_aopsz = (uint)RflagsInfo.W_aopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_c_C_aopsz = (uint)RflagsInfo.W_c_C_aopsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_c_U_aops = (uint)RflagsInfo.W_c_U_aops << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_c = (uint)RflagsInfo.W_c << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_co = (uint)RflagsInfo.W_co << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_co_U_apsz = (uint)RflagsInfo.W_co_U_apsz << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_copsz_U_a = (uint)RflagsInfo.W_copsz_U_a << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_cosz_C_ap = (uint)RflagsInfo.W_cosz_C_ap << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_cpz_C_aos = (uint)RflagsInfo.W_cpz_C_aos << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_cs_C_oz_U_ap = (uint)RflagsInfo.W_cs_C_oz_U_ap << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_csz_C_o_U_ap = (uint)RflagsInfo.W_csz_C_o_U_ap << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_cz_C_aops = (uint)RflagsInfo.W_cz_C_aops << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_cz_U_aops = (uint)RflagsInfo.W_cz_U_aops << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_psz_C_co_U_a = (uint)RflagsInfo.W_psz_C_co_U_a << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_psz_U_aco = (uint)RflagsInfo.W_psz_U_aco << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_sz_C_co_U_ap = (uint)RflagsInfo.W_sz_C_co_U_ap << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_z = (uint)RflagsInfo.W_z << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_z_C_acops = (uint)RflagsInfo.W_z_C_acops << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_z_C_co_U_aps = (uint)RflagsInfo.W_z_C_co_U_aps << (int)InfoFlags1.RflagsInfoShift;
		const uint RflagsInfo_W_z_U_acops = (uint)RflagsInfo.W_z_U_acops << (int)InfoFlags1.RflagsInfoShift;

		const uint SaveRestore = (uint)InfoFlags1.SaveRestore;
		const uint NoSegmentRead = (uint)InfoFlags1.NoSegmentRead;
		const uint StackInstruction = (uint)InfoFlags1.StackInstruction;
		const uint ProtectedMode = (uint)InfoFlags1.ProtectedMode;
		const uint Privileged = (uint)InfoFlags1.Privileged;
		const uint OpMaskRegReadWrite = (uint)InfoFlags2.OpMaskRegReadWrite;
		const uint AVX2_Check = (uint)InfoFlags2.AVX2_Check;

		const uint CodeInfo_Cdq = (uint)CodeInfo.Cdq << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cdqe = (uint)CodeInfo.Cdqe << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Clzero = (uint)CodeInfo.Clzero << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Clear_rflags = (uint)CodeInfo.Clear_rflags << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Clear_reg_regmem = (uint)CodeInfo.Clear_reg_regmem << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Clear_reg_reg_regmem = (uint)CodeInfo.Clear_reg_reg_regmem << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cmps = (uint)CodeInfo.Cmps << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cmpxchg = (uint)CodeInfo.Cmpxchg << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cmpxchg8b = (uint)CodeInfo.Cmpxchg8b << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cpuid = (uint)CodeInfo.Cpuid << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cqo = (uint)CodeInfo.Cqo << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cwd = (uint)CodeInfo.Cwd << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Cwde = (uint)CodeInfo.Cwde << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Div = (uint)CodeInfo.Div << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Encls = (uint)CodeInfo.Encls << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Enter = (uint)CodeInfo.Enter << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Ins = (uint)CodeInfo.Ins << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Invlpga = (uint)CodeInfo.Invlpga << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Iret = (uint)CodeInfo.Iret << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Jrcxz = (uint)CodeInfo.Jrcxz << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_KP1 = (uint)CodeInfo.KP1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Lahf = (uint)CodeInfo.Lahf << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Lds = (uint)CodeInfo.Lds << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Leave = (uint)CodeInfo.Leave << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Llwpcb = (uint)CodeInfo.Llwpcb << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Loadall386 = (uint)CodeInfo.Loadall386 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Lods = (uint)CodeInfo.Lods << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Loop = (uint)CodeInfo.Loop << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Maskmovq = (uint)CodeInfo.Maskmovq << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Monitor = (uint)CodeInfo.Monitor << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Montmul = (uint)CodeInfo.Montmul << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Movdir64b = (uint)CodeInfo.Movdir64b << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Movs = (uint)CodeInfo.Movs << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Mul = (uint)CodeInfo.Mul << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Mulx = (uint)CodeInfo.Mulx << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Mwait = (uint)CodeInfo.Mwait << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Mwaitx = (uint)CodeInfo.Mwaitx << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Outs = (uint)CodeInfo.Outs << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_PcmpXstrY = (uint)CodeInfo.PcmpXstrY << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pconfig = (uint)CodeInfo.Pconfig << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_2 = (uint)CodeInfo.Pop_2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_2_2 = (uint)CodeInfo.Pop_2_2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_4 = (uint)CodeInfo.Pop_4 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_4_4 = (uint)CodeInfo.Pop_4_4 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_8 = (uint)CodeInfo.Pop_8 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_8_8 = (uint)CodeInfo.Pop_8_8 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pop_Ev = (uint)CodeInfo.Pop_Ev << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Popa = (uint)CodeInfo.Popa << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_2 = (uint)CodeInfo.Push_2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_2_2 = (uint)CodeInfo.Push_2_2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_4 = (uint)CodeInfo.Push_4 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_4_4 = (uint)CodeInfo.Push_4_4 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_8 = (uint)CodeInfo.Push_8 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Push_8_8 = (uint)CodeInfo.Push_8_8 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Pusha = (uint)CodeInfo.Pusha << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_AL_W_AH = (uint)CodeInfo.R_AL_W_AH << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_AL_W_AX = (uint)CodeInfo.R_AL_W_AX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_CR0 = (uint)CodeInfo.R_CR0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_EAX_ECX_EDX = (uint)CodeInfo.R_EAX_ECX_EDX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_EAX_EDX = (uint)CodeInfo.R_EAX_EDX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_ECX_W_EAX_EDX = (uint)CodeInfo.R_ECX_W_EAX_EDX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_ST0 = (uint)CodeInfo.R_ST0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_ST0_R_ST1 = (uint)CodeInfo.R_ST0_R_ST1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_ST0_RW_ST1 = (uint)CodeInfo.R_ST0_RW_ST1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_ST0_ST1 = (uint)CodeInfo.R_ST0_ST1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_R_XMM0 = (uint)CodeInfo.R_XMM0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg8_Op0 = (uint)CodeInfo.Read_Reg8_Op0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg8_Op1 = (uint)CodeInfo.Read_Reg8_Op1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg8_Op2 = (uint)CodeInfo.Read_Reg8_Op2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg16_Op0 = (uint)CodeInfo.Read_Reg16_Op0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg16_Op1 = (uint)CodeInfo.Read_Reg16_Op1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Read_Reg16_Op2 = (uint)CodeInfo.Read_Reg16_Op2 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_RW_AL = (uint)CodeInfo.RW_AL << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_RW_AX = (uint)CodeInfo.RW_AX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_RW_CR0 = (uint)CodeInfo.RW_CR0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_RW_ST0 = (uint)CodeInfo.RW_ST0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_RW_ST0_R_ST1 = (uint)CodeInfo.RW_ST0_R_ST1 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Salc = (uint)CodeInfo.Salc << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Scas = (uint)CodeInfo.Scas << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Shift_Ib_MASK1FMOD9 = (uint)CodeInfo.Shift_Ib_MASK1FMOD9 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Shift_Ib_MASK1FMOD11 = (uint)CodeInfo.Shift_Ib_MASK1FMOD11 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Shift_Ib_MASK1F = (uint)CodeInfo.Shift_Ib_MASK1F << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Shift_Ib_MASK3F = (uint)CodeInfo.Shift_Ib_MASK3F << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Stos = (uint)CodeInfo.Stos << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Syscall = (uint)CodeInfo.Syscall << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Umonitor = (uint)CodeInfo.Umonitor << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Vmfunc = (uint)CodeInfo.Vmfunc << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Vmload = (uint)CodeInfo.Vmload << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Vzeroall = (uint)CodeInfo.Vzeroall << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_W_EAX_ECX_EDX = (uint)CodeInfo.W_EAX_ECX_EDX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_W_EAX_EDX = (uint)CodeInfo.W_EAX_EDX << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_W_ST0 = (uint)CodeInfo.W_ST0 << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Xbts = (uint)CodeInfo.Xbts << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Xcrypt = (uint)CodeInfo.Xcrypt << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Xsha = (uint)CodeInfo.Xsha << (int)InfoFlags1.CodeInfoShift;
		const uint CodeInfo_Xstore = (uint)CodeInfo.Xstore << (int)InfoFlags1.CodeInfoShift;

		const uint Op0_None = (uint)OpInfo0.None << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_Read = (uint)OpInfo0.Read << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_Write = (uint)OpInfo0.Write << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_WriteForce = (uint)OpInfo0.WriteForce << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_CondWrite = (uint)OpInfo0.CondWrite << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_CondWrite32_ReadWrite64 = (uint)OpInfo0.CondWrite32_ReadWrite64 << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_NoMemAccess = (uint)OpInfo0.NoMemAccess << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_ReadWrite = (uint)OpInfo0.ReadWrite << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_ReadCondWrite = (uint)OpInfo0.ReadCondWrite << (int)InfoFlags2.OpInfo0Shift;
		const uint Op0_WriteMem_ReadWriteReg = (uint)OpInfo0.WriteMem_ReadWriteReg << (int)InfoFlags2.OpInfo0Shift;

		const uint Op1_None = (uint)OpInfo1.None << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_Read = (uint)OpInfo1.Read << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_ReadP3 = (uint)OpInfo1.ReadP3 << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_Write = (uint)OpInfo1.Write << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_CondRead = (uint)OpInfo1.CondRead << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_ReadWrite = (uint)OpInfo1.ReadWrite << (int)InfoFlags2.OpInfo1Shift;
		const uint Op1_NoMemAccess = (uint)OpInfo1.NoMemAccess << (int)InfoFlags2.OpInfo1Shift;

		const uint Op2_None = (uint)OpInfo2.None << (int)InfoFlags2.OpInfo2Shift;
		const uint Op2_Read = (uint)OpInfo2.Read << (int)InfoFlags2.OpInfo2Shift;
		const uint Op2_ReadWrite = (uint)OpInfo2.ReadWrite << (int)InfoFlags2.OpInfo2Shift;

		const uint Op3_None = (uint)OpInfo3.None << (int)InfoFlags2.OpInfo3Shift;
		const uint Op3_Read = (uint)OpInfo3.Read << (int)InfoFlags2.OpInfo3Shift;

		const uint Op4_None = (uint)OpInfo4.None << (int)InfoFlags2.OpInfo4Shift;
		const uint Op4_Read = (uint)OpInfo4.Read << (int)InfoFlags2.OpInfo4Shift;

		const uint CPUID_INTEL8086 = (uint)CpuidFeatureInternal.INTEL8086 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL8086_ONLY = (uint)CpuidFeatureInternal.INTEL8086_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL186 = (uint)CpuidFeatureInternal.INTEL186 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL286 = (uint)CpuidFeatureInternal.INTEL286 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL286_ONLY = (uint)CpuidFeatureInternal.INTEL286_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL386 = (uint)CpuidFeatureInternal.INTEL386 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL386_ONLY = (uint)CpuidFeatureInternal.INTEL386_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL386_A0_ONLY = (uint)CpuidFeatureInternal.INTEL386_A0_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL486 = (uint)CpuidFeatureInternal.INTEL486 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL486_A_ONLY = (uint)CpuidFeatureInternal.INTEL486_A_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INTEL386_486_ONLY = (uint)CpuidFeatureInternal.INTEL386_486_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_IA64 = (uint)CpuidFeatureInternal.IA64 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_X64 = (uint)CpuidFeatureInternal.X64 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_ADX = (uint)CpuidFeatureInternal.ADX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AES = (uint)CpuidFeatureInternal.AES << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AES_and_AVX = (uint)CpuidFeatureInternal.AES_and_AVX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX = (uint)CpuidFeatureInternal.AVX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX_and_GFNI = (uint)CpuidFeatureInternal.AVX_and_GFNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX2 = (uint)CpuidFeatureInternal.AVX2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_4FMAPS = (uint)CpuidFeatureInternal.AVX512_4FMAPS << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_4VNNIW = (uint)CpuidFeatureInternal.AVX512_4VNNIW << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_BITALG = (uint)CpuidFeatureInternal.AVX512_BITALG << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_IFMA = (uint)CpuidFeatureInternal.AVX512_IFMA << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_VBMI = (uint)CpuidFeatureInternal.AVX512_VBMI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_VBMI2 = (uint)CpuidFeatureInternal.AVX512_VBMI2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_VNNI = (uint)CpuidFeatureInternal.AVX512_VNNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512_VPOPCNTDQ = (uint)CpuidFeatureInternal.AVX512_VPOPCNTDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512BW = (uint)CpuidFeatureInternal.AVX512BW << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512CD = (uint)CpuidFeatureInternal.AVX512CD << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512DQ = (uint)CpuidFeatureInternal.AVX512DQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512ER = (uint)CpuidFeatureInternal.AVX512ER << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512F = (uint)CpuidFeatureInternal.AVX512F << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512F_and_AVX512_VP2INTERSECT = (uint)CpuidFeatureInternal.AVX512F_and_AVX512_VP2INTERSECT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512F_and_GFNI = (uint)CpuidFeatureInternal.AVX512F_and_GFNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512F_and_VAES = (uint)CpuidFeatureInternal.AVX512F_and_VAES << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512F_and_VPCLMULQDQ = (uint)CpuidFeatureInternal.AVX512F_and_VPCLMULQDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512PF = (uint)CpuidFeatureInternal.AVX512PF << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_BF16 = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_BF16 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_BITALG = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_BITALG << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_IFMA = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_IFMA << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_VBMI = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_VBMI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_VBMI2 = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_VBMI2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_VNNI = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_VNNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_VP2INTERSECT = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_VP2INTERSECT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512_VPOPCNTDQ = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512_VPOPCNTDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512BW = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512BW << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512CD = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512CD << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512DQ = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512DQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_AVX512F = (uint)CpuidFeatureInternal.AVX512VL_and_AVX512F << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_GFNI = (uint)CpuidFeatureInternal.AVX512VL_and_GFNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_VAES = (uint)CpuidFeatureInternal.AVX512VL_and_VAES << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_AVX512VL_and_VPCLMULQDQ = (uint)CpuidFeatureInternal.AVX512VL_and_VPCLMULQDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_BMI1 = (uint)CpuidFeatureInternal.BMI1 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_BMI2 = (uint)CpuidFeatureInternal.BMI2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CET_IBT = (uint)CpuidFeatureInternal.CET_IBT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CET_SS = (uint)CpuidFeatureInternal.CET_SS << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CL1INVMB = (uint)CpuidFeatureInternal.CL1INVMB << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CLDEMOTE = (uint)CpuidFeatureInternal.CLDEMOTE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CLFLUSHOPT = (uint)CpuidFeatureInternal.CLFLUSHOPT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CLFSH = (uint)CpuidFeatureInternal.CLFSH << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CLWB = (uint)CpuidFeatureInternal.CLWB << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CLZERO = (uint)CpuidFeatureInternal.CLZERO << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CMOV = (uint)CpuidFeatureInternal.CMOV << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CMPXCHG16B = (uint)CpuidFeatureInternal.CMPXCHG16B << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CPUID = (uint)CpuidFeatureInternal.CPUID << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_CX8 = (uint)CpuidFeatureInternal.CX8 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_D3NOW = (uint)CpuidFeatureInternal.D3NOW << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_D3NOWEXT = (uint)CpuidFeatureInternal.D3NOWEXT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_ENCLV = (uint)CpuidFeatureInternal.ENCLV << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_ENQCMD = (uint)CpuidFeatureInternal.ENQCMD << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_F16C = (uint)CpuidFeatureInternal.F16C << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FMA = (uint)CpuidFeatureInternal.FMA << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FMA4 = (uint)CpuidFeatureInternal.FMA4 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU = (uint)CpuidFeatureInternal.FPU << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU_and_CMOV = (uint)CpuidFeatureInternal.FPU_and_CMOV << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU_and_SSE3 = (uint)CpuidFeatureInternal.FPU_and_SSE3 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU287 = (uint)CpuidFeatureInternal.FPU287 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU287XL_ONLY = (uint)CpuidFeatureInternal.FPU287XL_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU387 = (uint)CpuidFeatureInternal.FPU387 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FPU387SL_ONLY = (uint)CpuidFeatureInternal.FPU387SL_ONLY << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FSGSBASE = (uint)CpuidFeatureInternal.FSGSBASE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_FXSR = (uint)CpuidFeatureInternal.FXSR << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_GEODE = (uint)CpuidFeatureInternal.GEODE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_GFNI = (uint)CpuidFeatureInternal.GFNI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_HLE_or_RTM = (uint)CpuidFeatureInternal.HLE_or_RTM << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INVEPT = (uint)CpuidFeatureInternal.INVEPT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INVPCID = (uint)CpuidFeatureInternal.INVPCID << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_INVVPID = (uint)CpuidFeatureInternal.INVVPID << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_LWP = (uint)CpuidFeatureInternal.LWP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_LZCNT = (uint)CpuidFeatureInternal.LZCNT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MCOMMIT = (uint)CpuidFeatureInternal.MCOMMIT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MMX = (uint)CpuidFeatureInternal.MMX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MONITOR = (uint)CpuidFeatureInternal.MONITOR << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MONITORX = (uint)CpuidFeatureInternal.MONITORX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MOVBE = (uint)CpuidFeatureInternal.MOVBE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MOVDIR64B = (uint)CpuidFeatureInternal.MOVDIR64B << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MOVDIRI = (uint)CpuidFeatureInternal.MOVDIRI << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MPX = (uint)CpuidFeatureInternal.MPX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MSR = (uint)CpuidFeatureInternal.MSR << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_MULTIBYTENOP = (uint)CpuidFeatureInternal.MULTIBYTENOP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PADLOCK_ACE = (uint)CpuidFeatureInternal.PADLOCK_ACE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PADLOCK_PHE = (uint)CpuidFeatureInternal.PADLOCK_PHE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PADLOCK_PMM = (uint)CpuidFeatureInternal.PADLOCK_PMM << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PADLOCK_RNG = (uint)CpuidFeatureInternal.PADLOCK_RNG << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PAUSE = (uint)CpuidFeatureInternal.PAUSE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PCLMULQDQ = (uint)CpuidFeatureInternal.PCLMULQDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PCLMULQDQ_and_AVX = (uint)CpuidFeatureInternal.PCLMULQDQ_and_AVX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PCOMMIT = (uint)CpuidFeatureInternal.PCOMMIT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PCONFIG = (uint)CpuidFeatureInternal.PCONFIG << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PKU = (uint)CpuidFeatureInternal.PKU << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_POPCNT = (uint)CpuidFeatureInternal.POPCNT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PREFETCHW = (uint)CpuidFeatureInternal.PREFETCHW << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PREFETCHWT1 = (uint)CpuidFeatureInternal.PREFETCHWT1 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_PTWRITE = (uint)CpuidFeatureInternal.PTWRITE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDPID = (uint)CpuidFeatureInternal.RDPID << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDPMC = (uint)CpuidFeatureInternal.RDPMC << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDPRU = (uint)CpuidFeatureInternal.RDPRU << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDRAND = (uint)CpuidFeatureInternal.RDRAND << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDSEED = (uint)CpuidFeatureInternal.RDSEED << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RDTSCP = (uint)CpuidFeatureInternal.RDTSCP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_RTM = (uint)CpuidFeatureInternal.RTM << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SEP = (uint)CpuidFeatureInternal.SEP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SGX1 = (uint)CpuidFeatureInternal.SGX1 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SHA = (uint)CpuidFeatureInternal.SHA << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SKINIT_or_SVML = (uint)CpuidFeatureInternal.SKINIT_or_SVML << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SMAP = (uint)CpuidFeatureInternal.SMAP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SMX = (uint)CpuidFeatureInternal.SMX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE = (uint)CpuidFeatureInternal.SSE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE2 = (uint)CpuidFeatureInternal.SSE2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE3 = (uint)CpuidFeatureInternal.SSE3 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE4_1 = (uint)CpuidFeatureInternal.SSE4_1 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE4_2 = (uint)CpuidFeatureInternal.SSE4_2 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSE4A = (uint)CpuidFeatureInternal.SSE4A << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SSSE3 = (uint)CpuidFeatureInternal.SSSE3 << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SVM = (uint)CpuidFeatureInternal.SVM << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_SYSCALL = (uint)CpuidFeatureInternal.SYSCALL << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_TBM = (uint)CpuidFeatureInternal.TBM << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_TSC = (uint)CpuidFeatureInternal.TSC << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_VAES = (uint)CpuidFeatureInternal.VAES << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_VMX = (uint)CpuidFeatureInternal.VMX << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_VPCLMULQDQ = (uint)CpuidFeatureInternal.VPCLMULQDQ << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_WAITPKG = (uint)CpuidFeatureInternal.WAITPKG << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_WBNOINVD = (uint)CpuidFeatureInternal.WBNOINVD << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_XOP = (uint)CpuidFeatureInternal.XOP << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_XSAVE = (uint)CpuidFeatureInternal.XSAVE << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_XSAVEC = (uint)CpuidFeatureInternal.XSAVEC << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_XSAVEOPT = (uint)CpuidFeatureInternal.XSAVEOPT << (int)InfoFlags2.CpuidFeatureShift;
		const uint CPUID_XSAVES = (uint)CpuidFeatureInternal.XSAVES << (int)InfoFlags2.CpuidFeatureShift;

		const uint FlowControl_Unconditional = (uint)FlowControl.UnconditionalBranch << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_IndirectBranch = (uint)FlowControl.IndirectBranch << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_Conditional = (uint)FlowControl.ConditionalBranch << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_Return = (uint)FlowControl.Return << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_Call = (uint)FlowControl.Call << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_IndirectCall = (uint)FlowControl.IndirectCall << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_Interrupt = (uint)FlowControl.Interrupt << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_XbeginXabortXend = (uint)FlowControl.XbeginXabortXend << (int)InfoFlags2.FlowControlShift;
		const uint FlowControl_Exception = (uint)FlowControl.Exception << (int)InfoFlags2.FlowControlShift;

		internal static readonly uint[] Data = new uint[DecoderConstants.NumberOfCodeValues * 2] {
			(uint)Code.INVALID, FlowControl_Exception | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm8_r8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm16_r16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm32_r32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Add_rm64_r64 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Add_r8_rm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_r16_rm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_r32_rm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Add_r64_rm64 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Add_AL_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_AX_imm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_EAX_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Add_RAX_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushw_ES | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushd_ES | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popw_ES | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Popd_ES | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_rm8_r8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm16_r16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm32_r32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_rm64_r64 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Or_r8_rm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_r16_rm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_r32_rm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_r64_rm64 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Or_AL_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_AX_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_EAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_RAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushw_CS | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushd_CS | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popw_CS | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL8086_ONLY | Encoding_Legacy,
			(uint)Code.Adc_rm8_r8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm16_r16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm32_r32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Adc_rm64_r64 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Adc_r8_rm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_r16_rm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_r32_rm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Adc_r64_rm64 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Adc_AL_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_AX_imm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_EAX_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Adc_RAX_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushw_SS | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushd_SS | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popw_SS | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Popd_SS | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_rm8_r8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm16_r16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm32_r32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_rm64_r64 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sbb_r8_rm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_r16_rm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_r32_rm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_r64_rm64 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sbb_AL_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_AX_imm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_EAX_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_RAX_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushw_DS | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushd_DS | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popw_DS | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Popd_DS | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_rm8_r8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm16_r16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm32_r32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_rm64_r64 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.And_r8_rm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_r16_rm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_r32_rm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_r64_rm64 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.And_AL_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_AX_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_EAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_RAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Daa | CodeInfo_RW_AL | RflagsInfo_R_ac_W_acpsz_U_o, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm8_r8 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm16_r16 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm32_r32 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sub_rm64_r64 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sub_r8_rm8 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_r16_rm16 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_r32_rm32 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sub_r64_rm64 | RflagsInfo_W_acopsz | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sub_AL_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_AX_imm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_EAX_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sub_RAX_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Das | CodeInfo_RW_AL | RflagsInfo_R_ac_W_acpsz_U_o, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm8_r8 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm16_r16 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm32_r32 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xor_rm64_r64 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xor_r8_rm8 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_r16_rm16 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_r32_rm32 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xor_r64_rm64 | RflagsInfo_W_psz_C_co_U_a | CodeInfo_Clear_rflags, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xor_AL_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_AX_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_EAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xor_RAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Aaa | RflagsInfo_R_a_W_ac_U_opsz | CodeInfo_RW_AX, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm8_r8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm16_r16 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm32_r32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmp_rm64_r64 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmp_r8_rm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_r16_rm16 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_r32_rm32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmp_r64_rm64 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmp_AL_imm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_AX_imm16 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_EAX_imm32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmp_RAX_imm32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Aas | RflagsInfo_R_a_W_ac_U_opsz | CodeInfo_RW_AX, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Inc_r16 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Inc_r32 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Dec_r16 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Dec_r32 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Push_r16 | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Push_r32 | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Push_r64 | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pop_r16 | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pop_r32 | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pop_r64 | StackInstruction | CodeInfo_Pop_8, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushaw | StackInstruction | CodeInfo_Pusha, CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Pushad | StackInstruction | CodeInfo_Pusha, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popaw | StackInstruction | CodeInfo_Popa, CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Popad | StackInstruction | CodeInfo_Popa, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bound_r16_m1616, Op0_Read | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Bound_r32_m3232, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Arpl_rm16_r16 | RflagsInfo_W_z | ProtectedMode, Op0_ReadCondWrite | Op1_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Arpl_r32m16_r32 | RflagsInfo_W_z | ProtectedMode, Op0_ReadCondWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsxd_r16_rm16, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movsxd_r32_rm32, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movsxd_r64_rm32, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Push_imm16 | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Pushd_imm32 | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushq_imm32 | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Imul_r16_rm16_imm16 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Imul_r32_rm32_imm32 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Imul_r64_rm64_imm32 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pushw_imm8 | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Pushd_imm8 | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushq_imm8 | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Imul_r16_rm16_imm8 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Imul_r32_rm32_imm8 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Imul_r64_rm64_imm8 | RflagsInfo_W_co_U_apsz, Op0_Write | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Insb_m8_DX | Privileged | RflagsInfo_R_d | CodeInfo_Ins, Op0_Write | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Insw_m16_DX | Privileged | RflagsInfo_R_d | CodeInfo_Ins, Op0_Write | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Insd_m32_DX | Privileged | RflagsInfo_R_d | CodeInfo_Ins, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Outsb_DX_m8 | Privileged | RflagsInfo_R_d | CodeInfo_Outs, Op0_Read | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Outsw_DX_m16 | Privileged | RflagsInfo_R_d | CodeInfo_Outs, Op0_Read | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Outsd_DX_m32 | Privileged | RflagsInfo_R_d | CodeInfo_Outs, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jo_rel8_16 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jo_rel8_32 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jo_rel8_64 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jno_rel8_16 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jno_rel8_32 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jno_rel8_64 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jb_rel8_16 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jb_rel8_32 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jb_rel8_64 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jae_rel8_16 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jae_rel8_32 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jae_rel8_64 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Je_rel8_16 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Je_rel8_32 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Je_rel8_64 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jne_rel8_16 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jne_rel8_32 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jne_rel8_64 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jbe_rel8_16 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jbe_rel8_32 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jbe_rel8_64 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ja_rel8_16 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Ja_rel8_32 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ja_rel8_64 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Js_rel8_16 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Js_rel8_32 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Js_rel8_64 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jns_rel8_16 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jns_rel8_32 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jns_rel8_64 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jp_rel8_16 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jp_rel8_32 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jp_rel8_64 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jnp_rel8_16 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jnp_rel8_32 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jnp_rel8_64 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jl_rel8_16 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jl_rel8_32 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jl_rel8_64 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jge_rel8_16 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jge_rel8_32 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jge_rel8_64 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jle_rel8_16 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jle_rel8_32 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jle_rel8_64 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jg_rel8_16 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jg_rel8_32 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jg_rel8_64 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Add_rm8_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm8_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm8_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm8_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm8_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm8_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm8_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm8_imm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm16_imm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm32_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Add_rm64_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Or_rm16_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm32_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_rm64_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Adc_rm16_imm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm32_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Adc_rm64_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sbb_rm16_imm16 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm32_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_rm64_imm32 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.And_rm16_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm32_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_rm64_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sub_rm16_imm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm32_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sub_rm64_imm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xor_rm16_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm32_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xor_rm64_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmp_rm16_imm16 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm32_imm32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmp_rm64_imm32 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Add_rm8_imm8_82 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm8_imm8_82 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm8_imm8_82 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm8_imm8_82 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm8_imm8_82 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm8_imm8_82 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm8_imm8_82 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm8_imm8_82 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm16_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Add_rm32_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Add_rm64_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Or_rm16_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Or_rm32_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Or_rm64_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Adc_rm16_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Adc_rm32_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Adc_rm64_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sbb_rm16_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sbb_rm32_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sbb_rm64_imm8 | RflagsInfo_R_c_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.And_rm16_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.And_rm32_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.And_rm64_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sub_rm16_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sub_rm32_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sub_rm64_imm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xor_rm16_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xor_rm32_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xor_rm64_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmp_rm16_imm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmp_rm32_imm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmp_rm64_imm8 | RflagsInfo_W_acopsz, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Test_rm8_r8 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm16_r16 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm32_r32 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Test_rm64_r64 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xchg_rm8_r8, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xchg_rm16_r16, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xchg_rm32_r32, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xchg_rm64_r64, Op0_ReadWrite | Op1_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_rm8_r8, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_rm16_r16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_rm32_r32, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_rm64_r64, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_r8_rm8, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_r16_rm16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_r32_rm32, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_r64_rm64, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_rm16_Sreg, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_r32m16_Sreg, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_r64m16_Sreg, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lea_r16_m | NoSegmentRead, Op0_Write | Op1_NoMemAccess | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Lea_r32_m | NoSegmentRead, Op0_Write | Op1_NoMemAccess | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lea_r64_m | NoSegmentRead, Op0_Write | Op1_NoMemAccess | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_Sreg_rm16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_Sreg_r32m16 | CodeInfo_Read_Reg16_Op1, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_Sreg_r64m16 | CodeInfo_Read_Reg16_Op1, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pop_rm16 | StackInstruction | CodeInfo_Pop_Ev, Op0_Write | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pop_rm32 | StackInstruction | CodeInfo_Pop_Ev, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pop_rm64 | StackInstruction | CodeInfo_Pop_Ev, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Nopw, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Nopd, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Nopq, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xchg_r16_AX, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xchg_r32_EAX, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Xchg_r64_RAX, Op0_ReadWrite | Op1_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pause, CPUID_PAUSE | Encoding_Legacy,
			(uint)Code.Cbw | CodeInfo_R_AL_W_AH, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cwde | CodeInfo_Cwde, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cdqe | CodeInfo_Cdqe, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cwd | CodeInfo_Cwd, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cdq | CodeInfo_Cdq, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cqo | CodeInfo_Cqo, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Call_ptr1616 | StackInstruction | CodeInfo_Push_2_2, Op0_Read | FlowControl_Call | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Call_ptr1632 | StackInstruction | CodeInfo_Push_4_4, Op0_Read | FlowControl_Call | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Wait, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushfw | StackInstruction | RflagsInfo_R_acopszid | CodeInfo_Push_2, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Pushfd | StackInstruction | RflagsInfo_R_acopszidAC | CodeInfo_Push_4, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushfq | StackInstruction | RflagsInfo_R_acopszidAC | CodeInfo_Push_8, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Popfw | StackInstruction | RflagsInfo_W_acopszid | CodeInfo_Pop_2, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Popfd | StackInstruction | RflagsInfo_W_acopszidAC | CodeInfo_Pop_4, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popfq | StackInstruction | RflagsInfo_W_acopszidAC | CodeInfo_Pop_8, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sahf | CodeInfo_Lahf | RflagsInfo_W_acpsz, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Lahf | CodeInfo_Lahf | RflagsInfo_R_acpsz, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_AL_moffs8, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_AX_moffs16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_EAX_moffs32, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_RAX_moffs64, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_moffs8_AL, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_moffs16_AX, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_moffs32_EAX, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_moffs64_RAX, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movsb_m8_m8 | RflagsInfo_R_d | CodeInfo_Movs, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Movsw_m16_m16 | RflagsInfo_R_d | CodeInfo_Movs, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Movsd_m32_m32 | RflagsInfo_R_d | CodeInfo_Movs, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsq_m64_m64 | RflagsInfo_R_d | CodeInfo_Movs, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmpsb_m8_m8 | RflagsInfo_R_d_W_acopsz | CodeInfo_Cmps, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmpsw_m16_m16 | RflagsInfo_R_d_W_acopsz | CodeInfo_Cmps, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmpsd_m32_m32 | RflagsInfo_R_d_W_acopsz | CodeInfo_Cmps, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Cmpsq_m64_m64 | RflagsInfo_R_d_W_acopsz | CodeInfo_Cmps, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Test_AL_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_AX_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_EAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Test_RAX_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Stosb_m8_AL | RflagsInfo_R_d | CodeInfo_Stos, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Stosw_m16_AX | RflagsInfo_R_d | CodeInfo_Stos, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Stosd_m32_EAX | RflagsInfo_R_d | CodeInfo_Stos, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Stosq_m64_RAX | RflagsInfo_R_d | CodeInfo_Stos, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lodsb_AL_m8 | RflagsInfo_R_d | CodeInfo_Lods, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Lodsw_AX_m16 | RflagsInfo_R_d | CodeInfo_Lods, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Lodsd_EAX_m32 | RflagsInfo_R_d | CodeInfo_Lods, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lodsq_RAX_m64 | RflagsInfo_R_d | CodeInfo_Lods, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Scasb_AL_m8 | RflagsInfo_R_d_W_acopsz | CodeInfo_Scas, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Scasw_AX_m16 | RflagsInfo_R_d_W_acopsz | CodeInfo_Scas, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Scasd_EAX_m32 | RflagsInfo_R_d_W_acopsz | CodeInfo_Scas, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Scasq_RAX_m64 | RflagsInfo_R_d_W_acopsz | CodeInfo_Scas, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_r8_imm8, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_r16_imm16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_r32_imm32, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_r64_imm64, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rol_rm8_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Ror_rm8_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rcl_rm8_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1FMOD9, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rcr_rm8_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1FMOD9, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Shl_rm8_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Shr_rm8_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Sal_rm8_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Sar_rm8_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rol_rm16_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rol_rm32_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rol_rm64_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ror_rm16_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Ror_rm32_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ror_rm64_imm8 | RflagsInfo_W_co | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcl_rm16_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1FMOD11, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rcl_rm32_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcl_rm64_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcr_rm16_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1FMOD11, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Rcr_rm32_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcr_rm64_imm8 | RflagsInfo_R_c_W_co | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shl_rm16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Shl_rm32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shl_rm64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shr_rm16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Shr_rm32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shr_rm64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sal_rm16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Sal_rm32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sal_rm64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sar_rm16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Sar_rm32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sar_rm64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Retnw_imm16 | StackInstruction | CodeInfo_Pop_2, Op0_Read | FlowControl_Return | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Retnd_imm16 | StackInstruction | CodeInfo_Pop_4, Op0_Read | FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Retnq_imm16 | StackInstruction | CodeInfo_Pop_8, Op0_Read | FlowControl_Return | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Retnw | StackInstruction | CodeInfo_Pop_2, FlowControl_Return | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Retnd | StackInstruction | CodeInfo_Pop_4, FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Retnq | StackInstruction | CodeInfo_Pop_8, FlowControl_Return | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Les_r16_m1616 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Les_r32_m1632 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lds_r16_m1616 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Lds_r32_m1632 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_rm8_imm8, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xabort_imm8 | SaveRestore, Op0_Read | FlowControl_XbeginXabortXend | CPUID_RTM | Encoding_Legacy,
			(uint)Code.Mov_rm16_imm16, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mov_rm32_imm32, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_rm64_imm32, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xbegin_rel16, Op0_Read | FlowControl_XbeginXabortXend | CPUID_RTM | Encoding_Legacy,
			(uint)Code.Xbegin_rel32, Op0_Read | FlowControl_XbeginXabortXend | CPUID_RTM | Encoding_Legacy,
			(uint)Code.Enterw_imm16_imm8 | StackInstruction | CodeInfo_Enter, Op0_Read | Op1_Read | CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Enterd_imm16_imm8 | StackInstruction | CodeInfo_Enter, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Enterq_imm16_imm8 | StackInstruction | CodeInfo_Enter, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Leavew | StackInstruction | CodeInfo_Leave, CPUID_INTEL186 | Encoding_Legacy,
			(uint)Code.Leaved | StackInstruction | CodeInfo_Leave, CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Leaveq | StackInstruction | CodeInfo_Leave, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Retfw_imm16 | StackInstruction | CodeInfo_Pop_2_2, Op0_Read | FlowControl_Return | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Retfd_imm16 | StackInstruction | CodeInfo_Pop_4_4, Op0_Read | FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Retfq_imm16 | StackInstruction | CodeInfo_Pop_8_8, Op0_Read | FlowControl_Return | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Retfw | StackInstruction | CodeInfo_Pop_2_2, FlowControl_Return | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Retfd | StackInstruction | CodeInfo_Pop_4_4, FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Retfq | StackInstruction | CodeInfo_Pop_8_8, FlowControl_Return | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Int3, FlowControl_Interrupt | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Int_imm8, Op0_Read | FlowControl_Interrupt | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Into | RflagsInfo_R_o, FlowControl_Interrupt | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Iretw | StackInstruction | RflagsInfo_W_acopszid | CodeInfo_Iret, FlowControl_Return | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Iretd | StackInstruction | RflagsInfo_W_acopszidAC | CodeInfo_Iret, FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Iretq | StackInstruction | RflagsInfo_W_acopszidAC | CodeInfo_Iret, FlowControl_Return | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rol_rm8_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Ror_rm8_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcl_rm8_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcr_rm8_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shl_rm8_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shr_rm8_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sal_rm8_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sar_rm8_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rol_rm16_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rol_rm32_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rol_rm64_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ror_rm16_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Ror_rm32_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ror_rm64_1 | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcl_rm16_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcl_rm32_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcl_rm64_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcr_rm16_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcr_rm32_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcr_rm64_1 | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shl_rm16_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shl_rm32_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shl_rm64_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shr_rm16_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shr_rm32_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shr_rm64_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sal_rm16_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sal_rm32_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sal_rm64_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sar_rm16_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sar_rm32_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sar_rm64_1 | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rol_rm8_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Ror_rm8_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcl_rm8_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcr_rm8_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shl_rm8_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shr_rm8_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sal_rm8_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sar_rm8_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rol_rm16_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rol_rm32_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rol_rm64_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ror_rm16_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Ror_rm32_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ror_rm64_CL | RflagsInfo_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcl_rm16_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcl_rm32_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcl_rm64_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rcr_rm16_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Rcr_rm32_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Rcr_rm64_CL | RflagsInfo_R_c_W_co, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shl_rm16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shl_rm32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shl_rm64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shr_rm16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Shr_rm32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shr_rm64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sal_rm16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sal_rm32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sal_rm64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sar_rm16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sar_rm32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sar_rm64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Aam_imm8 | RflagsInfo_W_psz_U_aco | CodeInfo_R_AL_W_AX, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Aad_imm8 | RflagsInfo_W_psz_U_aco | CodeInfo_RW_AX, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Salc | RflagsInfo_R_c | CodeInfo_Salc, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Xlat_m8 | CodeInfo_RW_AL, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Fadd_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fmul_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcom_m32fp | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcomp_m32fp | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsub_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubr_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdiv_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivr_m32fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fadd_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fmul_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcom_st0_sti, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcomp_st0_sti, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsub_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubr_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdiv_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivr_st0_sti, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fld_m32fp | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fst_m32fp | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_m32fp | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldenv_m14byte, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldenv_m28byte, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldcw_m2byte, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnstenv_m14byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstenv_m14byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnstenv_m28byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstenv_m28byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnstcw_m2byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstcw_m2byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fld_st0_sti, Op0_Write | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fxch_st0_sti, Op0_ReadWrite | Op1_ReadWrite | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnop, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstpnce_sti | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fchs | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fabs | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ftst | CodeInfo_R_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fxam | CodeInfo_R_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fld1 | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldl2t | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldl2e | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldpi | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldlg2 | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldln2 | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fldz | CodeInfo_W_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.F2xm1 | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fyl2x | CodeInfo_R_ST0_RW_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fptan | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fpatan | CodeInfo_R_ST0_RW_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fxtract | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fprem1 | CodeInfo_RW_ST0_R_ST1, CPUID_FPU387 | Encoding_Legacy,
			(uint)Code.Fdecstp, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fincstp, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fprem | CodeInfo_RW_ST0_R_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fyl2xp1 | CodeInfo_R_ST0_RW_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsqrt | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsincos | CodeInfo_RW_ST0, CPUID_FPU387 | Encoding_Legacy,
			(uint)Code.Frndint | CodeInfo_RW_ST0, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fscale | CodeInfo_RW_ST0_R_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsin | CodeInfo_RW_ST0, CPUID_FPU387 | Encoding_Legacy,
			(uint)Code.Fcos | CodeInfo_RW_ST0, CPUID_FPU387 | Encoding_Legacy,
			(uint)Code.Fiadd_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fimul_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ficom_m32int | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ficomp_m32int | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisub_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisubr_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fidiv_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fidivr_m32int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcmovb_st0_sti | RflagsInfo_R_c, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmove_st0_sti | RflagsInfo_R_z, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmovbe_st0_sti | RflagsInfo_R_cz, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmovu_st0_sti | RflagsInfo_R_p, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fucompp | CodeInfo_R_ST0_R_ST1, CPUID_FPU387 | Encoding_Legacy,
			(uint)Code.Fild_m32int | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisttp_m32int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU_and_SSE3 | Encoding_Legacy,
			(uint)Code.Fist_m32int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fistp_m32int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fld_m80fp | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_m80fp | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcmovnb_st0_sti | RflagsInfo_R_c, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmovne_st0_sti | RflagsInfo_R_z, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmovnbe_st0_sti | RflagsInfo_R_cz, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcmovnu_st0_sti | RflagsInfo_R_p, Op0_CondWrite | Op1_CondRead | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fneni, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Feni, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fndisi, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdisi, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnclex, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fclex, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fninit, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Finit, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnsetpm, CPUID_FPU287 | Encoding_Legacy,
			(uint)Code.Fsetpm, CPUID_FPU287 | Encoding_Legacy,
			(uint)Code.Frstpm, CPUID_FPU287XL_ONLY | Encoding_Legacy,
			(uint)Code.Fucomi_st0_sti | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcomi_st0_sti | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fadd_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fmul_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcom_m64fp | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcomp_m64fp | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsub_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubr_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdiv_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivr_m64fp | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fadd_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fmul_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcom_st0_sti_DCD0, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcomp_st0_sti_DCD8, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubr_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsub_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivr_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdiv_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fld_m64fp | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisttp_m64int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU_and_SSE3 | Encoding_Legacy,
			(uint)Code.Fst_m64fp | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_m64fp | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Frstor_m94byte | SaveRestore, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Frstor_m108byte | SaveRestore, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnsave_m94byte | SaveRestore, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsave_m94byte | SaveRestore, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnsave_m108byte | SaveRestore, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsave_m108byte | SaveRestore, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnstsw_m2byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstsw_m2byte, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ffree_sti, Op0_None | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fxch_st0_sti_DDC8, Op0_ReadWrite | Op1_ReadWrite | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fst_sti | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_sti | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fucom_st0_sti, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fucomp_st0_sti, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fiadd_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fimul_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ficom_m16int | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ficomp_m16int | CodeInfo_R_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisub_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisubr_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fidiv_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fidivr_m16int | CodeInfo_RW_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Faddp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fmulp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcomp_st0_sti_DED0, Op0_Read | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fcompp | CodeInfo_R_ST0_ST1, CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubrp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fsubp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivrp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fdivp_sti_st0, Op0_ReadWrite | Op1_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fild_m16int | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fisttp_m16int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU_and_SSE3 | Encoding_Legacy,
			(uint)Code.Fist_m16int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fistp_m16int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fbld_m80bcd | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fild_m64int | CodeInfo_W_ST0, Op0_Read | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fbstp_m80bcd | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fistp_m64int | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Ffreep_sti, Op0_None | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fxch_st0_sti_DFC8, Op0_ReadWrite | Op1_ReadWrite | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_sti_DFD0 | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fstp_sti_DFD8 | CodeInfo_R_ST0, Op0_Write | CPUID_FPU | Encoding_Legacy,
			(uint)Code.Fnstsw_AX, Op0_Write | CPUID_FPU287 | Encoding_Legacy,
			(uint)Code.Fstsw_AX, Op0_Write | CPUID_FPU287 | Encoding_Legacy,
			(uint)Code.Fstdw_AX, Op0_Write | CPUID_FPU387SL_ONLY | Encoding_Legacy,
			(uint)Code.Fstsg_AX, Op0_Write | CPUID_FPU387SL_ONLY | Encoding_Legacy,
			(uint)Code.Fucomip_st0_sti | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Fcomip_st0_sti | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_FPU_and_CMOV | Encoding_Legacy,
			(uint)Code.Loopne_rel8_16_CX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_32_CX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_16_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_32_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_64_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_16_RCX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loopne_rel8_64_RCX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loope_rel8_16_CX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Loope_rel8_32_CX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loope_rel8_16_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loope_rel8_32_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loope_rel8_64_ECX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loope_rel8_16_RCX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loope_rel8_64_RCX | RflagsInfo_R_z | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loop_rel8_16_CX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Loop_rel8_32_CX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loop_rel8_16_ECX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loop_rel8_32_ECX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Loop_rel8_64_ECX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loop_rel8_16_RCX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loop_rel8_64_RCX | CodeInfo_Loop, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jcxz_rel8_16 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jcxz_rel8_32 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jecxz_rel8_16 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jecxz_rel8_32 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jecxz_rel8_64 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jrcxz_rel8_16 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jrcxz_rel8_64 | CodeInfo_Jrcxz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.In_AL_imm8 | Privileged, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.In_AX_imm8 | Privileged, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.In_EAX_imm8 | Privileged, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Out_imm8_AL | Privileged, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Out_imm8_AX | Privileged, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Out_imm8_EAX | Privileged, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Call_rel16 | StackInstruction | CodeInfo_Push_2, Op0_Read | FlowControl_Call | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Call_rel32_32 | StackInstruction | CodeInfo_Push_4, Op0_Read | FlowControl_Call | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Call_rel32_64 | StackInstruction | CodeInfo_Push_8, Op0_Read | FlowControl_Call | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmp_rel16, Op0_Read | FlowControl_Unconditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jmp_rel32_32, Op0_Read | FlowControl_Unconditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jmp_rel32_64, Op0_Read | FlowControl_Unconditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmp_ptr1616, Op0_Read | FlowControl_Unconditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jmp_ptr1632, Op0_Read | FlowControl_Unconditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jmp_rel8_16, Op0_Read | FlowControl_Unconditional | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jmp_rel8_32, Op0_Read | FlowControl_Unconditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jmp_rel8_64, Op0_Read | FlowControl_Unconditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.In_AL_DX | Privileged, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.In_AX_DX | Privileged, Op0_Write | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.In_EAX_DX | Privileged, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Out_DX_AL | Privileged, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Out_DX_AX | Privileged, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Out_DX_EAX | Privileged, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Int1, FlowControl_Interrupt | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Hlt | Privileged, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cmc | RflagsInfo_R_c_W_c, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm8_imm8 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm8_imm8_F6r1 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Not_rm8, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Neg_rm8 | RflagsInfo_W_acopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mul_rm8 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Imul_rm8 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Div_rm8 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Idiv_rm8 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm16_imm16 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm32_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Test_rm64_imm32 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Test_rm16_imm16_F7r1 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Test_rm32_imm32_F7r1 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Test_rm64_imm32_F7r1 | RflagsInfo_W_psz_C_co_U_a, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Not_rm16, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Not_rm32, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Not_rm64, Op0_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Neg_rm16 | RflagsInfo_W_acopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Neg_rm32 | RflagsInfo_W_acopsz, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Neg_rm64 | RflagsInfo_W_acopsz, Op0_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mul_rm16 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Mul_rm32 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mul_rm64 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Imul_rm16 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Imul_rm32 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Imul_rm64 | RflagsInfo_W_co_U_apsz | CodeInfo_Mul, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Div_rm16 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Div_rm32 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Div_rm64 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Idiv_rm16 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Idiv_rm32 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Idiv_rm64 | RflagsInfo_U_acopsz | CodeInfo_Div, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Clc | RflagsInfo_C_c, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Stc | RflagsInfo_S_c, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cli | Privileged | RflagsInfo_C_i, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Sti | Privileged | RflagsInfo_S_i, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Cld | RflagsInfo_C_d, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Std | RflagsInfo_S_d, CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Inc_rm8 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Dec_rm8 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Inc_rm16 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Inc_rm32 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Inc_rm64 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Dec_rm16 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Dec_rm32 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Dec_rm64 | RflagsInfo_W_aopsz, Op0_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Call_rm16 | StackInstruction | CodeInfo_Push_2, Op0_Read | FlowControl_IndirectCall | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Call_rm32 | StackInstruction | CodeInfo_Push_4, Op0_Read | FlowControl_IndirectCall | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Call_rm64 | StackInstruction | CodeInfo_Push_8, Op0_Read | FlowControl_IndirectCall | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Call_m1616 | StackInstruction | CodeInfo_Push_2_2, Op0_Read | FlowControl_IndirectCall | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Call_m1632 | StackInstruction | CodeInfo_Push_4_4, Op0_Read | FlowControl_IndirectCall | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Call_m1664 | StackInstruction | CodeInfo_Push_8_8, Op0_Read | FlowControl_IndirectCall | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmp_rm16, Op0_Read | FlowControl_IndirectBranch | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jmp_rm32, Op0_Read | FlowControl_IndirectBranch | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jmp_rm64, Op0_Read | FlowControl_IndirectBranch | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmp_m1616, Op0_Read | FlowControl_IndirectBranch | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Jmp_m1632, Op0_Read | FlowControl_IndirectBranch | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jmp_m1664, Op0_Read | FlowControl_IndirectBranch | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Push_rm16 | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.Push_rm32 | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Push_rm64 | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sldt_rm16 | ProtectedMode, Op0_Write | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Sldt_r32m16 | ProtectedMode, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sldt_r64m16 | ProtectedMode, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Str_rm16 | ProtectedMode, Op0_Write | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Str_r32m16 | ProtectedMode, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Str_r64m16 | ProtectedMode, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lldt_rm16 | Privileged | ProtectedMode, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lldt_r32m16 | Privileged | ProtectedMode | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lldt_r64m16 | Privileged | ProtectedMode | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ltr_rm16 | Privileged | ProtectedMode, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Ltr_r32m16 | Privileged | ProtectedMode | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ltr_r64m16 | Privileged | ProtectedMode | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Verr_rm16 | ProtectedMode | RflagsInfo_W_z, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Verr_r32m16 | ProtectedMode | RflagsInfo_W_z | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Verr_r64m16 | ProtectedMode | RflagsInfo_W_z | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Verw_rm16 | ProtectedMode | RflagsInfo_W_z, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Verw_r32m16 | ProtectedMode | RflagsInfo_W_z | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Verw_r64m16 | ProtectedMode | RflagsInfo_W_z | CodeInfo_Read_Reg16_Op0, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmpe_rm16, Op0_Read | FlowControl_IndirectBranch | CPUID_IA64 | Encoding_Legacy,
			(uint)Code.Jmpe_rm32, Op0_Read | FlowControl_IndirectBranch | CPUID_IA64 | Encoding_Legacy,
			(uint)Code.Sgdt_m1632_16, Op0_Write | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Sgdt_m1632, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sgdt_m1664, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Sidt_m1632_16, Op0_Write | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Sidt_m1632, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sidt_m1664, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lgdt_m1632_16 | Privileged, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lgdt_m1632 | Privileged, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lgdt_m1664 | Privileged, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lidt_m1632_16 | Privileged, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lidt_m1632 | Privileged, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lidt_m1664 | Privileged, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Smsw_rm16 | CodeInfo_R_CR0, Op0_Write | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Smsw_r32m16 | CodeInfo_R_CR0, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Smsw_r64m16 | CodeInfo_R_CR0, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rstorssp_m64 | RflagsInfo_W_c_C_aopsz | ProtectedMode, Op0_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Lmsw_rm16 | Privileged | CodeInfo_RW_CR0, Op0_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lmsw_r32m16 | Privileged | CodeInfo_RW_CR0, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lmsw_r64m16 | Privileged | CodeInfo_RW_CR0, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Invlpg_m | Privileged, Op0_NoMemAccess | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Enclv | Privileged | ProtectedMode | RflagsInfo_W_acopsz | CodeInfo_Encls, CPUID_ENCLV | Encoding_Legacy,
			(uint)Code.Vmcall | Privileged, FlowControl_Call | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmlaunch | Privileged | ProtectedMode | RflagsInfo_W_acopszidAC, FlowControl_Return | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmresume | Privileged | ProtectedMode | RflagsInfo_W_acopszidAC, FlowControl_Return | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmxoff | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, CPUID_VMX | Encoding_Legacy,
			(uint)Code.Pconfig | Privileged | RflagsInfo_W_z_C_acops | CodeInfo_Pconfig, CPUID_PCONFIG | Encoding_Legacy,
			(uint)Code.Monitorw | Privileged | CodeInfo_Monitor, CPUID_MONITOR | Encoding_Legacy,
			(uint)Code.Monitord | Privileged | CodeInfo_Monitor, CPUID_MONITOR | Encoding_Legacy,
			(uint)Code.Monitorq | Privileged | CodeInfo_Monitor, CPUID_MONITOR | Encoding_Legacy,
			(uint)Code.Mwait | Privileged | CodeInfo_Mwait, CPUID_MONITOR | Encoding_Legacy,
			(uint)Code.Clac | Privileged | RflagsInfo_C_AC, CPUID_SMAP | Encoding_Legacy,
			(uint)Code.Stac | Privileged | RflagsInfo_S_AC, CPUID_SMAP | Encoding_Legacy,
			(uint)Code.Encls | Privileged | ProtectedMode | RflagsInfo_W_acopsz | CodeInfo_Encls, CPUID_SGX1 | Encoding_Legacy,
			(uint)Code.Xgetbv | CodeInfo_R_ECX_W_EAX_EDX, CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Xsetbv | Privileged | CodeInfo_R_EAX_ECX_EDX, CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Vmfunc | RflagsInfo_W_acopsz | CodeInfo_Vmfunc, CPUID_VMX | Encoding_Legacy,
			(uint)Code.Xend, FlowControl_XbeginXabortXend | CPUID_RTM | Encoding_Legacy,
			(uint)Code.Xtest | RflagsInfo_W_z_C_acops, CPUID_HLE_or_RTM | Encoding_Legacy,
			(uint)Code.Enclu | ProtectedMode | RflagsInfo_W_acopsz | CodeInfo_Encls, CPUID_SGX1 | Encoding_Legacy,
			(uint)Code.Vmrunw | ProtectedMode | Privileged | SaveRestore | RflagsInfo_R_acopszidAC | CodeInfo_Vmload, CPUID_SVM | FlowControl_Return | Encoding_Legacy,
			(uint)Code.Vmrund | ProtectedMode | Privileged | SaveRestore | RflagsInfo_R_acopszidAC | CodeInfo_Vmload, CPUID_SVM | FlowControl_Return | Encoding_Legacy,
			(uint)Code.Vmrunq | ProtectedMode | Privileged | SaveRestore | RflagsInfo_R_acopszidAC | CodeInfo_Vmload, CPUID_SVM | FlowControl_Return | Encoding_Legacy,
			(uint)Code.Vmmcall, CPUID_SVM | FlowControl_Call | Encoding_Legacy,
			(uint)Code.Vmloadw | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Vmloadd | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Vmloadq | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Vmsavew | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Vmsaved | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Vmsaveq | ProtectedMode | Privileged | SaveRestore | CodeInfo_Vmload, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Stgi | ProtectedMode | Privileged, CPUID_SKINIT_or_SVML | Encoding_Legacy,
			(uint)Code.Clgi | ProtectedMode | Privileged, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Skinit | ProtectedMode | Privileged | SaveRestore | RflagsInfo_W_acopszidAC, CPUID_SKINIT_or_SVML | FlowControl_Return | Encoding_Legacy,
			(uint)Code.Invlpgaw | ProtectedMode | Privileged | CodeInfo_Invlpga, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Invlpgad | ProtectedMode | Privileged | CodeInfo_Invlpga, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Invlpgaq | ProtectedMode | Privileged | CodeInfo_Invlpga, CPUID_SVM | Encoding_Legacy,
			(uint)Code.Setssbsy | ProtectedMode | Privileged, CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Saveprevssp | RflagsInfo_R_c | ProtectedMode, CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Rdpkru | CodeInfo_R_ECX_W_EAX_EDX, CPUID_PKU | Encoding_Legacy,
			(uint)Code.Wrpkru | CodeInfo_R_EAX_ECX_EDX, CPUID_PKU | Encoding_Legacy,
			(uint)Code.Swapgs | Privileged, CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rdtscp | CodeInfo_W_EAX_ECX_EDX, CPUID_RDTSCP | Encoding_Legacy,
			(uint)Code.Monitorxw | CodeInfo_Monitor, CPUID_MONITORX | Encoding_Legacy,
			(uint)Code.Monitorxd | CodeInfo_Monitor, CPUID_MONITORX | Encoding_Legacy,
			(uint)Code.Monitorxq | CodeInfo_Monitor, CPUID_MONITORX | Encoding_Legacy,
			(uint)Code.Mcommit | RflagsInfo_W_c_C_aopsz, CPUID_MCOMMIT | Encoding_Legacy,
			(uint)Code.Mwaitx | CodeInfo_Mwaitx, CPUID_MONITORX | Encoding_Legacy,
			(uint)Code.Clzerow | CodeInfo_Clzero, CPUID_CLZERO | Encoding_Legacy,
			(uint)Code.Clzerod | CodeInfo_Clzero, CPUID_CLZERO | Encoding_Legacy,
			(uint)Code.Clzeroq | CodeInfo_Clzero, CPUID_CLZERO | Encoding_Legacy,
			(uint)Code.Rdpru | RflagsInfo_W_c_C_aopsz | CodeInfo_R_ECX_W_EAX_EDX, CPUID_RDPRU | Encoding_Legacy,
			(uint)Code.Lar_r16_rm16 | RflagsInfo_W_z | ProtectedMode, Op0_CondWrite | Op1_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lar_r32_r32m16 | RflagsInfo_W_z | ProtectedMode | CodeInfo_Read_Reg16_Op1, Op0_CondWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lar_r64_r64m16 | RflagsInfo_W_z | ProtectedMode | CodeInfo_Read_Reg16_Op1, Op0_CondWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lsl_r16_rm16 | RflagsInfo_W_z | ProtectedMode, Op0_CondWrite | Op1_Read | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Lsl_r32_r32m16 | RflagsInfo_W_z | ProtectedMode | CodeInfo_Read_Reg16_Op1, Op0_CondWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lsl_r64_r64m16 | RflagsInfo_W_z | ProtectedMode | CodeInfo_Read_Reg16_Op1, Op0_CondWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Loadallreset286 | Privileged | SaveRestore, CPUID_INTEL286_ONLY | Encoding_Legacy,
			(uint)Code.Loadall286 | Privileged | SaveRestore, CPUID_INTEL286_ONLY | Encoding_Legacy,
			(uint)Code.Syscall | RflagsInfo_R_acopszidAC | CodeInfo_Syscall, FlowControl_Call | CPUID_SYSCALL | Encoding_Legacy,
			(uint)Code.Clts | Privileged | CodeInfo_RW_CR0, CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Loadall386 | Privileged | SaveRestore | CodeInfo_Loadall386, CPUID_INTEL386_ONLY | Encoding_Legacy,
			(uint)Code.Sysretd | RflagsInfo_W_acopszidAC | Privileged | ProtectedMode | CodeInfo_Syscall, FlowControl_Return | CPUID_SYSCALL | Encoding_Legacy,
			(uint)Code.Sysretq | RflagsInfo_W_acopszidAC | Privileged | ProtectedMode | CodeInfo_Syscall, FlowControl_Return | CPUID_SYSCALL | Encoding_Legacy,
			(uint)Code.Invd | Privileged, CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Wbinvd | Privileged, CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Wbnoinvd | Privileged, CPUID_WBNOINVD | Encoding_Legacy,
			(uint)Code.Cl1invmb | Privileged, CPUID_CL1INVMB | Encoding_Legacy,
			(uint)Code.Ud2, FlowControl_Exception | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F0D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F0D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F0D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.Prefetch_m8, Op0_NoMemAccess | CPUID_PREFETCHW | Encoding_Legacy,
			(uint)Code.Prefetchw_m8, Op0_NoMemAccess | CPUID_PREFETCHW | Encoding_Legacy,
			(uint)Code.Prefetchwt1_m8, Op0_NoMemAccess | CPUID_PREFETCHWT1 | Encoding_Legacy,
			(uint)Code.Femms, CPUID_D3NOW | Encoding_Legacy,
			(uint)Code.Umov_rm8_r8, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Umov_rm16_r16, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Umov_rm32_r32, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Umov_r8_rm8, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Umov_r16_rm16, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Umov_r32_rm32, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Movups_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovups_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovups_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovups_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovups_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovups_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movupd_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovupd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovupd_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovupd_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovupd_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovupd_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movss_xmm_xmmm32, Op0_WriteMem_ReadWriteReg | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovss_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovss_xmm_m32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovss_xmm_k1z_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovss_xmm_k1z_m32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movsd_xmm_xmmm64, Op0_WriteMem_ReadWriteReg | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovsd_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovsd_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovsd_xmm_k1z_m64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movups_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovups_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovups_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovups_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovups_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovups_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movupd_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovupd_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovupd_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovupd_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovupd_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovupd_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movss_xmmm32_xmm, Op0_WriteMem_ReadWriteReg | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovss_xmm_xmm_xmm_0F11 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovss_m32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovss_m32_k1_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movsd_xmmm64_xmm, Op0_WriteMem_ReadWriteReg | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovsd_xmm_xmm_xmm_0F11 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovsd_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovsd_m64_k1_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movhlps_xmm_xmm, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Movlps_xmm_m64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovhlps_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovlps_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovhlps_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovlps_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movlpd_xmm_m64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovlpd_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovlpd_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movsldup_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vmovsldup_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovsldup_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovsldup_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovsldup_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovsldup_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movddup_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vmovddup_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovddup_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovddup_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovddup_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovddup_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movlps_m64_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovlps_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovlps_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movlpd_m64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovlpd_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovlpd_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Unpcklps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vunpcklps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vunpcklps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Unpcklpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vunpcklpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vunpcklpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Unpckhps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vunpckhps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vunpckhps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Unpckhpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vunpckhpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vunpckhpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movlhps_xmm_xmm, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovlhps_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovlhps_xmm_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movhps_xmm_m64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovhps_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovhps_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movhpd_xmm_m64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovhpd_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovhpd_xmm_xmm_m64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movshdup_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vmovshdup_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovshdup_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovshdup_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovshdup_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovshdup_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movhps_m64_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovhps_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovhps_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movhpd_m64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovhpd_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovhpd_m64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.ReservedNop_rm16_r16_0F18, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F18, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F18, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F19, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F19, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F19, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1A, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1A, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1A, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1B, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1B, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1B, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1C, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1C, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1C, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1D, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1E, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1E, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1E, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm16_r16_0F1F, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm32_r32_0F1F, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.ReservedNop_rm64_r64_0F1F, Op0_None | Op1_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.Prefetchnta_m8, Op0_NoMemAccess | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Prefetcht0_m8, Op0_NoMemAccess | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Prefetcht1_m8, Op0_NoMemAccess | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Prefetcht2_m8, Op0_NoMemAccess | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Bndldx_bnd_mib, Op0_Write | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmov_bnd_bndm64, Op0_Write | Op1_Read | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmov_bnd_bndm128, Op0_Write | Op1_Read | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcl_bnd_rm32 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcl_bnd_rm64 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcu_bnd_rm32 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcu_bnd_rm64 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndstx_mib_bnd, Op0_NoMemAccess | Op1_Read | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmov_bndm64_bnd, Op0_Write | Op1_Read | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmov_bndm128_bnd, Op0_Write | Op1_Read | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmk_bnd_m32 | NoSegmentRead, Op0_Write | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndmk_bnd_m64 | NoSegmentRead, Op0_Write | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcn_bnd_rm32 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Bndcn_bnd_rm64 | NoSegmentRead, Op0_Read | Op1_NoMemAccess | CPUID_MPX | Encoding_Legacy,
			(uint)Code.Cldemote_m8, Op0_NoMemAccess | CPUID_CLDEMOTE | Encoding_Legacy,
			(uint)Code.Rdsspd_r32, Op0_Write | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Rdsspq_r64, Op0_Write | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Endbr64, CPUID_CET_IBT | Encoding_Legacy,
			(uint)Code.Endbr32, CPUID_CET_IBT | Encoding_Legacy,
			(uint)Code.Nop_rm16, Op0_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.Nop_rm32, Op0_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.Nop_rm64, Op0_None | CPUID_MULTIBYTENOP | Encoding_Legacy,
			(uint)Code.Mov_r32_cr | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_r64_cr | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_r32_dr | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_r64_dr | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_cr_r32 | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_cr_r64 | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_dr_r32 | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Mov_dr_r64 | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Mov_r32_tr | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Mov_tr_r32 | Privileged | RflagsInfo_U_acopsz, Op0_Write | Op1_Read | CPUID_INTEL386_486_ONLY | Encoding_Legacy,
			(uint)Code.Movaps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovaps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovaps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovaps_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovaps_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovaps_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movapd_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovapd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovapd_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovapd_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovapd_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovapd_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movaps_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovaps_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovaps_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovaps_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovaps_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovaps_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movapd_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovapd_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovapd_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovapd_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovapd_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovapd_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtpi2ps_xmm_mmm64, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvtpi2pd_xmm_mmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvtsi2ss_xmm_rm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvtsi2ss_xmm_rm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vcvtsi2ss_xmm_xmm_rm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtsi2ss_xmm_xmm_rm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtsi2sd_xmm_rm32, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvtsi2sd_xmm_rm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtsi2sd_xmm_xmm_rm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtsi2sd_xmm_xmm_rm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movntps_m128_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovntps_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovntps_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovntps_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntps_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntps_m512_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movntpd_m128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovntpd_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovntpd_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovntpd_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntpd_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntpd_m512_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movntss_m32_xmm, Op0_Write | Op1_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.Movntsd_m64_xmm, Op0_Write | Op1_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.Cvttps2pi_mm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvttpd2pi_mm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvttss2si_r32_xmmm32, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvttss2si_r64_xmmm32, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vcvttss2si_r32_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvttss2si_r64_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvttss2si_r32_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttss2si_r64_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvttsd2si_r32_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvttsd2si_r64_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvttsd2si_r32_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvttsd2si_r64_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvttsd2si_r32_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttsd2si_r64_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtps2pi_mm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvtpd2pi_mm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvtss2si_r32_xmmm32, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Cvtss2si_r64_xmmm32, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vcvtss2si_r32_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtss2si_r64_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtss2si_r32_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtss2si_r64_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtsd2si_r32_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Cvtsd2si_r64_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtsd2si_r32_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtsd2si_r64_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtsd2si_r32_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtsd2si_r64_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Ucomiss_xmm_xmmm32 | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vucomiss_xmm_xmmm32 | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vucomiss_xmm_xmmm32_sae | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Ucomisd_xmm_xmmm64 | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vucomisd_xmm_xmmm64 | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vucomisd_xmm_xmmm64_sae | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Comiss_xmm_xmmm32 | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Comisd_xmm_xmmm64 | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcomiss_xmm_xmmm32 | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcomisd_xmm_xmmm64 | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcomiss_xmm_xmmm32_sae | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcomisd_xmm_xmmm64_sae | ProtectedMode | RflagsInfo_W_cpz_C_aos, Op0_Read | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Wrmsr | Privileged | CodeInfo_R_EAX_ECX_EDX, CPUID_MSR | Encoding_Legacy,
			(uint)Code.Rdtsc | CodeInfo_W_EAX_EDX, CPUID_TSC | Encoding_Legacy,
			(uint)Code.Rdmsr | Privileged | CodeInfo_R_ECX_W_EAX_EDX, CPUID_MSR | Encoding_Legacy,
			(uint)Code.Rdpmc | CodeInfo_R_ECX_W_EAX_EDX, CPUID_RDPMC | Encoding_Legacy,
			(uint)Code.Sysenter | RflagsInfo_C_i | CodeInfo_Syscall, FlowControl_Call | CPUID_SEP | Encoding_Legacy,
			(uint)Code.Sysexitd | Privileged | ProtectedMode | CodeInfo_Syscall, FlowControl_Return | CPUID_SEP | Encoding_Legacy,
			(uint)Code.Sysexitq | Privileged | ProtectedMode | CodeInfo_Syscall, FlowControl_Return | CPUID_SEP | Encoding_Legacy,
			(uint)Code.Getsec | SaveRestore, CPUID_SMX | Encoding_Legacy,
			(uint)Code.Cmovo_r16_rm16 | RflagsInfo_R_o, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovo_r32_rm32 | RflagsInfo_R_o, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovo_r64_rm64 | RflagsInfo_R_o, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovno_r16_rm16 | RflagsInfo_R_o, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovno_r32_rm32 | RflagsInfo_R_o, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovno_r64_rm64 | RflagsInfo_R_o, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovb_r16_rm16 | RflagsInfo_R_c, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovb_r32_rm32 | RflagsInfo_R_c, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovb_r64_rm64 | RflagsInfo_R_c, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovae_r16_rm16 | RflagsInfo_R_c, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovae_r32_rm32 | RflagsInfo_R_c, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovae_r64_rm64 | RflagsInfo_R_c, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmove_r16_rm16 | RflagsInfo_R_z, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmove_r32_rm32 | RflagsInfo_R_z, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmove_r64_rm64 | RflagsInfo_R_z, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovne_r16_rm16 | RflagsInfo_R_z, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovne_r32_rm32 | RflagsInfo_R_z, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovne_r64_rm64 | RflagsInfo_R_z, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovbe_r16_rm16 | RflagsInfo_R_cz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovbe_r32_rm32 | RflagsInfo_R_cz, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovbe_r64_rm64 | RflagsInfo_R_cz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmova_r16_rm16 | RflagsInfo_R_cz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmova_r32_rm32 | RflagsInfo_R_cz, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmova_r64_rm64 | RflagsInfo_R_cz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovs_r16_rm16 | RflagsInfo_R_s, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovs_r32_rm32 | RflagsInfo_R_s, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovs_r64_rm64 | RflagsInfo_R_s, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovns_r16_rm16 | RflagsInfo_R_s, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovns_r32_rm32 | RflagsInfo_R_s, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovns_r64_rm64 | RflagsInfo_R_s, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovp_r16_rm16 | RflagsInfo_R_p, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovp_r32_rm32 | RflagsInfo_R_p, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovp_r64_rm64 | RflagsInfo_R_p, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovnp_r16_rm16 | RflagsInfo_R_p, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovnp_r32_rm32 | RflagsInfo_R_p, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovnp_r64_rm64 | RflagsInfo_R_p, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovl_r16_rm16 | RflagsInfo_R_os, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovl_r32_rm32 | RflagsInfo_R_os, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovl_r64_rm64 | RflagsInfo_R_os, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovge_r16_rm16 | RflagsInfo_R_os, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovge_r32_rm32 | RflagsInfo_R_os, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovge_r64_rm64 | RflagsInfo_R_os, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovle_r16_rm16 | RflagsInfo_R_osz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovle_r32_rm32 | RflagsInfo_R_osz, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovle_r64_rm64 | RflagsInfo_R_osz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovg_r16_rm16 | RflagsInfo_R_osz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovg_r32_rm32 | RflagsInfo_R_osz, Op0_CondWrite32_ReadWrite64 | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.Cmovg_r64_rm64 | RflagsInfo_R_osz, Op0_CondWrite | Op1_Read | CPUID_CMOV | Encoding_Legacy,
			(uint)Code.VEX_Kandw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kandq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kandb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kandd_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kandnw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kandnq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kandnb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kandnd_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Knotw_k_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Knotq_k_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Knotb_k_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Knotd_k_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Korw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Korq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Korb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kord_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kxnorw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kxnorq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kxnorb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kxnord_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kxorw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kxorq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kxorb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kxord_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kaddw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kaddq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kaddb_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kaddd_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kunpckwd_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kunpckdq_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kunpckbw_k_k_k | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.Movmskps_r32_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Movmskps_r64_xmm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmovmskps_r32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskps_r64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskps_r32_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskps_r64_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Movmskpd_r32_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Movmskpd_r64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovmskpd_r32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskpd_r64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskpd_r32_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovmskpd_r64_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Sqrtps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vsqrtps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vsqrtps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Sqrtpd_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vsqrtpd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vsqrtpd_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Sqrtss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vsqrtss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Sqrtsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vsqrtsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Rsqrtps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vrsqrtps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vrsqrtps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Rsqrtss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vrsqrtss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Rcpps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vrcpps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vrcpps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Rcpss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vrcpss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Andps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vandps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vandps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Andpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vandpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vandpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Andnps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vandnps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vandnps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Andnpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vandnpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vandnpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Orps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vorps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vorps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Orpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vorpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vorpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Xorps_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vxorps_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vxorps_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Xorpd_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vxorpd_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vxorpd_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Addps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vaddps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaddps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Addpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vaddpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaddpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Addss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vaddss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Addsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vaddsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Mulps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmulps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmulps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Mulpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmulpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmulpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Mulss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmulss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Mulsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmulsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtps2pd_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtps2pd_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtps2pd_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtpd2ps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtpd2ps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtpd2ps_xmm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtss2sd_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtsd2ss_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtdq2ps_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtdq2ps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtdq2ps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Cvtps2dq_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtps2dq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtps2dq_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvttps2dq_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvttps2dq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvttps2dq_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Subps_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vsubps_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vsubps_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Subpd_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vsubpd_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vsubpd_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Subss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vsubss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Subsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vsubsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Minps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vminps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vminps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Minpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vminpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vminpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Minss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vminss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Minsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vminsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Divps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vdivps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vdivps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Divpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vdivpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vdivpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Divss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vdivss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Divsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vdivsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Maxps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmaxps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaxps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Maxpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmaxpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaxpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Maxss_xmm_xmmm32, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vmaxss_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Maxsd_xmm_xmmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmaxsd_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Punpcklbw_mm_mmm32, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpcklbw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpcklwd_mm_mmm32, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpcklwd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpckldq_mm_mmm32, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpckldq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpckldq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpckldq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Packsswb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Packsswb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpacksswb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpacksswb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpgtb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpgtb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpgtb_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtb_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtb_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpgtw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpgtw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpgtw_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtw_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtw_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpgtd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpgtd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpgtd_k_k1_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtd_k_k1_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtd_k_k1_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Packuswb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Packuswb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpackuswb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpackuswb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpckhbw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpckhbw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpckhwd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpckhwd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpckhdq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Punpckhdq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Packssdw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Packssdw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpackssdw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpackssdw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Punpcklqdq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Punpckhqdq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movd_mm_rm32, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movq_mm_rm64, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movd_xmm_rm32, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Movq_xmm_rm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovd_xmm_rm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovq_xmm_rm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovd_xmm_rm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovq_xmm_rm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movq_mm_mmm64, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movdqa_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovdqa_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovdqa_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movdqu_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovdqu_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovdqu_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pshufw_mm_mmm64_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pshufd_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpshufd_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpshufd_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pshufhw_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpshufhw_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpshufhw_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pshuflw_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpshuflw_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpshuflw_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psrlw_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrlw_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrlw_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrlw_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psraw_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psraw_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsraw_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsraw_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psllw_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psllw_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsllw_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsllw_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psrld_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrld_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrld_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrld_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psrad_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrad_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrad_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrad_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pslld_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pslld_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpslld_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpslld_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psrlq_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrlq_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrlq_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrlq_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psrldq_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrldq_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrldq_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrldq_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrldq_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrldq_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psllq_mm_imm8, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psllq_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsllq_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsllq_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pslldq_xmm_imm8, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpslldq_xmm_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpslldq_ymm_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpslldq_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpslldq_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpslldq_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpeqb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpeqb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpeqb_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqb_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqb_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpeqw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpeqw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpeqw_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqw_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqw_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpeqd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pcmpeqd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpeqd_k_k1_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqd_k_k1_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqd_k_k1_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Emms, CPUID_MMX | Encoding_Legacy,
			(uint)Code.VEX_Vzeroupper | ProtectedMode | CodeInfo_Vzeroall, CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vzeroall | ProtectedMode | CodeInfo_Vzeroall, CPUID_AVX | Encoding_VEX,
			(uint)Code.Vmread_rm32_r32 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Write | Op1_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmread_rm64_r64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Write | Op1_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Extrq_xmm_imm8_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttss2usi_r32_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttss2usi_r64_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Insertq_xmm_xmm_imm8_imm8, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Vmwrite_r32_rm32 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmwrite_r64_rm64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Extrq_xmm_xmm, Op0_ReadWrite | Op1_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtss2usi_r32_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtss2usi_r64_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Insertq_xmm_xmm, Op0_ReadWrite | Op1_Read | CPUID_SSE4A | Encoding_Legacy,
			(uint)Code.EVEX_Vcvtsd2usi_r32_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtsd2usi_r64_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Haddpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vhaddpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vhaddpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Haddps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vhaddps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vhaddps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Hsubpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vhsubpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vhsubpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Hsubps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vhsubps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vhsubps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Movd_rm32_mm, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movq_rm64_mm, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movd_rm32_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Movq_rm64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovd_rm32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovq_rm64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovd_rm32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovq_rm64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movq_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovq_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovq_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movq_mmm64_mm, Op0_Write | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Movdqa_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovdqa_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovdqa_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movdqu_xmmm128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovdqu_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovdqu_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Jo_rel16 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jo_rel32_32 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jo_rel32_64 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jno_rel16 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jno_rel32_32 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jno_rel32_64 | RflagsInfo_R_o, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jb_rel16 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jb_rel32_32 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jb_rel32_64 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jae_rel16 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jae_rel32_32 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jae_rel32_64 | RflagsInfo_R_c, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Je_rel16 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Je_rel32_32 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Je_rel32_64 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jne_rel16 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jne_rel32_32 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jne_rel32_64 | RflagsInfo_R_z, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jbe_rel16 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jbe_rel32_32 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jbe_rel32_64 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Ja_rel16 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ja_rel32_32 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ja_rel32_64 | RflagsInfo_R_cz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Js_rel16 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Js_rel32_32 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Js_rel32_64 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jns_rel16 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jns_rel32_32 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jns_rel32_64 | RflagsInfo_R_s, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jp_rel16 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jp_rel32_32 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jp_rel32_64 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jnp_rel16 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jnp_rel32_32 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jnp_rel32_64 | RflagsInfo_R_p, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jl_rel16 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jl_rel32_32 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jl_rel32_64 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jge_rel16 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jge_rel32_32 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jge_rel32_64 | RflagsInfo_R_os, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jle_rel16 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jle_rel32_32 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jle_rel32_64 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jg_rel16 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jg_rel32_32 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Jg_rel32_64 | RflagsInfo_R_osz, Op0_Read | FlowControl_Conditional | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Seto_rm8 | RflagsInfo_R_o, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setno_rm8 | RflagsInfo_R_o, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setb_rm8 | RflagsInfo_R_c, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setae_rm8 | RflagsInfo_R_c, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sete_rm8 | RflagsInfo_R_z, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setne_rm8 | RflagsInfo_R_z, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setbe_rm8 | RflagsInfo_R_cz, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Seta_rm8 | RflagsInfo_R_cz, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Sets_rm8 | RflagsInfo_R_s, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setns_rm8 | RflagsInfo_R_s, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setp_rm8 | RflagsInfo_R_p, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setnp_rm8 | RflagsInfo_R_p, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setl_rm8 | RflagsInfo_R_os, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setge_rm8 | RflagsInfo_R_os, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setle_rm8 | RflagsInfo_R_osz, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Setg_rm8 | RflagsInfo_R_osz, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.VEX_Kmovw_k_km16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kmovq_k_km64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovb_k_km8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kmovd_k_km32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovw_m16_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kmovq_m64_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovb_m8_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kmovd_m32_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovw_k_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kmovb_k_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kmovd_k_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovq_k_r64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovw_r32_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kmovb_r32_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kmovd_r32_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kmovq_r64_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kortestw_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kortestq_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kortestb_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kortestd_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Ktestw_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Ktestq_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Ktestb_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Ktestd_k_k | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.Pushw_FS | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushd_FS | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushq_FS | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Popw_FS | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popd_FS | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popq_FS | StackInstruction | CodeInfo_Pop_8, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cpuid | CodeInfo_Cpuid, CPUID_CPUID | Encoding_Legacy,
			(uint)Code.Bt_rm16_r16 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bt_rm32_r32 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bt_rm64_r64 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shld_rm16_r16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shld_rm32_r32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shld_rm64_r64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shld_rm16_r16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shld_rm32_r32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shld_rm64_r64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Montmul_16 | CodeInfo_Montmul, CPUID_PADLOCK_PMM | Encoding_Legacy,
			(uint)Code.Montmul_32 | CodeInfo_Montmul, CPUID_PADLOCK_PMM | Encoding_Legacy,
			(uint)Code.Montmul_64 | CodeInfo_Montmul, CPUID_PADLOCK_PMM | Encoding_Legacy,
			(uint)Code.Xsha1_16 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xsha1_32 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xsha1_64 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xsha256_16 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xsha256_32 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xsha256_64 | CodeInfo_Xsha, CPUID_PADLOCK_PHE | Encoding_Legacy,
			(uint)Code.Xbts_r16_rm16 | CodeInfo_Xbts, Op0_ReadWrite | Op1_Read | CPUID_INTEL386_A0_ONLY | Encoding_Legacy,
			(uint)Code.Xbts_r32_rm32 | CodeInfo_Xbts, Op0_ReadWrite | Op1_Read | CPUID_INTEL386_A0_ONLY | Encoding_Legacy,
			(uint)Code.Xstore_16 | CodeInfo_Xstore, CPUID_PADLOCK_RNG | Encoding_Legacy,
			(uint)Code.Xstore_32 | CodeInfo_Xstore, CPUID_PADLOCK_RNG | Encoding_Legacy,
			(uint)Code.Xstore_64 | CodeInfo_Xstore, CPUID_PADLOCK_RNG | Encoding_Legacy,
			(uint)Code.XcryptEcb_16 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptEcb_32 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptEcb_64 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCbc_16 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCbc_32 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCbc_64 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCtr_16 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCtr_32 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCtr_64 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCfb_16 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCfb_32 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptCfb_64 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptOfb_16 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptOfb_32 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.XcryptOfb_64 | CodeInfo_Xcrypt, CPUID_PADLOCK_ACE | Encoding_Legacy,
			(uint)Code.Ibts_rm16_r16 | CodeInfo_Xbts, Op0_ReadWrite | Op1_Read | CPUID_INTEL386_A0_ONLY | Encoding_Legacy,
			(uint)Code.Ibts_rm32_r32 | CodeInfo_Xbts, Op0_ReadWrite | Op1_Read | CPUID_INTEL386_A0_ONLY | Encoding_Legacy,
			(uint)Code.Cmpxchg486_rm8_r8 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486_A_ONLY | Encoding_Legacy,
			(uint)Code.Cmpxchg486_rm16_r16 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486_A_ONLY | Encoding_Legacy,
			(uint)Code.Cmpxchg486_rm32_r32 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486_A_ONLY | Encoding_Legacy,
			(uint)Code.Pushw_GS | StackInstruction | CodeInfo_Push_2, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushd_GS | StackInstruction | CodeInfo_Push_4, Op0_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Pushq_GS | StackInstruction | CodeInfo_Push_8, Op0_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Popw_GS | StackInstruction | CodeInfo_Pop_2, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popd_GS | StackInstruction | CodeInfo_Pop_4, Op0_Write | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Popq_GS | StackInstruction | CodeInfo_Pop_8, Op0_Write | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Rsm | Privileged | SaveRestore | RflagsInfo_W_acopszidAC, FlowControl_Return | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bts_rm16_r16 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bts_rm32_r32 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bts_rm64_r64 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shrd_rm16_r16_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shrd_rm32_r32_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK1F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shrd_rm64_r64_imm8 | RflagsInfo_W_copsz_U_a | CodeInfo_Shift_Ib_MASK3F, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Shrd_rm16_r16_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shrd_rm32_r32_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Shrd_rm64_r64_CL | RflagsInfo_W_copsz_U_a, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Fxsave_m512byte | SaveRestore, Op0_Write | CPUID_FXSR | Encoding_Legacy,
			(uint)Code.Fxsave64_m512byte | SaveRestore, Op0_Write | CPUID_FXSR | Encoding_Legacy,
			(uint)Code.Rdfsbase_r32, Op0_Write | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Rdfsbase_r64, Op0_Write | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Fxrstor_m512byte | SaveRestore, Op0_Read | CPUID_FXSR | Encoding_Legacy,
			(uint)Code.Fxrstor64_m512byte | SaveRestore, Op0_Read | CPUID_FXSR | Encoding_Legacy,
			(uint)Code.Rdgsbase_r32, Op0_Write | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Rdgsbase_r64, Op0_Write | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Ldmxcsr_m32, Op0_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Wrfsbase_r32, Op0_Read | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Wrfsbase_r64, Op0_Read | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.VEX_Vldmxcsr_m32 | ProtectedMode, Op0_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Stmxcsr_m32, Op0_Write | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Wrgsbase_r32, Op0_Read | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.Wrgsbase_r64, Op0_Read | CPUID_FSGSBASE | Encoding_Legacy,
			(uint)Code.VEX_Vstmxcsr_m32 | ProtectedMode, Op0_Write | CPUID_AVX | Encoding_VEX,
			(uint)Code.Xsave_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Xsave64_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Ptwrite_rm32, Op0_Read | CPUID_PTWRITE | Encoding_Legacy,
			(uint)Code.Ptwrite_rm64, Op0_Read | CPUID_PTWRITE | Encoding_Legacy,
			(uint)Code.Xrstor_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Xrstor64_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVE | Encoding_Legacy,
			(uint)Code.Incsspd_r32 | CodeInfo_Read_Reg8_Op0 | ProtectedMode, Op0_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Incsspq_r64 | CodeInfo_Read_Reg8_Op0 | ProtectedMode, Op0_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Xsaveopt_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVEOPT | Encoding_Legacy,
			(uint)Code.Xsaveopt64_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVEOPT | Encoding_Legacy,
			(uint)Code.Clwb_m8, Op0_NoMemAccess | CPUID_CLWB | Encoding_Legacy,
			(uint)Code.Tpause_r32 | RflagsInfo_W_c_C_aopsz | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Tpause_r64 | RflagsInfo_W_c_C_aopsz | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Clrssbsy_m64 | ProtectedMode | Privileged | RflagsInfo_W_c_C_aopsz, Op0_ReadWrite | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Umonitor_r16 | CodeInfo_Umonitor, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Umonitor_r32 | CodeInfo_Umonitor, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Umonitor_r64 | CodeInfo_Umonitor, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Umwait_r32 | RflagsInfo_W_c_C_aopsz | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Umwait_r64 | RflagsInfo_W_c_C_aopsz | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_WAITPKG | Encoding_Legacy,
			(uint)Code.Clflush_m8, Op0_NoMemAccess | CPUID_CLFSH | Encoding_Legacy,
			(uint)Code.Clflushopt_m8, Op0_NoMemAccess | CPUID_CLFLUSHOPT | Encoding_Legacy,
			(uint)Code.Lfence, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_E9, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_EA, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_EB, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_EC, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_ED, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_EE, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Lfence_EF, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F1, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F2, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F3, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F4, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F5, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F6, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Mfence_F7, CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Sfence, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_F9, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FA, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FB, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FC, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FD, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FE, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Sfence_FF, CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pcommit, CPUID_PCOMMIT | Encoding_Legacy,
			(uint)Code.Imul_r16_rm16 | RflagsInfo_W_co_U_apsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Imul_r32_rm32 | RflagsInfo_W_co_U_apsz, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Imul_r64_rm64 | RflagsInfo_W_co_U_apsz, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmpxchg_rm8_r8 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Cmpxchg_rm16_r16 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Cmpxchg_rm32_r32 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Cmpxchg_rm64_r64 | RflagsInfo_W_acopsz | CodeInfo_Cmpxchg, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lss_r16_m1616 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lss_r32_m1632 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lss_r64_m1664 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Btr_rm16_r16 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btr_rm32_r32 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btr_rm64_r64 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lfs_r16_m1616 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lfs_r32_m1632 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lfs_r64_m1664 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lgs_r16_m1616 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lgs_r32_m1632 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Lgs_r64_m1664 | CodeInfo_Lds, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movzx_r16_rm8, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movzx_r32_rm8, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movzx_r64_rm8, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movzx_r16_rm16, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movzx_r32_rm16, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movzx_r64_rm16, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Jmpe_disp16, Op0_Read | FlowControl_Unconditional | CPUID_IA64 | Encoding_Legacy,
			(uint)Code.Jmpe_disp32, Op0_Read | FlowControl_Unconditional | CPUID_IA64 | Encoding_Legacy,
			(uint)Code.Popcnt_r16_rm16 | RflagsInfo_W_z_C_acops, Op0_Write | Op1_Read | CPUID_POPCNT | Encoding_Legacy,
			(uint)Code.Popcnt_r32_rm32 | RflagsInfo_W_z_C_acops, Op0_Write | Op1_Read | CPUID_POPCNT | Encoding_Legacy,
			(uint)Code.Popcnt_r64_rm64 | RflagsInfo_W_z_C_acops, Op0_Write | Op1_Read | CPUID_POPCNT | Encoding_Legacy,
			(uint)Code.Ud1_r16_rm16, Op0_None | Op1_None | FlowControl_Exception | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Ud1_r32_rm32, Op0_None | Op1_None | FlowControl_Exception | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ud1_r64_rm64, Op0_None | Op1_None | FlowControl_Exception | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Bt_rm16_imm8 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bt_rm32_imm8 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bt_rm64_imm8 | RflagsInfo_W_c_U_aops, Op0_Read | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Bts_rm16_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bts_rm32_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bts_rm64_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Btr_rm16_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btr_rm32_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btr_rm64_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Btc_rm16_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btc_rm32_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btc_rm64_imm8 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Btc_rm16_r16 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btc_rm32_r32 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Btc_rm64_r64 | RflagsInfo_W_c_U_aops, Op0_ReadWrite | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Bsf_r16_rm16 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bsf_r32_rm32 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bsf_r64_rm64 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Tzcnt_r16_rm16 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_Legacy,
			(uint)Code.Tzcnt_r32_rm32 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_Legacy,
			(uint)Code.Tzcnt_r64_rm64 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_Legacy,
			(uint)Code.Bsr_r16_rm16 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bsr_r32_rm32 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Bsr_r64_rm64 | RflagsInfo_W_z_U_acops, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Lzcnt_r16_rm16 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_LZCNT | Encoding_Legacy,
			(uint)Code.Lzcnt_r32_rm32 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_LZCNT | Encoding_Legacy,
			(uint)Code.Lzcnt_r64_rm64 | RflagsInfo_W_cz_U_aops, Op0_Write | Op1_Read | CPUID_LZCNT | Encoding_Legacy,
			(uint)Code.Movsx_r16_rm8, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsx_r32_rm8, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsx_r64_rm8, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Movsx_r16_rm16, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsx_r32_rm16, Op0_Write | Op1_Read | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Movsx_r64_rm16, Op0_Write | Op1_Read | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Xadd_rm8_r8 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Xadd_rm16_r16 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Xadd_rm32_r32 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_ReadWrite | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Xadd_rm64_r64 | RflagsInfo_W_acopsz, Op0_ReadWrite | Op1_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Cmpps_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cmppd_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cmpss_xmm_xmmm32_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cmpsd_xmm_xmmm64_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movnti_m32_r32, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Movnti_m64_r64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pinsrw_mm_r32m16_imm8 | CodeInfo_Read_Reg16_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pinsrw_mm_r64m16_imm8 | CodeInfo_Read_Reg16_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pinsrw_xmm_r32m16_imm8 | CodeInfo_Read_Reg16_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pinsrw_xmm_r64m16_imm8 | CodeInfo_Read_Reg16_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8 | ProtectedMode | CodeInfo_Read_Reg16_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8 | ProtectedMode | CodeInfo_Read_Reg16_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8 | ProtectedMode | CodeInfo_Read_Reg16_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8 | ProtectedMode | CodeInfo_Read_Reg16_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pextrw_r32_mm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pextrw_r64_mm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pextrw_r32_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pextrw_r64_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpextrw_r32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpextrw_r64_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpextrw_r32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpextrw_r64_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Shufps_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Shufpd_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cmpxchg8b_m64 | RflagsInfo_W_z | CodeInfo_Cmpxchg8b, Op0_ReadWrite | CPUID_CX8 | Encoding_Legacy,
			(uint)Code.Cmpxchg16b_m128 | RflagsInfo_W_z | CodeInfo_Cmpxchg8b, Op0_ReadWrite | CPUID_CMPXCHG16B | Encoding_Legacy,
			(uint)Code.Xrstors_mem | Privileged | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVES | Encoding_Legacy,
			(uint)Code.Xrstors64_mem | Privileged | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVES | Encoding_Legacy,
			(uint)Code.Xsavec_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVEC | Encoding_Legacy,
			(uint)Code.Xsavec64_mem | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVEC | Encoding_Legacy,
			(uint)Code.Xsaves_mem | Privileged | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVES | Encoding_Legacy,
			(uint)Code.Xsaves64_mem | Privileged | SaveRestore | CodeInfo_R_EAX_EDX, Op0_Read | CPUID_XSAVES | Encoding_Legacy,
			(uint)Code.Vmptrld_m64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmclear_m64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Vmxon_m64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Rdrand_r16 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDRAND | Encoding_Legacy,
			(uint)Code.Rdrand_r32 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDRAND | Encoding_Legacy,
			(uint)Code.Rdrand_r64 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDRAND | Encoding_Legacy,
			(uint)Code.Vmptrst_m64 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Write | CPUID_VMX | Encoding_Legacy,
			(uint)Code.Rdseed_r16 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDSEED | Encoding_Legacy,
			(uint)Code.Rdseed_r32 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDSEED | Encoding_Legacy,
			(uint)Code.Rdseed_r64 | RflagsInfo_W_c_C_aopsz, Op0_Write | CPUID_RDSEED | Encoding_Legacy,
			(uint)Code.Rdpid_r32, Op0_Write | CPUID_RDPID | Encoding_Legacy,
			(uint)Code.Rdpid_r64, Op0_Write | CPUID_RDPID | Encoding_Legacy,
			(uint)Code.Bswap_r16, Op0_ReadWrite | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Bswap_r32, Op0_ReadWrite | CPUID_INTEL486 | Encoding_Legacy,
			(uint)Code.Bswap_r64, Op0_ReadWrite | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Addsubpd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vaddsubpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaddsubpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Addsubps_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vaddsubps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaddsubps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Psrlw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrlw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrlw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrlw_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psrld_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrld_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrld_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrld_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psrlq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrlq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrlq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrlq_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Paddq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmullw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pmullw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmullw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmullw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Movq_xmmm64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovq_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovq_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movq2dq_xmm_mm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Movdq2q_mm_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pmovmskb_r32_mm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pmovmskb_r64_mm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pmovmskb_r32_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pmovmskb_r64_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovmskb_r32_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovmskb_r64_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovmskb_r32_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmovmskb_r64_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Psubusb_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubusb_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubusb_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubusb_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psubusw_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubusw_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubusw_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubusw_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pminub_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pminub_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpminub_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminub_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pand_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pand_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpand_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpand_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Paddusb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddusb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddusb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddusb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Paddusw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddusw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddusw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddusw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmaxub_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pmaxub_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxub_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxub_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pandn_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pandn_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpandn_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpandn_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pavgb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pavgb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpavgb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpavgb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psraw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psraw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsraw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsraw_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psrad_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psrad_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsrad_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsrad_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pavgw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pavgw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpavgw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpavgw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmulhuw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pmulhuw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmulhuw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmulhuw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmulhw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pmulhw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmulhw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmulhw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Cvttpd2dq_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvttpd2dq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvttpd2dq_xmm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Cvtdq2pd_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtdq2pd_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtdq2pd_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Cvtpd2dq_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vcvtpd2dq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vcvtpd2dq_xmm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Movntq_m64_mm, Op0_Write | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Movntdq_m128_xmm, Op0_Write | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmovntdq_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovntdq_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vmovntdq_m128_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntdq_m256_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntdq_m512_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psubsb_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubsb_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubsb_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubsb_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psubsw_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubsw_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubsw_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubsw_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pminsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pminsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpminsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Por_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Por_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpor_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpor_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Paddsb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddsb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddsb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddsb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Paddsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmaxsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Pmaxsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pxor_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pxor_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpxor_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpxor_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Lddqu_xmm_m128, Op0_Write | Op1_Read | CPUID_SSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vlddqu_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vlddqu_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Psllw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psllw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsllw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsllw_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pslld_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pslld_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpslld_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpslld_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psllq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psllq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsllq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsllq_ymm_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmuludq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Pmuludq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmuludq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmuludq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmaddwd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Pmaddwd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaddwd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaddwd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psadbw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Psadbw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsadbw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsadbw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsadbw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsadbw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsadbw_zmm_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Maskmovq_rDI_mm_mm | CodeInfo_Maskmovq, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE | Encoding_Legacy,
			(uint)Code.Maskmovdqu_rDI_xmm_xmm | CodeInfo_Maskmovq, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vmaskmovdqu_rDI_xmm_xmm | ProtectedMode | CodeInfo_Maskmovq, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Psubb_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubb_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubb_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubb_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psubw_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubw_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubw_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubw_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Psubd_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Psubd_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubd_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubd_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Psubq_mm_mmm64 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.Psubq_xmm_xmmm128 | CodeInfo_Clear_reg_regmem, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpsubq_xmm_xmm_xmmm128 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsubq_ymm_ymm_ymmm256 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode | CodeInfo_Clear_reg_reg_regmem, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Paddb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Paddw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Paddd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_MMX | Encoding_Legacy,
			(uint)Code.Paddd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE2 | Encoding_Legacy,
			(uint)Code.VEX_Vpaddd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpaddd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Ud0_r16_rm16, Op0_None | Op1_None | FlowControl_Exception | CPUID_INTEL286 | Encoding_Legacy,
			(uint)Code.Ud0_r32_rm32, Op0_None | Op1_None | FlowControl_Exception | CPUID_INTEL386 | Encoding_Legacy,
			(uint)Code.Ud0_r64_rm64, Op0_None | Op1_None | FlowControl_Exception | CPUID_X64 | Encoding_Legacy,
			(uint)Code.Pshufb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pshufb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpshufb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpshufb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Phaddw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phaddw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphaddw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphaddw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Phaddd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phaddd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphaddd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphaddd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Phaddsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phaddsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphaddsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphaddsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Pmaddubsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pmaddubsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Phsubw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phsubw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphsubw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphsubw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Phsubd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phsubd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphsubd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphsubd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Phsubsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Phsubsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vphsubsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vphsubsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Psignb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Psignb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpsignb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsignb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Psignw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Psignw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpsignw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsignw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Psignd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Psignd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpsignd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpsignd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Pmulhrsw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pmulhrsw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.VEX_Vpermilps_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpermilps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermilpd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpermilpd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vtestps_xmm_xmmm128 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vtestps_ymm_ymmm256 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vtestpd_xmm_xmmm128 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vtestpd_ymm_ymmm256 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pblendvb_xmm_xmmm128 | CodeInfo_R_XMM0, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vcvtph2ps_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_F16C | Encoding_VEX,
			(uint)Code.VEX_Vcvtph2ps_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_F16C | Encoding_VEX,
			(uint)Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Blendvps_xmm_xmmm128 | CodeInfo_R_XMM0, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Blendvpd_xmm_xmmm128 | CodeInfo_R_XMM0, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermps_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Ptest_xmm_xmmm128 | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vptest_xmm_xmmm128 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vptest_ymm_ymmm256 | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vbroadcastss_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | AVX2_Check | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vbroadcastss_ymm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | AVX2_Check | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vbroadcastsd_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | AVX2_Check | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vbroadcastf128_ymm_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pabsb_mm_mmm64, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pabsb_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpabsb_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpabsb_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpabsb_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsb_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsb_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pabsw_mm_mmm64, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pabsw_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpabsw_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpabsw_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpabsw_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsw_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsw_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pabsd_mm_mmm64, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Pabsd_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpabsd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpabsd_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovsxbw_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxbw_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxbw_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovswb_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovswb_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovswb_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmovsxbd_xmm_xmmm32, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxbd_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxbd_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovsxbq_xmm_xmmm16, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxbq_xmm_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxbq_ymm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovsxwd_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxwd_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxwd_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovsxwq_xmm_xmmm32, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxwq_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxwq_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovsxdq_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovsxdq_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovsxdq_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmb_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmb_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmb_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmw_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmw_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmw_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmb_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmb_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmb_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmw_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmw_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmw_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmd_k_k1_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmd_k_k1_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmd_k_k1_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmq_k_k1_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmq_k_k1_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestmq_k_k1_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmd_k_k1_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmd_k_k1_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmd_k_k1_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmq_k_k1_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmq_k_k1_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vptestnmq_k_k1_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmuldq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmuldq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmuldq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2b_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2b_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2b_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2w_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2w_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2w_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pcmpeqq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpeqq_k_k1_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqq_k_k1_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpeqq_k_k1_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovb2m_k_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovb2m_k_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovb2m_k_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovw2m_k_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovw2m_k_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovw2m_k_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Movntdqa_xmm_m128, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vmovntdqa_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmovntdqa_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vmovntdqa_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntdqa_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vmovntdqa_zmm_m512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmb2q_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmb2q_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmb2q_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.Packusdw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpackusdw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpackusdw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.VEX_Vmaskmovps_xmm_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaskmovps_ymm_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vmaskmovpd_xmm_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaskmovpd_ymm_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vmaskmovps_m128_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaskmovps_m256_ymm_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaskmovpd_m128_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmaskmovpd_m256_ymm_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pmovzxbw_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxbw_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxbw_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovwb_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovwb_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovwb_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmovzxbd_xmm_xmmm32, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxbd_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxbd_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdb_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdb_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdb_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovzxbq_xmm_xmmm16, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxbq_xmm_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxbq_ymm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqb_xmmm16_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqb_xmmm32_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqb_xmmm64_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovzxwd_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxwd_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxwd_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdw_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdw_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovdw_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovzxwq_xmm_xmmm32, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxwq_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxwq_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqw_xmmm32_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqw_xmmm64_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqw_xmmm128_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmovzxdq_xmm_xmmm64, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmovzxdq_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmovzxdq_ymm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqd_xmmm64_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqd_xmmm128_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovqd_ymmm256_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pcmpgtq_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpcmpgtq_k_k1_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtq_k_k1_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpgtq_k_k1_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pminsb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpminsb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminsb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2d_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2d_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2d_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2q_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2q_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovm2q_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Pminsd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpminsd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminsd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovd2m_k_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovd2m_k_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovd2m_k_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovq2m_k_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovq2m_k_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmovq2m_k_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pminuw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpminuw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminuw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmw2d_xmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmw2d_ymm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastmw2d_zmm_k | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.Pminud_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpminud_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpminud_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmaxsb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxsb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxsb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmaxsd_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxsd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxsd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmaxuw_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxuw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxuw_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pmaxud_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmaxud_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmaxud_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pmulld_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpmulld_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpmulld_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Phminposuw_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vphminposuw_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.VEX_Vpsrlvd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsrlvd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsrlvq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsrlvq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpsravd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsravd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpsllvd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsllvd_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsllvq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpsllvq_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4VNNIW | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VNNI | Encoding_EVEX,
			(uint)Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4VNNIW | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntb_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntb_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntb_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntw_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntw_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntw_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VPOPCNTDQ | Encoding_EVEX,
			(uint)Code.VEX_Vpbroadcastd_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpbroadcastd_ymm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpbroadcastq_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpbroadcastq_ymm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vbroadcasti128_ymm_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandb_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandb_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandb_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandw_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandw_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandw_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressb_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressb_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressb_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressw_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressw_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressw_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64 | ProtectedMode | CodeInfo_KP1, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512F_and_AVX512_VP2INTERSECT | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BF16 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpbroadcastb_xmm_xmmm8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpbroadcastb_ymm_xmmm8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.VEX_Vpbroadcastw_xmm_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpbroadcastw_ymm_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastb_xmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastb_ymm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastb_zmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastw_xmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastw_ymm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastw_zmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastd_xmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastd_ymm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastd_zmm_k1z_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_xmm_k1z_r64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_ymm_k1z_r64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpbroadcastq_zmm_k1z_r64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Invept_r32_m128 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_INVEPT | Encoding_Legacy,
			(uint)Code.Invept_r64_m128 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_INVEPT | Encoding_Legacy,
			(uint)Code.Invvpid_r32_m128 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_INVVPID | Encoding_Legacy,
			(uint)Code.Invvpid_r64_m128 | Privileged | ProtectedMode | RflagsInfo_W_cz_C_aops, Op0_Read | Op1_Read | CPUID_INVVPID | Encoding_Legacy,
			(uint)Code.Invpcid_r32_m128 | Privileged, Op0_Read | Op1_Read | CPUID_INVPCID | Encoding_Legacy,
			(uint)Code.Invpcid_r64_m128 | Privileged, Op0_Read | Op1_Read | CPUID_INVPCID | Encoding_Legacy,
			(uint)Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandps_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandps_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandps_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandpd_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandpd_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vexpandpd_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandd_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandd_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandd_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandq_xmm_k1z_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandq_ymm_k1z_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpexpandq_zmm_k1z_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompressps_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompressps_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompressps_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompresspd_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompresspd_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcompresspd_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressd_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressd_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressd_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressq_xmmm128_k1z_xmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressq_ymmm256_k1z_ymm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcompressq_zmmm512_k1z_zmm | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpmaskmovd_xmm_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovd_ymm_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovq_xmm_xmm_m128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovq_ymm_ymm_m256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512_VBMI | Encoding_EVEX,
			(uint)Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.VEX_Vpmaskmovd_m128_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovd_m256_ymm_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovq_m128_xmm_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpmaskmovq_m256_ymm_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpshufbitqmb_k_k1_xmm_xmmm128 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufbitqmb_k_k1_ymm_ymmm256 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.EVEX_Vpshufbitqmb_k_k1_zmm_zmmm512 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512_BITALG | Encoding_EVEX,
			(uint)Code.VEX_Vpgatherdd_xmm_vm32x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherdd_ymm_vm32y_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherdq_xmm_vm32x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherdq_ymm_vm32x_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpgatherdd_xmm_k1_vm32x | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherdd_ymm_k1_vm32y | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherdd_zmm_k1_vm32z | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherdq_xmm_k1_vm32x | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherdq_ymm_k1_vm32x | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherdq_zmm_k1_vm32y | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpgatherqd_xmm_vm64x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherqd_xmm_vm64y_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherqq_xmm_vm64x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpgatherqq_ymm_vm64y_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpgatherqd_xmm_k1_vm64x | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherqd_xmm_k1_vm64y | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherqd_ymm_k1_vm64z | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherqq_xmm_k1_vm64x | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherqq_ymm_k1_vm64y | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpgatherqq_zmm_k1_vm64z | ProtectedMode, Op0_Write | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vgatherdps_xmm_vm32x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherdps_ymm_vm32y_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherdpd_xmm_vm32x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherdpd_ymm_vm32x_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vgatherdps_xmm_k1_vm32x | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherdps_ymm_k1_vm32y | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherdps_zmm_k1_vm32z | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherdpd_xmm_k1_vm32x | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherdpd_ymm_k1_vm32x | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherdpd_zmm_k1_vm32y | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vgatherqps_xmm_vm64x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherqps_xmm_vm64y_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherqpd_xmm_vm64x_xmm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vgatherqpd_ymm_vm64y_ymm | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_ReadWrite | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vgatherqps_xmm_k1_vm64x | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherqps_xmm_k1_vm64y | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherqps_ymm_k1_vm64z | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherqpd_xmm_k1_vm64x | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherqpd_ymm_k1_vm64y | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherqpd_zmm_k1_vm64z | ProtectedMode, Op0_ReadWrite | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4FMAPS | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4FMAPS | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdd_vm32x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdd_vm32y_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdd_vm32z_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdq_vm32x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdq_vm32x_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterdq_vm32y_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqd_vm64x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqd_vm64y_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqd_vm64z_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqq_vm64x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqq_vm64y_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpscatterqq_vm64z_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdps_vm32x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdps_vm32y_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdps_vm32z_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdpd_vm32x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdpd_vm32x_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterdpd_vm32y_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqps_vm64x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqps_vm64y_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqps_vm64z_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqpd_vm64x_k1_xmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqpd_vm64y_k1_ymm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterqpd_vm64z_k1_zmm | ProtectedMode, Op0_WriteForce | Op1_Read | OpMaskRegReadWrite | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4FMAPS | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128 | ProtectedMode, Op0_ReadWrite | Op1_ReadP3 | Op2_Read | CPUID_AVX512_4FMAPS | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512_IFMA | Encoding_EVEX,
			(uint)Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_FMA | Encoding_VEX,
			(uint)Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512VL_and_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512CD | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf0dps_vm32z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf0dpd_vm32y_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf1dps_vm32z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf1dpd_vm32y_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf0dps_vm32z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf0dpd_vm32y_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf1dps_vm32z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf1dpd_vm32y_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf0qps_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf0qpd_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf1qps_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vgatherpf1qpd_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf0qps_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf0qpd_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf1qps_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.EVEX_Vscatterpf1qpd_vm64z_k1 | ProtectedMode, Op0_NoMemAccess | CPUID_AVX512PF | Encoding_EVEX,
			(uint)Code.Sha1nexte_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.Sha1msg1_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.Sha1msg2_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.Sha256rnds2_xmm_xmmm128 | CodeInfo_R_XMM0, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.Sha256msg1_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae | ProtectedMode, Op0_Write | Op1_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.Sha256msg2_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512ER | Encoding_EVEX,
			(uint)Code.Gf2p8mulb_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_GFNI | Encoding_Legacy,
			(uint)Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F_and_GFNI | Encoding_EVEX,
			(uint)Code.Aesimc_xmm_xmmm128, Op0_Write | Op1_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaesimc_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.Aesenc_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaesenc_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaesenc_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_VAES | Encoding_VEX,
			(uint)Code.EVEX_Vaesenc_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesenc_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesenc_zmm_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F_and_VAES | Encoding_EVEX,
			(uint)Code.Aesenclast_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaesenclast_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaesenclast_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_VAES | Encoding_VEX,
			(uint)Code.EVEX_Vaesenclast_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesenclast_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesenclast_zmm_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F_and_VAES | Encoding_EVEX,
			(uint)Code.Aesdec_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaesdec_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaesdec_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_VAES | Encoding_VEX,
			(uint)Code.EVEX_Vaesdec_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesdec_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesdec_zmm_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F_and_VAES | Encoding_EVEX,
			(uint)Code.Aesdeclast_xmm_xmmm128, Op0_ReadWrite | Op1_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaesdeclast_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Vaesdeclast_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_VAES | Encoding_VEX,
			(uint)Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_VAES | Encoding_EVEX,
			(uint)Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F_and_VAES | Encoding_EVEX,
			(uint)Code.Movbe_r16_m16, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Movbe_r32_m32, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Movbe_r64_m64, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Crc32_r32_rm8, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Crc32_r64_rm8, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Movbe_m16_r16, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Movbe_m32_r32, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Movbe_m64_r64, Op0_Write | Op1_Read | CPUID_MOVBE | Encoding_Legacy,
			(uint)Code.Crc32_r32_rm16, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Crc32_r32_rm32, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Crc32_r64_rm64, Op0_ReadWrite | Op1_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Andn_r32_r32_rm32 | ProtectedMode | RflagsInfo_W_sz_C_co_U_ap, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Andn_r64_r64_rm64 | ProtectedMode | RflagsInfo_W_sz_C_co_U_ap, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsr_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsr_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsmsk_r32_rm32 | ProtectedMode | RflagsInfo_W_cs_C_oz_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsmsk_r64_rm64 | ProtectedMode | RflagsInfo_W_cs_C_oz_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsi_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Blsi_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Bzhi_r32_rm32_r32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Bzhi_r64_rm64_r64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.Wrussd_m32_r32 | ProtectedMode | Privileged, Op0_Write | Op1_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Wrussq_m64_r64 | ProtectedMode | Privileged, Op0_Write | Op1_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.VEX_Pext_r32_r32_rm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Pext_r64_r64_rm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Pdep_r32_r32_rm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Pdep_r64_r64_rm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.Wrssd_m32_r32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Wrssq_m64_r64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_CET_SS | Encoding_Legacy,
			(uint)Code.Adcx_r32_rm32 | RflagsInfo_R_c_W_c, Op0_ReadWrite | Op1_Read | CPUID_ADX | Encoding_Legacy,
			(uint)Code.Adcx_r64_rm64 | RflagsInfo_R_c_W_c, Op0_ReadWrite | Op1_Read | CPUID_ADX | Encoding_Legacy,
			(uint)Code.Adox_r32_rm32 | RflagsInfo_R_o_W_o, Op0_ReadWrite | Op1_Read | CPUID_ADX | Encoding_Legacy,
			(uint)Code.Adox_r64_rm64 | RflagsInfo_R_o_W_o, Op0_ReadWrite | Op1_Read | CPUID_ADX | Encoding_Legacy,
			(uint)Code.VEX_Mulx_r32_r32_rm32 | ProtectedMode | CodeInfo_Mulx, Op0_Write | Op1_Write | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Mulx_r64_r64_rm64 | ProtectedMode | CodeInfo_Mulx, Op0_Write | Op1_Write | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Bextr_r32_rm32_r32 | ProtectedMode | RflagsInfo_W_z_C_co_U_aps, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Bextr_r64_rm64_r64 | ProtectedMode | RflagsInfo_W_z_C_co_U_aps, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI1 | Encoding_VEX,
			(uint)Code.VEX_Shlx_r32_rm32_r32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Shlx_r64_rm64_r64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Sarx_r32_rm32_r32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Sarx_r64_rm64_r64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Shrx_r32_rm32_r32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Shrx_r64_rm64_r64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.Movdir64b_r16_m512 | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_MOVDIR64B | Encoding_Legacy,
			(uint)Code.Movdir64b_r32_m512 | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_MOVDIR64B | Encoding_Legacy,
			(uint)Code.Movdir64b_r64_m512 | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_MOVDIR64B | Encoding_Legacy,
			(uint)Code.Enqcmds_r16_m512 | Privileged | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Enqcmds_r32_m512 | Privileged | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Enqcmds_r64_m512 | Privileged | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Enqcmd_r16_m512 | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Enqcmd_r32_m512 | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Enqcmd_r64_m512 | RflagsInfo_W_z_C_acops | CodeInfo_Movdir64b, Op0_Read | Op1_Read | CPUID_ENQCMD | Encoding_Legacy,
			(uint)Code.Movdiri_m32_r32, Op0_Write | Op1_Read | CPUID_MOVDIRI | Encoding_Legacy,
			(uint)Code.Movdiri_m64_r64, Op0_Write | Op1_Read | CPUID_MOVDIRI | Encoding_Legacy,
			(uint)Code.VEX_Vpermq_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermpd_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermilps_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpermilps_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vpermilpd_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpermilpd_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Roundps_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vroundps_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vroundps_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Roundpd_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vroundpd_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vroundpd_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Roundss_xmm_xmmm32_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Roundsd_xmm_xmmm64_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Blendps_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Blendpd_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pblendw_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.Palignr_mm_mmm64_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.Palignr_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSSE3 | Encoding_Legacy,
			(uint)Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pextrb_r32m8_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Pextrb_r64m8_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpextrb_r32m8_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpextrb_r64m8_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpextrb_r32m8_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpextrb_r64m8_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pextrw_r32m16_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Pextrw_r64m16_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpextrw_r32m16_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpextrw_r64m16_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpextrw_r32m16_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpextrw_r64m16_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Pextrd_rm32_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Pextrq_rm64_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpextrd_rm32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpextrq_rm64_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpextrd_rm32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpextrq_rm64_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.Extractps_rm32_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Extractps_r64m32_xmm_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vextractps_rm32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vextractps_r64m32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vextractps_rm32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextractps_r64m32_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.VEX_Vextractf128_xmmm128_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_F16C | Encoding_VEX,
			(uint)Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_F16C | Encoding_VEX,
			(uint)Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpud_k_k1_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpud_k_k1_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpud_k_k1_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuq_k_k1_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuq_k_k1_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuq_k_k1_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpd_k_k1_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpd_k_k1_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpd_k_k1_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpq_k_k1_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpq_k_k1_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpq_k_k1_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pinsrb_xmm_r32m8_imm8 | CodeInfo_Read_Reg8_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Pinsrb_xmm_r64m8_imm8 | CodeInfo_Read_Reg8_Op1, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8 | ProtectedMode | CodeInfo_Read_Reg8_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8 | ProtectedMode | CodeInfo_Read_Reg8_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8 | ProtectedMode | CodeInfo_Read_Reg8_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8 | ProtectedMode | CodeInfo_Read_Reg8_Op2, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Insertps_xmm_xmmm32_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pinsrd_xmm_rm32_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.Pinsrq_xmm_rm64_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.VEX_Kshiftrb_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kshiftrw_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kshiftrd_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kshiftrq_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kshiftlb_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_VEX,
			(uint)Code.VEX_Kshiftlw_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_VEX,
			(uint)Code.VEX_Kshiftld_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Kshiftlq_k_k_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512BW | Encoding_VEX,
			(uint)Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.VEX_Vextracti128_xmmm128_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpub_k_k1_xmm_xmmm128_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpub_k_k1_ymm_ymmm256_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpub_k_k1_zmm_zmmm512_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuw_k_k1_xmm_xmmm128_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuw_k_k1_ymm_ymmm256_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpuw_k_k1_zmm_zmmm512_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpb_k_k1_xmm_xmmm128_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpb_k_k1_ymm_ymmm256_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpb_k_k1_zmm_zmmm512_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpw_k_k1_xmm_xmmm128_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpw_k_k1_ymm_ymmm256_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vpcmpw_k_k1_zmm_zmmm512_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.Dpps_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Dppd_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Mpsadbw_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SSE4_1 | Encoding_Legacy,
			(uint)Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512BW | Encoding_EVEX,
			(uint)Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.Pclmulqdq_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_PCLMULQDQ | Encoding_Legacy,
			(uint)Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_PCLMULQDQ_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_VPCLMULQDQ | Encoding_VEX,
			(uint)Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_VPCLMULQDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_VPCLMULQDQ | Encoding_EVEX,
			(uint)Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F_and_VPCLMULQDQ | Encoding_EVEX,
			(uint)Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | Op4_Read | CPUID_XOP | Encoding_VEX,
			(uint)Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX2 | Encoding_VEX,
			(uint)Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_ReadWrite | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F | Encoding_EVEX,
			(uint)Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.Pcmpestrm_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Pcmpestrm64_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pcmpestri_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.Pcmpestri64_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpestri_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pcmpistrm_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.Pcmpistri_xmm_xmmm128_imm8 | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_SSE4_2 | Encoding_Legacy,
			(uint)Code.VEX_Vpcmpistri_xmm_xmmm128_imm8 | ProtectedMode | CodeInfo_PcmpXstrY | RflagsInfo_W_cosz_C_ap, Op0_Read | Op1_Read | Op2_Read | CPUID_AVX | Encoding_VEX,
			(uint)Code.EVEX_Vfpclassps_k_k1_xmmm128b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclassps_k_k1_ymmm256b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclassps_k_k1_zmmm512b32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclasspd_k_k1_xmmm128b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclasspd_k_k1_ymmm256b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512VL_and_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclasspd_k_k1_zmmm512b64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclassss_k_k1_xmmm32_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.EVEX_Vfpclasssd_k_k1_xmmm64_imm8 | ProtectedMode, Op0_WriteForce | Op1_Read | Op2_Read | CPUID_AVX512DQ | Encoding_EVEX,
			(uint)Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512_VBMI2 | Encoding_EVEX,
			(uint)Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_FMA4 | Encoding_VEX,
			(uint)Code.Sha1rnds4_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_SHA | Encoding_Legacy,
			(uint)Code.Gf2p8affineqb_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_GFNI | Encoding_Legacy,
			(uint)Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F_and_GFNI | Encoding_EVEX,
			(uint)Code.Gf2p8affineinvqb_xmm_xmmm128_imm8, Op0_ReadWrite | Op1_Read | Op2_Read | CPUID_GFNI | Encoding_Legacy,
			(uint)Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX_and_GFNI | Encoding_VEX,
			(uint)Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512VL_and_GFNI | Encoding_EVEX,
			(uint)Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_AVX512F_and_GFNI | Encoding_EVEX,
			(uint)Code.Aeskeygenassist_xmm_xmmm128_imm8, Op0_Write | Op1_Read | Op2_Read | CPUID_AES | Encoding_Legacy,
			(uint)Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_AES_and_AVX | Encoding_VEX,
			(uint)Code.VEX_Rorx_r32_rm32_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.VEX_Rorx_r64_rm64_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_BMI2 | Encoding_VEX,
			(uint)Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotb_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotw_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotd_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotq_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | Op3_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Blcfill_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcfill_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blsfill_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blsfill_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcs_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcs_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Tzmsk_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Tzmsk_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcic_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcic_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blsic_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blsic_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_T1mskc_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_T1mskc_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcmsk_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blcmsk_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blci_r32_rm32 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Blci_r64_rm64 | ProtectedMode | RflagsInfo_W_csz_C_o_U_ap, Op0_Write | Op1_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Llwpcb_r32 | ProtectedMode | CodeInfo_Llwpcb, Op0_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Llwpcb_r64 | ProtectedMode | CodeInfo_Llwpcb, Op0_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Slwpcb_r32 | ProtectedMode, Op0_Write | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Slwpcb_r64 | ProtectedMode, Op0_Write | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Vfrczps_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vfrczps_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vfrczpd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vfrczpd_ymm_ymmm256 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vfrczss_xmm_xmmm32 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vfrczsd_xmm_xmmm64 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotb_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotw_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotd_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotd_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotq_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vprotq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlb_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlb_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlw_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshld_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshld_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlq_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshlq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshab_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshab_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshaw_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshaw_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshad_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshad_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshaq_xmm_xmmm128_xmm | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vpshaq_xmm_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | Op2_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddbw_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddbd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddbq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddwd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddwq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphadddq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddubw_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddubd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddubq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphadduwd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphadduwq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphaddudq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphsubbw_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphsubwd_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Vphsubdq_xmm_xmmm128 | ProtectedMode, Op0_Write | Op1_Read | CPUID_XOP | Encoding_XOP,
			(uint)Code.XOP_Bextr_r32_rm32_imm32 | ProtectedMode | RflagsInfo_W_z_C_co_U_aps, Op0_Write | Op1_Read | Op2_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Bextr_r64_rm64_imm32 | ProtectedMode | RflagsInfo_W_z_C_co_U_aps, Op0_Write | Op1_Read | Op2_Read | CPUID_TBM | Encoding_XOP,
			(uint)Code.XOP_Lwpins_r32_rm32_imm32 | ProtectedMode | RflagsInfo_W_c, Op0_Read | Op1_Read | Op2_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Lwpins_r64_rm32_imm32 | ProtectedMode | RflagsInfo_W_c, Op0_Read | Op1_Read | Op2_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Lwpval_r32_rm32_imm32 | ProtectedMode, Op0_Read | Op1_Read | Op2_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.XOP_Lwpval_r64_rm32_imm32 | ProtectedMode, Op0_Read | Op1_Read | Op2_Read | CPUID_LWP | Encoding_XOP,
			(uint)Code.D3NOW_Pi2fw_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOWEXT | Encoding_D3NOW,
			(uint)Code.D3NOW_Pi2fd_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pf2iw_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOWEXT | Encoding_D3NOW,
			(uint)Code.D3NOW_Pf2id_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrcpv_mm_mmm64, Op0_Write | Op1_Read | CPUID_GEODE | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrsqrtv_mm_mmm64, Op0_Write | Op1_Read | CPUID_GEODE | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfnacc_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOWEXT | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfpnacc_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOWEXT | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfcmpge_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfmin_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrcp_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrsqrt_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfsub_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfadd_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfcmpgt_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfmax_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrcpit1_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrsqit1_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfsubr_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfacc_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfcmpeq_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfmul_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pfrcpit2_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pmulhrw_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.D3NOW_Pswapd_mm_mmm64, Op0_Write | Op1_Read | CPUID_D3NOWEXT | Encoding_D3NOW,
			(uint)Code.D3NOW_Pavgusb_mm_mmm64, Op0_ReadWrite | Op1_Read | CPUID_D3NOW | Encoding_D3NOW,
			(uint)Code.DeclareByte, FlowControl_Exception | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.DeclareWord, FlowControl_Exception | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.DeclareDword, FlowControl_Exception | CPUID_INTEL8086 | Encoding_Legacy,
			(uint)Code.DeclareQword, FlowControl_Exception | CPUID_INTEL8086 | Encoding_Legacy,
		};
	}
}
#endif
